@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -63,57 +63,6 @@
 #include <signal.h>
 #include <errno.h>
 
-#include "../src/gkrellm-sysdeps.h"
-
-#if defined(__linux__)
-#if defined(__GLIBC__) && ((__GLIBC__>2)||(__GLIBC__==2 && __GLIBC_MINOR__>=1))
-#define HAVE_GETADDRINFO	1
-#endif
-#endif
-
-#if defined(__FreeBSD__)
-#include <sys/param.h>
-#if __FreeBSD_version >= 400000
-#define HAVE_GETADDRINFO	1
-#endif
-#endif
-
-#if defined(__OpenBSD__)
-#define HAVE_GETADDRINFO	1
-#endif
-
-#if defined(__NetBSD__)
-#define HAVE_GETADDRINFO	1
-#  if __NetBSD_Version__ <= 105010000
-#    define sa_family_t unsigned char
-#  endif
-#endif
-
-#if defined(__solaris__)
-# include <netconfig.h>
-# if defined(NC_INET6)
-#  define HAVE_GETADDRINFO	1
-# endif
-#endif
-
-#if defined(__APPLE__)
-# ifndef socklen_t
-#  define socklen_t int
-# endif
-#define HAVE_GETADDRINFO   1
-#endif
-
-#ifndef	NI_WITHSCOPEID
-#define	NI_WITHSCOPEID	0
-#endif
-
-#if !defined(__FreeBSD__) && !defined(__linux__) && !defined(__NetBSD__) \
-    && !defined(__OpenBSD__) && !defined(__solaris__) && !defined(WIN32) \
-    && !defined(__APPLE__)
-#define  USE_LIBGTOP
-#endif
-
-extern int	errno;
 
 #if !defined(PACKAGE_D)
 #define	PACKAGE_D	"gkrellmd"
@@ -143,29 +92,23 @@ extern int	errno;
 /* -------------------------------------------------------------------
 */
 #define GKRELLMD_VERSION_MAJOR   2
-#define GKRELLMD_VERSION_MINOR   1
-#define GKRELLMD_VERSION_REV     24
+#define GKRELLMD_VERSION_MINOR   2
+#define GKRELLMD_VERSION_REV     1
 #define GKRELLMD_EXTRAVERSION    ""
 
 #define GKRELLMD_CONFIG     "gkrellmd.conf"
 
+#define GKRELLMD_PLUGINS_DIR		".gkrellm2/plugins-gkrellmd"
+#define GKRELLMD_LOCAL_PLUGINS_DIR	"/usr/local/lib/gkrellm2/plugins-gkrellmd"
+#if !defined(GKRELLMD_SYSTEM_PLUGINS_DIR)
+#define GKRELLMD_SYSTEM_PLUGINS_DIR	"/usr/lib/gkrellm2/plugins-gkrellmd"
+#endif
+
 #if !defined(WIN32)
 #define	GKRELLMD_SYS_ETC	"/etc"
 #define GKRELLMD_LOCAL_ETC	"/usr/local/etc"
 #endif
 
-#define DEBUG_SYSDEP	0x1
-#define DEBUG_MAIL		0x10
-#define DEBUG_NET		0x20
-#define DEBUG_TIMER		0x40
-#define DEBUG_SENSORS	0x80
-#define DEBUG_REMOTE	0x100
-#define DEBUG_GUI		0x1000
-
-#define SENSOR_TEMPERATURE  0
-#define SENSOR_FAN          1
-#define SENSOR_VOLTAGE      2
-
 
 typedef struct
 	{
@@ -194,55 +137,66 @@ typedef struct
 	}
 	GkrellmdTicks;
 
+extern GkrellmdTicks			GK;
+
+
 typedef struct
+	{
+	gboolean		need_serve;
+	const gchar		*serve_name;
+	gboolean		serve_name_sent;
+	GString			*serve_gstring;
+	GkrellmdClient	*client;
+
+	gboolean		is_plugin;
+	void			*handle;
+	gchar			*path;
+	}
+	GkrellmdMonitorPrivate;
+
+
+typedef struct _GkrellmdMonitor
 	{
 	gchar		*name;
-	void		(*update_monitor)(gboolean force);
-	gchar		*(*serve_data)(GkrellmdClient *);
-	void		(*serve_setup)(GkrellmdClient *);
+	void		(*update_monitor)(struct _GkrellmdMonitor *mon,
+							gboolean first_update);
+	void		(*serve_data)(struct _GkrellmdMonitor *mon,
+							gboolean first_serve);
+	void		(*serve_setup)(struct _GkrellmdMonitor *mon);
+
+	GkrellmdMonitorPrivate
+				*privat;
 	}
 	GkrellmdMonitor;
 
-struct GkrellmdConfig
-	{
-	gint		update_HZ;
-	gint		debug_level;
-	gint		*server_fd;
-	gint		max_clients;
-	gint		server_port;
-	gint		verbose;
-	time_t		start_time;
-	gint		io_timeout;
-	gint		reconnect_timeout;
-
-	gint		fs_interval,
-				nfs_interval,
-				inet_interval;
-
-	gchar		*pidfile;
-	};
 
 
+  /* gkrellmd serve data functions used by builtins and plugins.
+  */
+void		gkrellmd_plugin_serve_setup(GkrellmdMonitor *mon,
+						gchar *name, gchar *line);
+void		gkrellmd_need_serve(GkrellmdMonitor *mon);
+void		gkrellmd_set_serve_name(GkrellmdMonitor *mon, const gchar *name);
+void		gkrellmd_serve_data(GkrellmdMonitor *mon, gchar *line);
+void		gkrellmd_add_serveflag_done(gboolean *);
+gboolean	gkrellmd_check_client_version(GkrellmdMonitor *mon,
+						gint major, gint minor, gint rev);
 
-extern struct GkrellmdConfig	_GK;
-extern GkrellmdTicks			GK;
+  /* Small set of useful functions.
+  */
+void		gkrellmd_free_glist_and_data(GList **list_head);
+gchar		*gkrellmd_dup_token(gchar **string, gchar *delimeters);
+gboolean	gkrellmd_dup_string(gchar **dst, gchar *src);
 
-extern GList	*gkrellmd_client_list;
+  /* Plugins should use above data serve functions instead of this.
+  */
+gint		gkrellmd_send_to_client(GkrellmdClient *client, gchar *buf);
 
-void			gkrellmd_client_read(gint client_fd, gint nbytes);
-
-GkrellmdMonitor *gkrellmd_init_mail_monitor(void);
-void			gkrellmd_load_monitors(void);
-gint			gkrellmd_update_monitors(void);
-void			gkrellmd_serve_setup(GkrellmdClient *client);
-gint			gkrellmd_send_to_client(GkrellmdClient *client, gchar *buf);
-void			gkrellmd_free_glist_and_data(GList **);
-void			gkrellmd_add_mailbox(gchar *);
-void			gkrellmd_add_serveflag_done(gboolean *);
 
 
 
 #if !GLIB_CHECK_VERSION(2,0,0)
+
 /* glib2 compatibility functions
 */
 #define G_FILE_TEST_EXISTS		1
@@ -262,6 +216,7 @@ gchar		*g_dir_read_name(GDir *dir);
 void		g_dir_close(GDir *dir);
 gboolean	g_file_test(gchar *filename, gint test);
 gchar		*g_build_filename(gchar *first, ...);
+gchar		*g_path_get_basename(gchar *file_name);
 
 #endif
 
