@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -20,6 +20,7 @@
 */
 
 #include "gkrellmd.h"
+#include "gkrellmd-private.h"
 
 
 GList			*gkrellmd_monitor_list;
@@ -35,8 +36,11 @@ gkrellm_get_timer_ticks(void)
 	}
 
 gboolean
-check_client_version(GkrellmdClient *client, gint major, gint minor, gint rev)
+gkrellmd_check_client_version(GkrellmdMonitor *mon,
+				gint major, gint minor, gint rev)
 	{
+	GkrellmdClient	*client = mon->privat->client;
+
 	if (   client->major_version > major
 		|| (client->major_version == major && client->minor_version > minor)
 	    || (   client->major_version == major && client->minor_version == minor
@@ -53,6 +57,42 @@ gkrellmd_add_serveflag_done(gboolean *flag)
 	serveflag_done_list = g_list_append(serveflag_done_list, flag);
 	}
 
+
+void
+gkrellmd_set_serve_name(GkrellmdMonitor *mon, const gchar *tag)
+	{
+	GkrellmdMonitorPrivate	*mp = mon->privat;
+
+	mp->serve_name = tag;
+	mp->serve_name_sent = FALSE;
+	}
+
+void
+gkrellmd_serve_data(GkrellmdMonitor *mon, gchar *line)
+	{
+	GkrellmdMonitorPrivate	*mp = mon->privat;
+	gchar					buf[128];
+
+	if (!line || !*line)
+		return;
+	if (!mp->serve_name_sent)
+		{
+		if (mp->serve_name)
+			{
+			snprintf(buf, sizeof(buf), "<%s>\n", mp->serve_name);
+			mp->serve_gstring = g_string_append(mp->serve_gstring, buf);
+			mp->serve_name_sent = TRUE;
+			}
+		else
+			{
+			printf("gkrellmd: %s forgot to gkrellmd_set_serve_name()\n",
+					mon->name);
+			return;
+			}
+		}
+	mp->serve_gstring = g_string_append(mp->serve_gstring, line);
+	}
+
 /* ======================================================= */
 typedef struct
 	{
@@ -132,36 +172,35 @@ gkrellm_cpu_assign_data(gint n, gulong user, gulong nice,
 	}
 
 static void
-update_cpu(gboolean force)
+update_cpu(GkrellmdMonitor *mon, gboolean first_update)
 	{
 	gkrellm_sys_cpu_read_data();
+	gkrellmd_need_serve(mon);
 	}
 
-static gchar *
-serve_cpu_data(GkrellmdClient *client)
+static void
+serve_cpu_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
 	CpuData	*cpu;
 	GList	*list;
-	gchar	*s, *d, buf[128];
+	gchar	buf[128];
 
-	s = g_strdup("<cpu>\n");
+	gkrellmd_set_serve_name(mon, "cpu");
 	for (list = cpu_list; list; list = list->next)
 		{
 		cpu = (CpuData *) list->data;
 		sprintf(buf, "%d %lu %lu %lu %lu\n", cpu->instance,
 				cpu->user, cpu->nice, cpu->sys, cpu->idle);
-		d = g_strconcat(s, buf, NULL);
-		g_free(s);
-		s = d;
+		gkrellmd_serve_data(mon, buf);
 		}
-	return s;
 	}
 
 static void
-serve_cpu_setup(GkrellmdClient *client)
+serve_cpu_setup(GkrellmdMonitor *mon)
 	{
-	GList	*list;
-	gchar	buf[64];
+	GkrellmdClient	*client = mon->privat->client;
+	GList			*list;
+	gchar			buf[64];
 
 	gkrellmd_send_to_client(client, "<cpu_setup>\n");
 	for (list = instance_list; list; list = list->next)
@@ -233,25 +272,27 @@ gkrellm_proc_assign_users(gint n_users)
 	}
 
 static void
-update_proc(gboolean force)
+update_proc(GkrellmdMonitor *mon, gboolean first_update)
 	{
+	proc.changed = FALSE;
 	gkrellm_sys_proc_read_data();
-	if (force || GK.five_second_tick)
+	if (first_update || GK.five_second_tick)
 		gkrellm_sys_proc_read_users();
+
+	if (proc.changed)
+		gkrellmd_need_serve(mon);
 	}
 
-static gchar *
-serve_proc_data(GkrellmdClient *client)
+static void
+serve_proc_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
-	gchar	*s = NULL;
+	gchar	buf[128];
 
-	if (proc.changed || !client->served)
-		{
-		s = g_strdup_printf("<proc>\n%d %d %lu %.2f %d\n",
-				proc.n_processes, proc.n_running,
-				proc.n_forks, proc.fload, proc.n_users);
-		}
-	return s;
+	gkrellmd_set_serve_name(mon, "proc");
+	snprintf(buf, sizeof(buf), "%d %d %lu %.2f %d\n",
+			 proc.n_processes, proc.n_running,
+			 proc.n_forks, proc.fload, proc.n_users);
+	gkrellmd_serve_data(mon, buf);
 	}
 
 static GkrellmdMonitor proc_monitor =
@@ -477,53 +518,57 @@ gkrellm_disk_subdisk_assign_data_by_name(gchar *subdisk_name, gchar *disk_name,
 	}
 
 static void
-update_disk(gboolean force)
+update_disk(GkrellmdMonitor *mon, gboolean first_update)
 	{
+	GList		*list;
+	DiskData	*disk = NULL;
+
 	gkrellm_sys_disk_read_data();
+	for (list = disk_list; list; list = list->next)
+		{
+		disk = (DiskData * ) list->data;
+		if (disk->changed)
+			{
+			gkrellmd_need_serve(mon);
+			break;
+			}
+		}
 	}
 
 
-static gchar *
-serve_disk_data(GkrellmdClient *client)
+static void
+serve_disk_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
 	DiskData	*disk;
 	GList		*list;
-	gchar		*d, *s, *line;
+	gchar		buf[128];
 
-	s = g_strdup("");
+	gkrellmd_set_serve_name(mon, "disk");
 	for (list = disk_list; list; list = list->next)
 		{
 		disk = (DiskData *) list->data;
-		if (!disk->changed && client->served)
+		if (!disk->changed && !first_serve)
 			continue;
 		if (!disk->subdisk_parent)
-			line = g_strdup_printf("%s %llu %llu\n",
+			snprintf(buf, sizeof(buf), "%s %llu %llu\n",
 						disk->name, disk->rb, disk->wb);
-		else if (client->feature_subdisk)
-			line = g_strdup_printf("%s %s %llu %llu\n",
+		else if (mon->privat->client->feature_subdisk)
+			snprintf(buf, sizeof(buf), "%s %s %llu %llu\n",
 						disk->name, disk->subdisk_parent, disk->rb, disk->wb);
 		else
 			continue;
-		d = g_strconcat(s, line, NULL);
-		g_free(line);
-		g_free(s);
-		s = d;
+		gkrellmd_serve_data(mon, buf);
 		}
-	if (*s)
-		{
-		d = g_strconcat("<disk>\n", s, NULL);
-		g_free(s);
-		s = d;
-		}
-	return s;
 	}
 
 static void
-serve_disk_setup(GkrellmdClient *client)
+serve_disk_setup(GkrellmdMonitor *mon)
 	{
+	GkrellmdClient	*client = mon->privat->client;
+
 	if (units_are_blocks)
 		gkrellmd_send_to_client(client, "<disk_setup>\nunits_are_blocks\n");
-	if (check_client_version(client, 2,1,3))
+	if (gkrellmd_check_client_version(mon, 2,1,3))
 		client->feature_subdisk = TRUE;
 	}
 
@@ -608,32 +653,18 @@ gkrellm_inet_log_tcp_port_data(gpointer data)
 	inet_list = g_list_append(inet_list, in);
 	}
 
-void
-gkrellmd_free_glist_and_data(GList **list_head)
-	{
-	GList	*list;
-
-	if (*list_head == NULL)
-		return;
-	for (list = *list_head; list; list = list->next)
-		if (list->data)
-			g_free(list->data);
-	g_list_free(*list_head);
-	*list_head = NULL;
-	}
-
 static void
-update_inet(gboolean force)
+update_inet(GkrellmdMonitor *mon, gboolean first_update)
 	{
 	GList		*list;
 	InetData	*in;
 	static gint	check_tcp;
 
 	
-	if (!force && !GK.second_tick)
+	if (!first_update && !GK.second_tick)
 		return;
 
-	if (force || check_tcp == 0)
+	if (first_update || check_tcp == 0)
 		{
 		gkrellmd_free_glist_and_data(&inet_dead_list);
 		inet_new = FALSE;
@@ -659,45 +690,42 @@ update_inet(gboolean force)
 			else
 				list = list->next;
 			}
+		if (inet_new || inet_dead_list)
+			gkrellmd_need_serve(mon);
 		}
 	check_tcp = (check_tcp + 1) % _GK.inet_interval;
 	}
 
-static gchar *
-serve_inet_data(GkrellmdClient *client)
+static void
+serve_inet_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
 	InetData	*in;
 	ActiveTCP	*tcp;
 	GList		*list;
-	gchar		buf[128], *d, *s, *cp;
+	gchar		buf[128], *cp;
 #if defined(INET6) && defined(HAVE_GETADDRINFO)
 	struct sockaddr_in6	sin6;
 	char		addrbuf[NI_MAXHOST];
 #endif
 
-	if (client->served && (!GK.second_tick || (!inet_new && !inet_dead_list)))
-		return NULL;
-	s = g_strdup("");
-	if (inet_new || !client->served)
+	if (inet_new || first_serve)
 		{
+		gkrellmd_set_serve_name(mon, "inet");
 		for (list = inet_list; list; list = list->next)
 			{
 			in = (InetData *) list->data;
 			tcp = &in->tcp;
 			if (   tcp->family == AF_INET
-				&& (in->new_connection || !client->served)
+				&& (in->new_connection || first_serve)
 			   )
 				{
 				cp = inet_ntoa(tcp->remote_addr);
 				sprintf(buf, "+0 %x %s:%x\n",
 							tcp->local_port, cp, tcp->remote_port);
-				d = g_strconcat(s, buf, NULL);
-				g_free(s);
-				s = d;
 				}
 #if defined(INET6) && defined(HAVE_GETADDRINFO)
 			else if (tcp->family == AF_INET6
-				 && (in->new_connection || !client->served))
+				 && (in->new_connection || first_serve))
 				{
 				memset(&sin6, 0, sizeof(sin6));
 				memcpy(&sin6.sin6_addr, &tcp->remote_addr6,
@@ -716,15 +744,17 @@ serve_inet_data(GkrellmdClient *client)
 				snprintf(buf, sizeof(buf), "+6 %x [%s]:%x\n",
 					 tcp->local_port, addrbuf,
 					 tcp->remote_port);
-				d = g_strconcat(s, buf, NULL);
-				g_free(s);
-				s = d;
 				}
 #endif
+			else
+				continue;
+
+			gkrellmd_serve_data(mon, buf);
 			}
 		}
-	if (client->served)
+	if (!first_serve)
 		{
+		gkrellmd_set_serve_name(mon, "inet");
 		for (list = inet_dead_list; list; list = list->next)
 			{
 			in = (InetData *) list->data;
@@ -734,9 +764,6 @@ serve_inet_data(GkrellmdClient *client)
 				cp = inet_ntoa(tcp->remote_addr);
 				sprintf(buf, "-0 %x %s:%x\n",
 							tcp->local_port, cp, tcp->remote_port);
-				d = g_strconcat(s, buf, NULL);
-				g_free(s);
-				s = d;
 				}
 #if defined(INET6) && defined(HAVE_GETADDRINFO)
 			else if (tcp->family == AF_INET6)
@@ -758,25 +785,21 @@ serve_inet_data(GkrellmdClient *client)
 				snprintf(buf, sizeof(buf), "-6 %x [%s]:%x\n",
 					 tcp->local_port, addrbuf,
 					 tcp->remote_port);
-				d = g_strconcat(s, buf, NULL);
-				g_free(s);
-				s = d;
 				}
 #endif
+			else
+				continue;
+
+			gkrellmd_serve_data(mon, buf);
 			}
 		}
-	if (*s)
-		{
-		d = g_strconcat("<inet>\n", s, NULL);
-		g_free(s);
-		s = d;
-		}
-	return s;
 	}
 
 static void
-serve_inet_setup(GkrellmdClient *client)
+serve_inet_setup(GkrellmdMonitor *mon)
 	{
+	GkrellmdClient	*client = mon->privat->client;
+
 	if (inet_unsupported)
 		gkrellmd_send_to_client(client, "<inet_setup>\ninet_unsupported\n");
 	}
@@ -925,7 +948,7 @@ gkrellm_net_set_lock_directory(gchar *dir)
 	}
 
 static void
-update_net(gboolean force)
+update_net(GkrellmdMonitor *mon, gboolean first_update)
 	{
 	GList	*list;
 	NetData	*net;
@@ -957,70 +980,55 @@ update_net(gboolean force)
 				net->down_event = TRUE;
 			}
 		}
+	gkrellmd_need_serve(mon);	/* serve func checks for changed */
 	}
 
-static gchar *
-serve_net_data(GkrellmdClient *client)
+static void
+serve_net_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
 	NetData		*net;
 	GList		*list;
-	gchar		buf[128], *d, *s, *ss;
+	gchar		buf[128];
 	gboolean	fake_up_event;
 
-	s = g_strdup("");
+	gkrellmd_set_serve_name(mon, "net");
 	for (list = net_list; list; list = list->next)
 		{
 		net = (NetData *) list->data;
-		if (net->changed || !client->served)
+		if (net->changed || first_serve)
 			{
 			sprintf(buf, "%s %lu %lu\n", net->name, net->rx, net->tx);
-			d = g_strconcat(s, buf, NULL);
-			g_free(s);
-			s = d;
+			gkrellmd_serve_data(mon, buf);
 			}
 		}
-	if (*s)
-		{
-		d = g_strconcat("<net>\n", s, NULL);
-		g_free(s);
-		s = d;
-		}
 
-  /* Since the server transmits changes only, use the routed interface
-  |  to the client regardless if the sysdep code uses routed.
-  */
-	if (GK.second_tick || !client->served)
+	/* Since the server transmits changes only, use the routed interface
+	|  to the client regardless if the sysdep code uses routed.
+	*/
+	if (GK.second_tick || first_serve)
 		{
-		ss = g_strdup("");
+		gkrellmd_set_serve_name(mon, "net_routed");
 		for (list = net_list; list; list = list->next)
 			{
 			net = (NetData *) list->data;
-			fake_up_event = (!client->served && net->up);
+			fake_up_event = (first_serve && net->up);
 			if (net->up_event || net->down_event || fake_up_event)
 				{
 				sprintf(buf, "%s %d\n", net->name,
 						fake_up_event ? TRUE : net->up_event);
-				d = g_strconcat(ss, buf, NULL);
-				g_free(ss);
-				ss = d;
+				gkrellmd_serve_data(mon, buf);
 				}
-			if (client->last_client)
+			if (mon->privat->client->last_client)
 				net->up_event = net->down_event = FALSE;
 			}
-		if (*ss)
-			{
-			d = g_strconcat(s, "<net_routed>\n", ss, NULL);
-			g_free(s);
-			s = d;
-			}
-		g_free(ss);
 		}
-	return s;
 	}
 
 static void
-serve_net_setup(GkrellmdClient *client)
+serve_net_setup(GkrellmdMonitor *mon)
 	{
+	GkrellmdClient	*client = mon->privat->client;
+
 	/* The client <-> server link always uses routed mode, but the client
 	|  needs to know if server sysdep uses routed for config purposes.
 	*/
@@ -1106,38 +1114,40 @@ gkrellm_swap_assign_data(guint64 total, guint64 used,
 	}
 
 static void
-update_mem(gboolean force)
+update_mem(GkrellmdMonitor *mon, gboolean first_update)
 	{
+	mem.mem_changed = mem.swap_changed = FALSE;
+
 	gkrellm_sys_swap_read_data();
-	if (force || GK.five_second_tick)
+	if (first_update || GK.five_second_tick)
 		gkrellm_sys_mem_read_data();
+
+	if (mem.mem_changed || mem.swap_changed)
+		gkrellmd_need_serve(mon);
 	}
 
-static gchar *
-serve_mem_data(GkrellmdClient *client)
+static void
+serve_mem_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
-	gchar	*s, *d, *e;
+	gchar	buf[128];
 
-	if (mem.mem_changed || !client->served)
+	if (mem.mem_changed || first_serve)
 		{
-		s = g_strdup_printf("<mem>\n%llu %llu %llu %llu %llu %llu\n",
+		gkrellmd_set_serve_name(mon, "mem");
+		snprintf(buf, sizeof(buf), "%llu %llu %llu %llu %llu %llu\n",
 				mem.total, mem.used, mem.free,
 				mem.shared, mem.buffers, mem.cached);
+		gkrellmd_serve_data(mon, buf);
 		}
-	else
-		s = g_strdup("");		
 
-	if (mem.swap_changed || !client->served)
+	if (mem.swap_changed || first_serve)
 		{
-		e = g_strdup_printf("<swap>\n%llu %llu %lu %lu\n",
+		gkrellmd_set_serve_name(mon, "swap");
+		snprintf(buf, sizeof(buf), "%llu %llu %lu %lu\n",
 				mem.swap_total, mem.swap_used,
 				mem.swap_in, mem.swap_out);
-		d = g_strconcat(s, e, NULL);
-		g_free(e);
-		g_free(s);
-		s = d;
+		gkrellmd_serve_data(mon, buf);
 		}
-	return s;
 	}
 
 static GkrellmdMonitor mem_monitor =
@@ -1171,7 +1181,7 @@ typedef struct
 				*device,
 				*type,
 				*options;
-	gulong		blocks,
+	glong		blocks,
 				bavail,
 				bfree,
 				bsize;
@@ -1253,7 +1263,7 @@ gkrellm_fs_add_to_fstab_list(gchar *dir, gchar *dev, gchar *type, gchar *opt)
 
 void
 gkrellm_fs_assign_fsusage_data(gpointer pointer,
-			gulong blocks, gulong bavail, gulong bfree, gulong bsize)
+			glong blocks, glong bavail, glong bfree, glong bsize)
 	{
 	Mount	*m = (Mount *) pointer;
 
@@ -1352,12 +1362,16 @@ get_fsusage_thread(void *data)
 #endif
 
 static void
-update_fs(gboolean force)
+update_fs(GkrellmdMonitor *mon, gboolean first_update)
 	{
 	GList		*list;
 	Mount		*m;
 	static gint	check_tick;
 
+	if (fs_need_serve)		/* Asynchronous change in fsusage thread? */
+		gkrellmd_need_serve(mon);
+	fs_need_serve = FALSE;
+
 	if (GK.second_tick)
 		++check_tick;
 	fs_check = !(check_tick % _GK.fs_interval);
@@ -1367,7 +1381,7 @@ update_fs(gboolean force)
 	else
 		nfs_check = 0;
 
-	if (!force && (!GK.second_tick || (!fs_check && !nfs_check)))
+	if (!first_update && (!GK.second_tick || (!fs_check && !nfs_check)))
 		return;
 	refresh_mounts_list();
 	for (list = mounts_list; list; list = list->next)
@@ -1387,76 +1401,63 @@ update_fs(gboolean force)
 		}
 	if (gkrellm_sys_fs_fstab_modified())
 		refresh_fstab_list();
-	fs_need_serve = TRUE;
+
+	gkrellmd_need_serve(mon);
 	}
 
-static gchar *
-serve_fs_data(GkrellmdClient *client)
+static void
+serve_fs_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
 	Mount	*m;
 	GList	*list;
-	gchar	buf[128], *d, *s;
+	gchar	buf[128];
 
-	if (!fs_need_serve && client->served)
-		return NULL;
-
-	if (mounts_list_modified || !client->served)
+	if (mounts_list_modified || first_serve)
 		{
-		s = g_strconcat("<fs_mounts>\n", ".clear\n", NULL);
+		gkrellmd_set_serve_name(mon, "fs_mounts");
+		gkrellmd_serve_data(mon, ".clear\n");
 		for (list = mounts_list; list; list = list->next)
 			{
 			m = (Mount *) list->data;
-			snprintf(buf, sizeof(buf), "%s %s %s %lu %lu %lu %lu\n",
+			snprintf(buf, sizeof(buf), "%s %s %s %ld %ld %ld %ld\n",
 					m->directory, m->device, m->type,
 					m->blocks, m->bavail, m->bfree, m->bsize);
-			d = g_strconcat(s, buf, NULL);
-			g_free(s);
-			s = d;
+			gkrellmd_serve_data(mon, buf);
 			}
 		}
 	else
 		{
-		s = g_strdup("");
+		gkrellmd_set_serve_name(mon, "fs");
 		for (list = mounts_list; list; list = list->next)
 			{
 			m = (Mount *) list->data;
 			if (!m->changed)
 				continue;
-			snprintf(buf, sizeof(buf), "%s %s %lu %lu %lu %lu\n",
+			snprintf(buf, sizeof(buf), "%s %s %ld %ld %ld %ld\n",
 					m->directory, m->device,
 					m->blocks, m->bavail, m->bfree, m->bsize);
-			d = g_strconcat(s, buf, NULL);
-			g_free(s);
-			s = d;
-			}
-		if (*s)
-			{
-			d = g_strconcat("<fs>\n", s, NULL);
-			g_free(s);
-			s = d;
+			gkrellmd_serve_data(mon, buf);
 			}
 		}
-	if (fstab_list_modified || !client->served)
+	if (fstab_list_modified || first_serve)
 		{
-		d = g_strconcat(s, "<fs_fstab>\n", ".clear\n", NULL);
-		g_free(s);
-		s = d;
+		gkrellmd_set_serve_name(mon, "fs_fstab");
+		gkrellmd_serve_data(mon, ".clear\n");
 		for (list = fstab_list; list; list = list->next)
 			{
 			m = (Mount *) list->data;
 			snprintf(buf, sizeof(buf), "%s %s %s\n",
 					m->directory, m->device, m->type);
-			d = g_strconcat(s, buf, NULL);
-			g_free(s);
-			s = d;
+			gkrellmd_serve_data(mon, buf);
 			}
 		}
-	return s;
 	}
 
 static void
-serve_fs_setup(GkrellmdClient *client)
+serve_fs_setup(GkrellmdMonitor *mon)
 	{
+	GkrellmdClient	*client = mon->privat->client;
+
 	if (mounting_unsupported)
 		gkrellmd_send_to_client(client, "<fs_setup>\nmounting_unsupported\n");
 	}
@@ -1475,7 +1476,6 @@ init_fs_monitor(void)
 	{
 	if (!gkrellm_sys_fs_init())
 		return NULL;
-	serveflag_done_list = g_list_append(serveflag_done_list, &fs_need_serve);
 	serveflag_done_list =
 				g_list_append(serveflag_done_list, &fstab_list_modified);
 	serveflag_done_list =
@@ -1576,26 +1576,22 @@ gkrellm_battery_full_cap_fallback()
 	}
 
 static void
-update_battery(gboolean force)
+update_battery(GkrellmdMonitor *mon, gboolean first_update)
 	{
 	GList	*list;
+	Battery	*bat;
 
-	if (force || GK.five_second_tick)
+	if (!first_update && !GK.five_second_tick)
+		return;
+
+	for (list = battery_list; list; list = list->next)
 		{
-		for (list = battery_list; list; list = list->next)
-			((Battery *) list->data)->have_data = FALSE;
-		gkrellm_sys_battery_read_data();
+		bat = (Battery *) list->data;
+		bat->have_data = FALSE;
+		bat->changed = FALSE;
 		}
-	}
+	gkrellm_sys_battery_read_data();
 
-static gchar *
-serve_battery_data(GkrellmdClient *client)
-	{
-	Battery		*bat;
-	GList		*list;
-	gchar		buf[128], *d, *s;
-
-	s = g_strdup("");
 	for (list = battery_list; list; list = list->next)
 		{
 		bat = (Battery *) list->data;
@@ -1604,34 +1600,45 @@ serve_battery_data(GkrellmdClient *client)
 			bat->present = FALSE;
 			bat->changed = TRUE;
 			}
-		if (   (!bat->changed && client->served)
-			|| (!check_client_version(client, 2,1,9) && bat->id > 0)
-		   )
-			continue;
-		sprintf(buf, "%d %d %d %d %d %d\n",
-					bat->present, bat->on_line, bat->charging,
-					bat->percent, bat->time_left, bat->id);
-		d = g_strconcat(s, buf, NULL);
-		g_free(s);
-		s = d;
+		if (bat->changed)
+			gkrellmd_need_serve(mon);
 		}
-	if (*s)
-		{
-		d = g_strconcat("<battery>\n", s, NULL);
-		g_free(s);
-		s = d;
-		}
-	return s;
 	}
 
 static void
-serve_battery_setup(GkrellmdClient *client)
+serve_battery_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
+	Battery		*bat;
+	GList		*list;
+	gchar		buf[128];
+
+	gkrellmd_set_serve_name(mon, "battery");
+	for (list = battery_list; list; list = list->next)
+		{
+		bat = (Battery *) list->data;
+
+		if (   (!bat->changed && !first_serve)
+			|| (   !gkrellmd_check_client_version(mon, 2,1,9)
+				&& bat->id > 0
+			   )
+		   )
+			continue;
+		snprintf(buf, sizeof(buf), "%d %d %d %d %d %d\n",
+					bat->present, bat->on_line, bat->charging,
+					bat->percent, bat->time_left, bat->id);
+		gkrellmd_serve_data(mon, buf);
+		}
+	}
+
+static void
+serve_battery_setup(GkrellmdMonitor *mon)
+	{
+	GkrellmdClient	*client = mon->privat->client;
+
 	gkrellm_sys_battery_read_data();
 	if (battery_list)
-		{
-		gkrellmd_send_to_client(client, "<battery_setup>\nbattery_available\n");
-		}
+		gkrellmd_send_to_client(client,
+					"<battery_setup>\nbattery_available\n");
 	}
 
 static GkrellmdMonitor battery_monitor =
@@ -1651,9 +1658,6 @@ init_battery_monitor(void)
 	}
 
 /* ======================================================= */
-#define SENSOR_TEMPERATURE  0
-#define SENSOR_FAN          1
-#define SENSOR_VOLTAGE      2
 
 typedef struct
 	{
@@ -1669,6 +1673,7 @@ typedef struct
 
 	gchar		*vref;
 	gchar		*default_label;
+	gint		group;
 
 	gfloat		factor;
 	gfloat		offset;
@@ -1684,9 +1689,10 @@ static gboolean thread_busy,
 static gpointer
 read_sensors(void *data)
 	{
-	GList	*list;
-	Sensor	*sensor;
-	gfloat	tmp;
+	GList		*list;
+	Sensor		*sensor;
+	gfloat		tmp;
+	gboolean	need_serve = FALSE;
 
 	for (list = sensors_list; list; list = list->next)
 		{
@@ -1704,12 +1710,14 @@ read_sensors(void *data)
 		if (sensor->raw_value != tmp)
 			{
 			sensor->changed = TRUE;
-			sensors_need_serve = TRUE;
+			need_serve = TRUE;
 			}
 		else
 			sensor->changed = FALSE;
 		}
 	thread_busy = FALSE;
+	sensors_need_serve = need_serve; /* Thread, so set after data collected */
+
 	return NULL;
 	}
 
@@ -1732,6 +1740,16 @@ gkrellm_sensors_update_volt_order_base(void)
 	{
 	}
 
+void
+gkrellm_sensors_set_group(gpointer sr, gint group)
+	{
+	Sensor	*sensor = (Sensor *) sr;
+
+	if (sensor)
+		sensor->group = group;
+	}
+
+
   /* A sensor within a type is uniquely identified by its id_name.
   |  A sysdep interface may additionally use any of the triple integer
   |  set (id, iodev, inter) for internal identification.
@@ -1739,7 +1757,7 @@ gkrellm_sensors_update_volt_order_base(void)
   |  passed to the client since that is all that is needed for identification
   |  (the client is no longer interfacing to sysdep code).
   */
-void
+gpointer
 gkrellm_sensors_add_sensor(gint type, gchar *sensor_path, gchar *id_name,
 		gint id, gint iodev, gint inter,
 		gfloat factor, gfloat offset, gchar *vref, gchar *default_label)
@@ -1747,7 +1765,7 @@ gkrellm_sensors_add_sensor(gint type, gchar *sensor_path, gchar *id_name,
 	Sensor	*sensor;
 
 	if (!id_name || !*id_name || type < 0 || type > 2)
-		return;
+		return NULL;
 
 	sensor = g_new0(Sensor, 1);
 	sensor->id_name = g_strdup(id_name);
@@ -1767,60 +1785,75 @@ gkrellm_sensors_add_sensor(gint type, gchar *sensor_path, gchar *id_name,
 	sensor->iodev = iodev;
 	sensor->inter = inter;
 	sensors_list = g_list_append(sensors_list, sensor);
+	return sensor;
 	}
 
 static void
-update_sensors(gboolean force)
+update_sensors(GkrellmdMonitor *mon, gboolean first_update)
 	{
-	if (!GK.five_second_tick && !force)
+	if (sensors_need_serve)		/* Asynchronously set in thread */
+		gkrellmd_need_serve(mon);
+	sensors_need_serve = FALSE;
+
+	if (!GK.five_second_tick && !first_update)
 		return;
-	if (force)
+	if (first_update)
 		read_sensors(NULL);		/* No thread on first read */
 	else
 		run_sensors_thread();
 	}
 
-static gchar *
-serve_sensors_data(GkrellmdClient *client)
+static void
+serve_sensors_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
-	Sensor	*sr;
-	GList	*list;
-	gchar	*s, *d, buf[128];
+	Sensor			*sr;
+	GList			*list;
+	gchar			buf[128];
+	gboolean		sensor_disk_ok;
 
-	if (!sensors_need_serve && client->served)
-		return NULL;
-	s = g_strdup("<sensors>\n");
+	gkrellmd_set_serve_name(mon, "sensors");
+	sensor_disk_ok = gkrellmd_check_client_version(mon, 2,2,0);
 	for (list = sensors_list; list; list = list->next)
 		{
 		sr = (Sensor *) list->data;
-		if (sr->changed || !client->served)
+		if (sr->group == SENSOR_GROUP_DISK && !sensor_disk_ok)
+			continue;
+		if (sr->changed || first_serve)
 			{
 			sprintf(buf, "%d \"%s\" %d %d %d %.2f\n",
 					sr->type, sr->id_name,
 					sr->id, sr->iodev, sr->inter, sr->raw_value);
-			d = g_strconcat(s, buf, NULL);
-			g_free(s);
-			s = d;
+			gkrellmd_serve_data(mon, buf);
 			}
 		}
-	return s;
 	}
 
 static void
-serve_sensors_setup(GkrellmdClient *client)
+serve_sensors_setup(GkrellmdMonitor *mon)
 	{
-	GList	*list;
-	Sensor	*s;
-	gchar	buf[256];
+	GkrellmdClient	*client = mon->privat->client;
+	GList			*list;
+	Sensor			*s;
+	gchar			buf[256];
+	gboolean		sensor_disk_ok;
 
 	gkrellmd_send_to_client(client, "<sensors_setup>\n");
+	sensor_disk_ok = gkrellmd_check_client_version(mon, 2,2,0);
 	for (list = sensors_list; list; list = list->next)
 		{
 		s = (Sensor *) list->data;
-		sprintf(buf, "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\"\n",
-				s->type, s->id_name,
-				s->id, s->iodev, s->inter,
-				s->factor, s->offset, s->vref, s->default_label);
+		if (s->group == SENSOR_GROUP_DISK && !sensor_disk_ok)
+			continue;
+		if (sensor_disk_ok)
+			sprintf(buf, "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\" %d\n",
+					s->type, s->id_name,
+					s->id, s->iodev, s->inter,
+					s->factor, s->offset, s->vref, s->default_label, s->group);
+		else
+			sprintf(buf, "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\"\n",
+					s->type, s->id_name,
+					s->id, s->iodev, s->inter,
+					s->factor, s->offset, s->vref, s->default_label);
 		gkrellmd_send_to_client(client, buf);
 		}
 	}
@@ -1838,8 +1871,6 @@ init_sensors_monitor(void)
 	{
 	if (!gkrellm_sys_sensors_init())
 		return NULL;
-	serveflag_done_list =
-			g_list_append(serveflag_done_list, &sensors_need_serve);
 	return &sensors_monitor;
 	}
 
@@ -1848,8 +1879,6 @@ static time_t	base_uptime,
 				up_seconds;
 static gulong	up_minutes = -1;
 
-static gboolean	need_uptime_serve;
-
 void
 gkrellm_uptime_set_base_uptime(time_t base)
 	{
@@ -1857,11 +1886,11 @@ gkrellm_uptime_set_base_uptime(time_t base)
 	}
 
 static void
-update_uptime(gboolean force)
+update_uptime(GkrellmdMonitor *mon, gboolean first_update)
 	{
 	gint	prev_up;
 
-	if (GK.ten_second_tick || up_minutes < 0 || force)
+	if (GK.ten_second_tick || up_minutes < 0 || first_update)
 		{
 		prev_up = up_minutes;
 		up_seconds = gkrellm_sys_uptime_read_uptime();
@@ -1870,21 +1899,18 @@ update_uptime(gboolean force)
 		else
 			up_minutes = (gint)(time(0) - _GK.start_time + base_uptime) / 60;
 		if (up_minutes != prev_up)
-			need_uptime_serve = TRUE;
+			gkrellmd_need_serve(mon);
 		}
 	}
 
-static gchar *
-serve_uptime_data(GkrellmdClient *client)
+static void
+serve_uptime_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
-	gchar	*s = NULL;
+	gchar	buf[128];
 
-	if (need_uptime_serve || !client->served)
-		{
-		s = g_strdup_printf("<uptime>\n%lu\n", (gulong) up_minutes);
-		need_uptime_serve = FALSE;
-		}
-	return s;
+	gkrellmd_set_serve_name(mon, "uptime");
+	snprintf(buf, sizeof(buf), "%lu\n", (gulong) up_minutes);
+	gkrellmd_serve_data(mon, buf);
 	}
 
 static GkrellmdMonitor uptime_monitor =
@@ -1920,17 +1946,35 @@ send_time(GkrellmdClient *client)
 
 /* ======================================================= */
 
+void
+gkrellmd_plugin_serve_setup(GkrellmdMonitor *mon, gchar *name, gchar *line)
+	{
+	GkrellmdClient	*client = mon->privat->client;
+	gchar			buf[256];
+
+	if (!mon || !name || !line)
+		return;
+	gkrellmd_send_to_client(client, "<plugin_setup>\n");
+	snprintf(buf, sizeof(buf), "%s %s\n", name, line);
+	gkrellmd_send_to_client(client, buf);
+	}
+
 static void
 add_monitor(GkrellmdMonitor *mon)
 	{
 	if (!mon)
 		return;
+	mon->privat = g_new0(GkrellmdMonitorPrivate, 1);
+	mon->privat->serve_gstring = g_string_new("");
 	gkrellmd_monitor_list = g_list_append(gkrellmd_monitor_list, mon);
 	}
 
 void
 gkrellmd_load_monitors(void)
 	{
+	GList			*list;
+	GkrellmdMonitor	*mon;
+
 	add_monitor(init_sensors_monitor());
 	add_monitor(init_cpu_monitor());
 	add_monitor(init_proc_monitor());
@@ -1942,26 +1986,48 @@ gkrellmd_load_monitors(void)
 	add_monitor(gkrellmd_init_mail_monitor());
 	add_monitor(init_battery_monitor());
 	add_monitor(init_uptime_monitor());
+
+	list = gkrellmd_plugins_load();
+	if (_GK.list_plugins)
+		exit(0);
+	if (_GK.log_plugins)
+		printf("%s\n", plugin_install_log);
+	for (  ; list; list = list->next)
+		{
+		mon = (GkrellmdMonitor *) list->data;
+		mon->privat->serve_gstring = g_string_new("");
+		mon->privat->is_plugin = TRUE;
+		gkrellmd_monitor_list = g_list_append(gkrellmd_monitor_list, mon);
+		}
+	}
+
+void
+gkrellmd_need_serve(GkrellmdMonitor *mon)
+	{
+	if (mon)
+		mon->privat->need_serve = TRUE;
 	}
 
 gint
 gkrellmd_update_monitors(void)
 	{
 	GList			*list, *c_list;
-	GkrellmdMonitor	*mon;
-	GkrellmdClient	*client;
-	struct tm		*pCur;
-	static time_t	time_prev, time_now;
-	gchar			*d, *s, *send_string, buf[64];
-	static gboolean	force = TRUE;
+	GkrellmdMonitor			*mon;
+	GkrellmdMonitorPrivate	*mp;
+	GkrellmdClient			*client;
+	struct tm				*pCur;
+	static time_t			time_prev;
+	gchar					buf[64];
+	static gboolean			first_update = TRUE;
+	static GString			*serve_gstring;
 
-	time(&time_now);
-	GK.second_tick = (time_now == time_prev) ? FALSE : TRUE;
-	time_prev = time_now;
+	time(&_GK.time_now);
+	GK.second_tick = (_GK.time_now == time_prev) ? FALSE : TRUE;
+	time_prev = _GK.time_now;
 
 	if (GK.second_tick)
 		{
-		pCur = localtime(&time_now);
+		pCur = localtime(&_GK.time_now);
 		GK.two_second_tick  = ((pCur->tm_sec % 2) == 0) ? TRUE : FALSE;
 		GK.five_second_tick = ((pCur->tm_sec % 5) == 0) ? TRUE : FALSE;
 		GK.ten_second_tick  = ((pCur->tm_sec % 10) == 0) ? TRUE : FALSE;
@@ -1980,34 +2046,37 @@ gkrellmd_update_monitors(void)
 		{
 		mon = (GkrellmdMonitor *) list->data;
 		if (mon->update_monitor)
-			(*(mon->update_monitor))(force);
+			(*(mon->update_monitor))(mon, first_update);
 		}
 	++GK.timer_ticks;
+	if (!serve_gstring)
+		serve_gstring = g_string_new("");
 	for (c_list = gkrellmd_client_list; c_list; c_list = c_list->next)
 		{
 		client = (GkrellmdClient *) c_list->data;
 		client->last_client = !c_list->next;
 		if (!client->served)
 			gkrellmd_send_to_client(client, "<initial_update>\n");
-		send_string = g_strdup("");
 		for (list = gkrellmd_monitor_list; list; list = list->next)
 			{
 			mon = (GkrellmdMonitor *) list->data;
-			if (mon->serve_data)
+			mp = mon->privat;
+			if (!mon->serve_data || (!mp->need_serve && client->served))
+				continue;
+			mp->client = client;
+			mp->serve_name = NULL;
+			mp->serve_name_sent = FALSE;
+			(*(mon->serve_data))(mon, client->served ? FALSE : TRUE);
+			if (mp->serve_gstring->len > 0)
 				{
-				d = (*(mon->serve_data))(client);
-				if (d && *d)
-					{
-					s = g_strconcat(send_string, d, NULL);
-					g_free(send_string);
-					send_string = s;
-					}
-				g_free(d);
+				serve_gstring =
+					g_string_append(serve_gstring, mp->serve_gstring->str);
+				mp->serve_gstring = g_string_truncate(mp->serve_gstring, 0);
 				}
 			}
-		gkrellmd_send_to_client(client, send_string);
-		g_free(send_string);
-
+		gkrellmd_send_to_client(client, serve_gstring->str);
+		serve_gstring = g_string_truncate(serve_gstring, 0);
+		
 		if (GK.minute_tick || !client->served)
 			send_time(client);
 		else if (GK.second_tick)
@@ -2021,10 +2090,17 @@ gkrellmd_update_monitors(void)
 		client->served = TRUE;
 		}
 
+	for (list = gkrellmd_monitor_list; list; list = list->next)
+		{
+		mon = (GkrellmdMonitor *) list->data;
+		mp = mon->privat;
+		mp->need_serve = FALSE;
+		}
+
 	for (list = serveflag_done_list; list; list = list->next)
 		*((gboolean *) list->data) = FALSE;
 
-	force = FALSE;
+	first_update = FALSE;
 	return TRUE;
 	}
 
@@ -2047,8 +2123,9 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 	for (list = gkrellmd_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmdMonitor *) list->data;
+		mon->privat->client = client;
 		if (mon->serve_setup)
-			(*(mon->serve_setup))(client);
+			(*(mon->serve_setup))(mon);
 		}
 	name = gkrellm_sys_get_host_name();
 	s = g_strconcat("<hostname>\n", name, "\n", NULL);
