@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -96,6 +96,8 @@ gkrellm_clear_chart(GkrellmChart *cp)
 		return;
 	gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_src_pixmap,
 						0, 0,	0, 0,   cp->w, cp->h);
+	gdk_draw_drawable(cp->bg_pixmap, _GK.draw1_GC, cp->bg_src_pixmap,
+						0, 0,	0, 0,   cp->w, cp->h);
 	if (cp->drawing_area->window)
 		gdk_draw_drawable(cp->drawing_area->window, _GK.draw1_GC, cp->pixmap,
 						0, 0,	0, 0,   cp->w, cp->h);
@@ -108,6 +110,8 @@ gkrellm_clear_chart_pixmap(GkrellmChart *cp)
 		return;
 	gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_src_pixmap,
 						0, 0,	0, 0,   cp->w, cp->h);
+	gdk_draw_drawable(cp->bg_pixmap, _GK.draw1_GC, cp->bg_src_pixmap,
+						0, 0,	0, 0,   cp->w, cp->h);
 	}
 
 void
@@ -118,6 +122,7 @@ gkrellm_clean_bg_src_pixmap(GkrellmChart *cp)
 	if (!gkrellm_winop_draw_rootpixmap_onto_transparent_chart(cp))
 		gdk_draw_drawable(cp->bg_src_pixmap, _GK.draw1_GC,
 				cp->bg_clean_pixmap, 0, 0, 0, 0, cp->w, cp->h);
+	cp->bg_sequence_id += 1;
 	}
 
 void
@@ -459,7 +464,7 @@ fix_y(gint yn, gint yp, gint ypp)
 static void
 draw_chartdata_lines(GkrellmChart *cp, GkrellmChartdata *cd, gint i0)
 	{
-	gint	n, x, y, y0, y1;
+	gint	n, x, xp, y, y0, y1;
 
 	y0 = chartdata_ycoord(cp, cd, 0);
 	for (n = i0; n < cp->w; ++n)
@@ -468,9 +473,16 @@ draw_chartdata_lines(GkrellmChart *cp, GkrellmChartdata *cd, gint i0)
 		y1 = chartdata_ycoord(cp, cd, cd->data[x]);
 		if (n == 0)
 			cd->y_p = y1;
+		else if (!cd->y_p_valid && i0 == cp->w - 1)
+			{
+			if ((xp = x - 1) < 0)
+				xp = cp->w - 1;
+			cd->y_p = cd->y_pp = chartdata_ycoord(cp, cd, cd->data[xp]);
+			}
 		y = fix_y(y1, cd->y_p, cd->y_pp);
 		cd->y_pp = y;
 		cd->y_p = y1;
+		cd->y_p_valid = FALSE;	/* Need a store_chartdata to make it valid */
 		if (cd->data[x] > 0 || (cd->inverted ? (y > y0) : (y < y0)))
 			{
 			if (y == y1)
@@ -882,6 +894,7 @@ gkrellm_store_chartdata(GkrellmChart *cp, gulong total, ...)
 			}
 		else if (cd->draw_style == CHARTDATA_LINE)
 			{
+			cd->y_p_valid = TRUE;
 			if (cd->discard && cd->discard >= cd->maxval)
 				need_scan = TRUE;
 			else
@@ -907,15 +920,28 @@ gkrellm_store_chartdata(GkrellmChart *cp, gulong total, ...)
 
 /* =================================================================== */
 
-static gint
-chartdata_text_y(GkrellmChart *cp, char key, gint ascent)
+static void
+chart_destroy_text_run_list(GkrellmChart *cp)
 	{
-	GList		*list;
+	GList			*list;
+
+	if (!cp || !cp->text_run_list)
+		return;
+	for (list = cp->text_run_list; list; list = list->next)
+		g_free(((GkrellmTextRun *) list->data)->text);
+	gkrellm_free_glist_and_data(&cp->text_run_list);
+	}
+
+static gint
+chartdata_text_y(GkrellmChart *cp, char key, gint height,
+				gint y_ink, gint shadow)
+	{
+	GList				*list;
 	GkrellmChartdata	*cd;
-	gint		n, y;
+	gint				n, y;
 
 	n = key - '0';
-	y = -1;
+	y = -100;
 	if (n >= 0)
 		{
 		list = g_list_nth(cp->cd_list, n / 2);
@@ -924,10 +950,14 @@ chartdata_text_y(GkrellmChart *cp, char key, gint ascent)
 			cd = (GkrellmChartdata *) list->data;
 			if (!cd->hide)
 				{
-				if (n & 1)
-					y = cd->y + cd->h - 3 - ascent;
-				else
-					y = cd->y + 1;
+				if (n & 1)	/* Justify 2 pixels from top of ChartData view */
+					{
+					y = cd->y + cd->h + y_ink - 3;
+					}
+				else		/* Justify to bottom of ChartData view */
+					{
+					y = cd->y + height + y_ink + shadow - 1;
+					}
 				y = Y_SCREEN_MAP(cp, y);
 				}
 			}
@@ -935,28 +965,66 @@ chartdata_text_y(GkrellmChart *cp, char key, gint ascent)
 	return y;
 	}
 
+void
+gkrellm_chart_reuse_text_format(GkrellmChart *cp)
+	{
+	cp->text_format_reuse = TRUE;
+	}
+
 void
 gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 	{
+	GList			*list;
+	GkrellmTextRun	*tr;
 	GkrellmTextstyle *ts, *ts_default, *ts_alt;
 	gchar			c, *s, *t;
-	gint			text_length, ascent, descent, field_width, fw;
-	gint			xx, x, y, w, l, r, a, d, center;
+	gint			h, text_length, field_width, fw;
+	gint			offset;
+	gint			xx, x, y, center, shadow;
 	gboolean		right, set_fw, fw_right;
 
 	if (!cp || !str)
 		return;
+
+	/* Assume text_format will be changed at each call unless caller has said
+	|  we can reuse it or the whole string compares equal.
+	*/
+	if (   !cp->text_format_reuse
+		&& !gkrellm_dup_string(&cp->text_format_string, str)
+	   )
+		cp->text_format_reuse = TRUE;
+
+	if (   !cp->text_format_reuse
+		|| cp->text_run_sequence_id != cp->bg_sequence_id
+	   )
+		chart_destroy_text_run_list(cp);
+	cp->text_run_sequence_id = cp->bg_sequence_id;
+	cp->text_format_reuse = FALSE;
+
 	ts_default = gkrellm_chart_textstyle(style_id);
 	ts_alt = gkrellm_chart_alt_textstyle(style_id);
 
-	w = l = r = a = d = 0;
 	x = xx = 2;
-	gkrellm_winop_text_extents(ts_default->font, _("Ay8"), 3, &l, &r, &w,
-					&ascent, &descent);
-	y = 2 + ascent;
-	fw = 0;
-	for (s = str; *s; s += text_length)
+	if (!cp->text_run_list)
+		gkrellm_text_extents(ts_default->font, _("Ag8"), 3, NULL,
+					&cp->h_text, &cp->baseline_ref, &cp->y_ink);
+
+	y = 2 - cp->y_ink;
+	h = fw = 0;
+	tr = NULL;
+	for (list = cp->text_run_list, s = str; *s; s += text_length)
 		{
+		if (!list)
+			{
+			tr = g_new0(GkrellmTextRun, 1);
+			cp->text_run_list = g_list_append(cp->text_run_list, tr);
+			}
+		else
+			{
+			tr = (GkrellmTextRun *) list->data;
+			list = list->next;
+			tr->cache_valid = TRUE;
+			}
 		c = '\0';
 		center = 0;
 		right = FALSE;
@@ -964,13 +1032,14 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 		fw_right = FALSE;
 		ts = ts_default;
 		field_width = 0;
+		shadow = ts_default->effect ? 1 : 0;
 		while (*s == '\\')
 			{
 			if ((c = *(++s)) != '\0')
 				++s;
 			if (c == 'n')
 				{
-				y += ascent + descent;
+				y += cp->h_text + 1;
 				x = xx;
 				}
 			else if (c == 'c')
@@ -980,24 +1049,24 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 			else if (c == 'N')
 				{
 				x = xx;		/* A conditional newline.  If previous string */
-				if (a > 2)	/* was spaces, no nl.  A space has nonzero a  */
-					y += ascent + descent;
+				if (h > 2)	/* was spaces, no nl.  A space has nonzero a  */
+					y += cp->h_text + 1;
 				}
 			else if (c == 'b')
 				{
-				y = cp->h - 3;
+				y = cp->h - cp->h_text - cp->y_ink - 1;
 				x = xx;
 				}
 			else if (c == 't')
 				{
-				y = 2 + ascent;
+				y = 2 - cp->y_ink;
 				x = xx;
 				}
 			else if (c == 'r')
 				right = TRUE;
 			else if (c == 'p')
 				{
-				y -= ascent + descent;
+				y -= cp->h_text + 1;
 				x = xx;
 				}
 			else if (c == 'w')
@@ -1010,14 +1079,17 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 				fw_right = TRUE;
 				}
 			else if (c == 'f')
+				{
 				ts = ts_alt;
+				shadow = ts_alt->effect ? 1 : 0;
+				}
 			else if (c == 'x' && isdigit(*s))
 				xx = *s++ - '0';
 			else if (c == 'y' && isdigit(*s))
 				y = *s++ - '0';
 			else if (c == 'D')
 				{
-				y = chartdata_text_y(cp, *s++, ascent);
+				y = chartdata_text_y(cp, *s++, cp->h_text, cp->y_ink, shadow);
 				x = xx;
 				}
 			}
@@ -1027,53 +1099,88 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 		else
 			text_length = strlen(s);
 
-		if (y < 0)	/* asked for a chartdata that is hidden */
+		if (y == -100)	/* asked for a chartdata that is hidden */
 			continue;
 
-		gkrellm_winop_text_extents(ts->font, s, text_length,
-					&l, &r, &w, &a, &d);
+		if (   !tr->cache_valid || !tr->text
+			|| strncmp(tr->text, s, text_length)
+			|| strlen(tr->text) != text_length
+		   )
+			{
+			gkrellm_text_extents(ts->font, s, text_length,
+						&tr->w, &tr->h, &tr->baseline, &tr->y_ink);
+			tr->cache_valid = FALSE;
+			g_free(tr->text);
+			tr->text = g_strndup(s, text_length);
+			}
+		h = tr->h;
 
 		if (set_fw)
 			{
-			fw = r + ts->effect;
+			fw = tr->w + shadow;
 			continue;
 			}
 		if (center == 1)
-			x = (cp->w - r) / 2;
+			x = (cp->w - tr->w) / 2;
 		else if (center == 2)
 			x = cp->w / 2;
 		else if (fw_right)
-			x = x + fw - r - ts->effect;
+			x = x + fw - tr->w - shadow;
 		else if (right)
-			x = cp->w - r - 2 - ts->effect;
+			x = cp->w - tr->w - 2 - shadow;
 		if (text_length > 1 || (text_length == 1 && *s != ' '))
 			{
-			gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_pixmap,
-					x - l, y - a - 1, x - l, y - a - 1,
-					l + r + ts->effect + 1, a + d + ts->effect + 1);
-    		gkrellm_draw_text(cp->pixmap, ts, x, y, s, text_length);
+			if (x != tr->x || y != tr->y)
+				tr->cache_valid = FALSE;
+			tr->x = x;
+			tr->y = y;
+#if 0
+gchar buf[128];
+strncpy(buf, s, text_length);
+buf[text_length] = '\0';
+if (!tr->cache_valid)
+printf("pango draw chart text: %s\n", buf);
+#endif
+			
+			offset = cp->baseline_ref - tr->baseline;	/* align baselines */
+			if (!tr->cache_valid)
+				{
+				gdk_draw_drawable(cp->bg_text_pixmap, _GK.draw1_GC,
+						cp->bg_pixmap,
+						x - 1, y + tr->y_ink + offset - 1,
+						x - 1, y + tr->y_ink + offset - 1,
+						tr->w + shadow + 2, tr->h + shadow + 1);
+	    		gkrellm_draw_text(cp->bg_text_pixmap, ts, x, y + offset, s,
+						text_length);
+				}
+			gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_text_pixmap,
+					x - 1, y + tr->y_ink + offset - 1,
+					x - 1, y + tr->y_ink + offset - 1,
+					tr->w + shadow + 2, tr->h + shadow + 1);
 			}
 		if (field_width && !fw_right)
-			x += (field_width > r + l) ? field_width : r + l;
+			x += (field_width > tr->w) ? field_width : tr->w;
 		else
-			x += r + l + ts->effect;
+			x += tr->w + shadow;
 		}
 	}
 
 gint
-gkrellm_draw_chart_label(GkrellmChart *cp, GkrellmTextstyle *ts, gint x, gint y, gchar *s)
+gkrellm_draw_chart_label(GkrellmChart *cp, GkrellmTextstyle *ts,
+				gint x, gint y, gchar *s)
 	{
-	gint	w, l, r, a, d;
+	gint	w, h, y_ink;
 
 	if (!cp || !ts || !s)
 		return x;
-	gkrellm_winop_text_extents(ts->font, s, strlen(s), &l, &r, &w, &a, &d);
+
+	gkrellm_text_extents(ts->font, s, strlen(s), &w, &h, NULL, &y_ink);
 
 	gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_pixmap,
-		x - l, y - a, x - l, y - a, l + r + ts->effect, a + d + ts->effect);
-    gkrellm_draw_string(cp->pixmap, ts, x, y, s);
+				x, y, x, y, w + ts->effect, h + ts->effect);
+    gkrellm_draw_string(cp->pixmap, ts, x, y - y_ink, s);
 
-	return x + r;
+	return x + w + ts->effect;
 	}
 
 void
@@ -1099,7 +1206,7 @@ gkrellm_destroy_chartdata_list(GList **cd_list_head)
 		|  creating monitor.
 		*/
 		}
-	free_glist_and_data(cd_list_head);
+	gkrellm_free_glist_and_data(cd_list_head);
 	}
 
 
@@ -1124,6 +1231,7 @@ free_chart_pixmaps(GkrellmChart *cp)
 	cp->cd_list_index = 0;
 
 	gkrellm_free_pixmap(&cp->bg_pixmap);
+	gkrellm_free_pixmap(&cp->bg_text_pixmap);
 	gkrellm_free_pixmap(&cp->bg_src_pixmap);
 	gkrellm_free_pixmap(&cp->bg_grid_pixmap);
 
@@ -1147,6 +1255,7 @@ destroy_chart_data(GkrellmChart *cp)
 	GkrellmChartdata *cd;
 
 	free_chart_pixmaps(cp);
+	chart_destroy_text_run_list(cp);
 	for (list = cp->cd_list; list; list = list->next)
 		{
 		cd = (GkrellmChartdata *) list->data;
@@ -1427,7 +1536,7 @@ render_default_data_pixmaps(GkrellmChart *cp)
 	}
 
 void
-setup_charts(void)
+gkrellm_chart_setup(void)
 	{
 	gkrellm_free_pixmap(&_GK.data_in_pixmap);
 	gkrellm_free_pixmap(&_GK.data_in_grid_pixmap);
@@ -1646,6 +1755,7 @@ render_chart_pixmaps(GkrellmChart *cp)
 	gkrellm_clone_pixmap(&cp->pixmap, &cp->bg_clean_pixmap);
 	gkrellm_clone_pixmap(&cp->bg_pixmap, &cp->bg_clean_pixmap);
 	gkrellm_clone_pixmap(&cp->bg_src_pixmap, &cp->bg_clean_pixmap);
+	gkrellm_clone_pixmap(&cp->bg_text_pixmap, &cp->bg_clean_pixmap);
 	}
 
 void
@@ -1726,6 +1836,8 @@ gkrellm_chart_create(GtkWidget *vbox, GkrellmMonitor *mon, GkrellmChart *cp,
 
 	gtk_widget_set_size_request(cp->drawing_area, cp->w, cp->h);
 
+	cp->bg_sequence_id += 1;
+
 	render_chart_pixmaps(cp);
 	render_chart_margin_spacers(cp);
 
