@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -54,11 +54,10 @@ static GkrellmLauncher
 typedef struct
 	{
 	GkrellmTextstyle ts;
-	gint		lbearing,
-				rbearing,
-				width,
-				ascent,
-				descent;
+	gint		width,
+				height,
+				baseline,
+				y_ink;
 	}
 	Extents;
 
@@ -106,9 +105,8 @@ gkrellm_get_current_time(void)
 static void
 string_extents(gchar *text, Extents *ext)
 	{
-	gkrellm_winop_text_extents(ext->ts.font, text, strlen(text),
-			&ext->lbearing, &ext->rbearing,
-			&ext->width, &ext->ascent, &ext->descent);
+	gkrellm_text_extents(ext->ts.font, text, strlen(text),
+			&ext->width, &ext->height, &ext->baseline, &ext->y_ink);
 	}
 
 static gpointer
@@ -131,11 +129,25 @@ chime_func(gpointer data)
 	return NULL;
 	}
 
+static gchar *
+utf8_from_strftime(gchar *buf)
+	{
+	gchar	*utf8 = NULL;
+
+	if (   g_utf8_validate(buf, -1, NULL)
+		|| (utf8 = g_locale_to_utf8(buf, -1, NULL, NULL, NULL)) == NULL
+	   )
+		utf8 = g_strdup(buf);
+
+	return utf8;
+	}
+
+
 static void
 draw_cal(void)
 	{
 	struct tm 	*t;
-	gchar		buf[32];
+	gchar		*utf8, buf[32];
 	gint		w;
 
 	if (!cal_enable)
@@ -146,9 +158,11 @@ draw_cal(void)
 		t = &gkrellm_current_tm;
 
 	strftime(buf, sizeof(buf), "%a", t);  /* Abbreviated weekday name */
-	w = gkrellm_gdk_string_width(d_wday->text_style.font, buf);
+	utf8 = utf8_from_strftime(buf);
+	w = gkrellm_gdk_string_width(d_wday->text_style.font, utf8);
 	d_wday->x_off = (w < d_wday->w) ? (d_wday->w - w) / 2 : 0;
-	gkrellm_draw_decal_text(pcal, d_wday, buf, -1);
+	gkrellm_draw_decal_text(pcal, d_wday, utf8, -1);
+	g_free(utf8);
 
 	/*strftime(buf, sizeof(buf), "%e", t);*/  /* Day in month 1-31 */
 	snprintf(buf, sizeof(buf), "%d", t->tm_mday);
@@ -157,7 +171,10 @@ draw_cal(void)
 	gkrellm_draw_decal_text(pcal, d_mday, buf, -1);
 
 	strftime(buf, sizeof(buf), "%b", t); /* Abbreviated month name */
-	gkrellm_draw_decal_text(pcal, d_month, buf, -1);
+	utf8 = utf8_from_strftime(buf);
+	gkrellm_draw_decal_text(pcal, d_month, utf8, -1);
+	g_free(utf8);
+
 	gkrellm_draw_panel_layers(pcal);
 	}
 
@@ -166,7 +183,7 @@ static void
 draw_clock(gboolean update_minutes)
 	{
 	struct tm 	*t;
-	gchar		buf[32], sec_ampm[32];
+	gchar		buf[32], sec_ampm[32], *utf8;
 	gint		w;
 
 	if (!clock_enable)
@@ -197,7 +214,14 @@ draw_clock(gboolean update_minutes)
 	else if (! clock_24hr_mode)
 		{
 		strftime(sec_ampm, sizeof(sec_ampm), "%p", t);	/* AM/PM */
-		gkrellm_draw_decal_text(pclock, d_seconds, sec_ampm, -1);
+		if (sec_ampm[0] == '\0')		/* locale doesn't support AM/PM */
+			strcpy(sec_ampm, "--");
+		if (   g_utf8_validate(sec_ampm, -1, NULL)
+			|| (utf8 = g_locale_to_utf8(sec_ampm, -1, NULL, NULL,NULL)) == NULL
+		   )
+			utf8 = g_strdup(sec_ampm);
+		gkrellm_draw_decal_text(pclock, d_seconds, utf8, -1);
+		g_free(utf8);
 		}
 	gkrellm_draw_panel_layers(pclock);
 	}
@@ -294,25 +318,75 @@ static void
 create_calendar(GtkWidget *vbox, gint first_create)
 	{
 	GkrellmStyle	*style;
-	gint	w;
+	Extents			ext;
+	gchar			buf[32], *utf8, *str = NULL;
+	gint			m, d, w;
+	struct tm		tm = *gkrellm_get_current_time();
 
 	if (first_create)
 		pcal = gkrellm_panel_new0();
 	style = gkrellm_meter_style(cal_style_id);
+	ext.ts = *gkrellm_meter_textstyle(cal_style_id);
 
+	/* Get max weekday name height for current locale
+	*/
+	wday_extents.width = wday_extents.height = 0;
+	wday_extents.baseline = wday_extents.y_ink = 0;
+	for (d = 0; d < 6; ++d)
+		{
+		tm.tm_wday = d;
+		strftime(buf, sizeof(buf), "%a", &tm);  /* Abbreviated weekday name */
+		utf8 = utf8_from_strftime(buf);
+		string_extents(utf8, &ext);
+		wday_extents.baseline = ext.baseline;	/* not used */
+		if (ext.width > wday_extents.width)
+			wday_extents.width = ext.width;
+		if (ext.height > wday_extents.height)
+			{
+			str = utf8;
+			wday_extents.height = ext.height;	/* not used */
+			wday_extents.y_ink = ext.y_ink;		/* not used */
+			}
+		else
+			g_free(utf8);
+		}
 	wday_extents.ts = *gkrellm_meter_textstyle(cal_style_id);
-	string_extents("Wed", &wday_extents);
+	d_wday = gkrellm_create_decal_text(pcal, str, &wday_extents.ts,
+				style, 0, -1 /*top margin*/, wday_extents.width + 2);
+	g_free(str);
+
 	mday_extents.ts = *gkrellm_meter_alt_textstyle(cal_style_id);
 	string_extents("28", &mday_extents);
-	month_extents.ts = *gkrellm_meter_textstyle(cal_style_id);
-	string_extents("Aug", &month_extents);
-
-	d_wday = gkrellm_create_decal_text(pcal, "88", &wday_extents.ts,
-				style, 0, -1 /*top margin*/, wday_extents.width + 2);
-	d_mday = gkrellm_create_decal_text(pcal, "88", &mday_extents.ts,
+	d_mday = gkrellm_create_decal_text(pcal, "28", &mday_extents.ts,
 				style, 0, -1 /*top margin*/, mday_extents.width + 2);
-	d_month = gkrellm_create_decal_text(pcal, "88", &month_extents.ts,
+
+	/* Get max month name height for current locale
+	*/
+	month_extents.width = month_extents.height = 0;
+	month_extents.baseline = month_extents.y_ink = 0;
+	for (m = 0; m < 11; ++m)
+		{
+		tm.tm_mon = m;
+		strftime(buf, sizeof(buf), "%b", &tm); /* Abbreviated month name */
+		utf8 = utf8_from_strftime(buf);
+		string_extents(utf8, &ext);
+		month_extents.baseline = ext.baseline;	/* not used */
+		if (ext.width > month_extents.width)
+			month_extents.width = ext.width;
+		if (ext.height > month_extents.height)
+			{
+			str = utf8;
+			month_extents.height = ext.height;	/* not used */
+			month_extents.y_ink = ext.y_ink;		/* not used */
+			}
+		else
+			g_free(utf8);
+		}
+	month_extents.ts = *gkrellm_meter_textstyle(cal_style_id);
+	d_month = gkrellm_create_decal_text(pcal, str, &month_extents.ts,
 				style, 0, -1 /*top margin*/, month_extents.width + 2);
+	g_free(str);
+
 	w = d_wday->w + d_mday->w + d_month->w;
 	d_wday->x = (gkrellm_chart_width() - w) / 2;
 	d_mday->x = d_wday->x + d_wday->w;
@@ -689,7 +763,7 @@ static GkrellmMonitor	monitor_clock =
 	};
 
 GkrellmMonitor *
-init_clock_monitor(void)
+gkrellm_init_clock_monitor(void)
 	{
 	clock_enable = TRUE;
     monitor_clock.name = _(monitor_clock.name);
@@ -728,7 +802,7 @@ static GkrellmMonitor	monitor_cal =
 	};
 
 GkrellmMonitor *
-init_cal_monitor(void)
+gkrellm_init_cal_monitor(void)
 	{
 	cal_enable = TRUE;
     monitor_cal.name = _(monitor_cal.name);
