@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -244,9 +244,9 @@ assign_textcolor(GkrellmStyle *style, gchar *arg, gint AorB)
 
 	values = g_strconcat(arg, NULL);
 
-	color = cut_quoted_string(values, &s);
-	shadowcolor = cut_quoted_string(s, &s);
-	effect = cut_quoted_string(s, &s);
+	color = gkrellm_cut_quoted_string(values, &s);
+	shadowcolor = gkrellm_cut_quoted_string(s, &s);
+	effect = gkrellm_cut_quoted_string(s, &s);
 	if (*color == '\0' || *shadowcolor == '\0' || *effect == '\0')
 		{
 		printf(_("Bad textcolor line %s\n"), arg);
@@ -255,9 +255,9 @@ assign_textcolor(GkrellmStyle *style, gchar *arg, gint AorB)
 		}
 	ts = (AorB == GKRELLMSTYLE_TEXTCOLOR_A)
 				? &style->label_tsA : &style->label_tsB;
-	map_color_string(color, &(ts->color));
-	map_color_string(shadowcolor, &(ts->shadow_color));
-	ts->effect = effect_string_value(effect);
+	gkrellm_map_color_string(color, &(ts->color));
+	gkrellm_map_color_string(shadowcolor, &(ts->shadow_color));
+	ts->effect = gkrellm_effect_string_value(effect);
 	g_free(values);
 	}
 
@@ -437,6 +437,8 @@ assign_style_entry(GkrellmStyle *style, gchar *value, gint entry_flag)
 		style->margin.right = atoi(value) * _GK.theme_scale / 100;
 	else if (entry_flag == GKRELLMSTYLE_TRANSPARENCY)
 		style->transparency = atoi(value);
+	else if (entry_flag == GKRELLMSTYLE_SCROLL_TEXT_CACHE_OFF)
+		style->scroll_text_cache_off = parse_boolean(value);
 	else if (entry_flag == GKRELLMSTYLE_TEXTCOLOR_A)
 		assign_textcolor(style, value, GKRELLMSTYLE_TEXTCOLOR_A);
 	else if (entry_flag == GKRELLMSTYLE_TEXTCOLOR_B)
@@ -528,7 +530,7 @@ assign_custom_style(gchar *debug_name, GList *style_list, gint index,
 	GkrellmStyle	*style, *custom_style;
 	gint	i;
 
-	if ((i = string_position_in_list(_GK.custom_name_list, custom_name)) < 0)
+	if ((i = gkrellm_string_position_in_list(_GK.custom_name_list, custom_name)) < 0)
 		{
 		style = (GkrellmStyle *) g_list_nth_data(style_list, index);
 		if (!style)
@@ -576,6 +578,7 @@ static struct string_map
 	{ "alt_font",			GKRELLMSTYLE_TEXTFONT_B },
 	{ "border",				GKRELLMSTYLE_BORDER },
 	{ "transparency",		GKRELLMSTYLE_TRANSPARENCY },
+	{ "scroll_text_cache_off",	GKRELLMSTYLE_SCROLL_TEXT_CACHE_OFF },
 	{ "margin",				OLD_SET_MARGIN },	/* deprecated */
 	};
 
@@ -645,7 +648,7 @@ assign_gkrellmrc_style(gchar *source_line, gchar *area, gchar *string)
 		g_free(custom_name);
 		return;
 		}
-	index = string_position_in_list(name_list, mon_name);
+	index = gkrellm_string_position_in_list(name_list, mon_name);
 	if (index == DEFAULT_STYLE_ID)
 		override = FALSE;
 
@@ -1140,7 +1143,7 @@ assign_gkrellmrc_spacer(gchar *source_line, gchar *area, gchar *string)
 		return;
 	if (n == 1)
 		strcpy(arg, style_name);
-	else if ((mon = lookup_monitor_from_style_name(style_name)) == NULL)
+	else if ((mon = gkrellm_monitor_from_style_name(style_name)) == NULL)
 		return;
 
 	for (s = arg; *s == ' ' || *s == '=' || *s == '\t'; ++s)
@@ -1274,7 +1277,7 @@ destroy_monitor_specific_piximages(void)
 	}
 
 void
-load_theme_piximages(void)
+gkrellm_load_theme_piximages(void)
 	{
 	GkrellmMonitor	*mon;
 	gint			n_base, n_default;
@@ -1332,35 +1335,35 @@ load_theme_piximages(void)
 					n_default, NULL);
 			}
 
-		if ((mon = lookup_monitor_from_style_name("timers")) != NULL)
+		if ((mon = gkrellm_monitor_from_style_name("timers")) != NULL)
 			{
 			gkrellm_load_piximage(NULL, spacer_top_timers_xpm,
 						&mon->privat->top_spacer.piximage, NULL);
 			gkrellm_load_piximage(NULL, spacer_bottom_timers_xpm,
 						&mon->privat->bottom_spacer.piximage, NULL);
 			}
-		if ((mon = lookup_monitor_from_style_name("volume")) != NULL)
+		if ((mon = gkrellm_monitor_from_style_name("volume")) != NULL)
 			{
 			gkrellm_load_piximage(NULL, spacer_top_volume_xpm,
 						&mon->privat->top_spacer.piximage, NULL);
 			gkrellm_load_piximage(NULL, spacer_bottom_volume_xpm,
 						&mon->privat->bottom_spacer.piximage, NULL);
 			}
-		if ((mon = lookup_monitor_from_style_name("gkrellmms")) != NULL)
+		if ((mon = gkrellm_monitor_from_style_name("gkrellmms")) != NULL)
 			{
 			gkrellm_load_piximage(NULL, spacer_top_gkrellmms_xpm,
 						&mon->privat->top_spacer.piximage, NULL);
 			gkrellm_load_piximage(NULL, spacer_bottom_gkrellmms_xpm,
 						&mon->privat->bottom_spacer.piximage, NULL);
 			}
-		if ((mon = lookup_monitor_from_style_name("pmu")) != NULL)
+		if ((mon = gkrellm_monitor_from_style_name("pmu")) != NULL)
 			{
 			gkrellm_load_piximage(NULL, spacer_top_pmu_xpm,
 						&mon->privat->top_spacer.piximage, NULL);
 			gkrellm_load_piximage(NULL, spacer_bottom_pmu_xpm,
 						&mon->privat->bottom_spacer.piximage, NULL);
 			}
-		if ((mon = lookup_monitor_from_style_name(FS_STYLE_NAME)) != NULL)
+		if ((mon = gkrellm_monitor_from_style_name(FS_STYLE_NAME)) != NULL)
 			{
 			gkrellm_load_piximage(NULL, spacer_top_fs_xpm,
 						&mon->privat->top_spacer.piximage, NULL);
@@ -1439,10 +1442,6 @@ static struct	_config
 	{"decal_alarm_frames",	&_GK.decal_alarm_frames,	NULL,			1  },
 	{"decal_warn_frames",	&_GK.decal_warn_frames,	NULL,			1  },
 
-	{"large_font",			NULL,		&_GK.large_font_string,		-100 },
-	{"normal_font",			NULL,		&_GK.normal_font_string,		-100 },
-	{"small_font",			NULL,		&_GK.small_font_string,		-100 },
-
 	{"chart_in_color",		NULL,		&_GK.chart_in_color,		-100 },
 	{"chart_in_color_grid",	NULL,		&_GK.chart_in_color_grid,	-100 },
 	{"chart_out_color",		NULL,		&_GK.chart_out_color,		-100 },
@@ -1474,12 +1473,6 @@ static struct	_config
 static void
 cleanup_gkrellmrc(void)
 	{
-	if (_GK.font_override)
-		{
-		gkrellm_dup_string(&_GK.large_font_string, get_large_font_override());
-		gkrellm_dup_string(&_GK.normal_font_string,get_normal_font_override());
-		gkrellm_dup_string(&_GK.small_font_string, get_small_font_override());
-		}
 	set_border(&_GK.frame_top_border, frame_top_border);
 	set_border(&_GK.frame_bottom_border, frame_bottom_border);
 	set_border(&_GK.frame_left_border, frame_left_border);
@@ -1612,7 +1605,7 @@ gkrellm_get_gkrellmrc_string(gchar *string_name)
 		sscanf(s, "%63s %[^\n]", name, string);
 		if (!strcmp(name, string_name))
 			{
-			if ((s = cut_quoted_string(string, NULL)) != NULL)
+			if ((s = gkrellm_cut_quoted_string(string, NULL)) != NULL)
 				return g_strdup(s);
 			break;
 			}
@@ -1676,13 +1669,13 @@ parse_gkrellmrc_keyword(gchar *line)
 		assign_gkrellmrc_spacer(line, s, arg);
 	else if (!strcmp(s, "cap_images_off"))
 		{
-		mon = lookup_monitor_from_style_name(arg);
+		mon = gkrellm_monitor_from_style_name(arg);
 		if (mon)
 			mon->privat->cap_images_off = TRUE;
 		}
 	else if (!strcmp(s, "spacer_overlap_off"))
 		{
-		mon = lookup_monitor_from_style_name(arg);
+		mon = gkrellm_monitor_from_style_name(arg);
 		if (mon)
 			mon->privat->spacer_overlap_off = TRUE;
 		}
@@ -1832,15 +1825,8 @@ static gchar	*base_theme[]	=
 	};
 
 
-static gchar	*default_fonts[] =
-	{
-	"-b&h-lucida-bold-r-normal-*-*-120-*-*-*-*-iso8859-*",		/* large  */
-	"-b&h-lucida-medium-r-normal-*-*-100-*-*-*-*-iso8859-*",	/* normal */
-	"-adobe-helvetica-medium-r-normal-*-*-80-*-*-*-*-iso8859-*"	/* small  */
-	};
-
 static void
-init_theme(void)
+gkrellm_init_theme(void)
 	{
 	GkrellmMonitor	*mon;
 	GList			*list;
@@ -1882,9 +1868,12 @@ init_theme(void)
 	_GK.tx_led_x = -3;
 	_GK.tx_led_y = 6;
 
-	gkrellm_dup_string(&_GK.large_font_string, default_fonts[0]);
-	gkrellm_dup_string(&_GK.normal_font_string, default_fonts[1]);
-	gkrellm_dup_string(&_GK.small_font_string, default_fonts[2]);
+	gkrellm_dup_string(&_GK.large_font_string,
+				gkrellm_get_large_font_string());
+	gkrellm_dup_string(&_GK.normal_font_string,
+				gkrellm_get_normal_font_string());
+	gkrellm_dup_string(&_GK.small_font_string,
+				gkrellm_get_small_font_string());
 
 	gkrellm_dup_string(&_GK.chart_in_color, "#10d3d3");
 	gkrellm_dup_string(&_GK.chart_in_color_grid, "#00a3a3");
@@ -1899,8 +1888,8 @@ init_theme(void)
 	clear_style_list(_GK.chart_style_list, _GK.chart_name_list);
 	clear_style_list(_GK.panel_style_list, _GK.chart_name_list);
 	clear_style_list(_GK.meter_style_list, _GK.meter_name_list);
-	free_glist_and_data(&_GK.custom_name_list);
-	free_glist_and_data(&_GK.custom_style_list);
+	gkrellm_free_glist_and_data(&_GK.custom_name_list);
+	gkrellm_free_glist_and_data(&_GK.custom_style_list);
 
 	for (i = 0; i < sizeof(base_theme) / sizeof(gchar *); ++i)
 		parse_gkrellmrc_line(base_theme[i]);
@@ -1955,9 +1944,9 @@ init_theme(void)
 	krell_slider_depth = 6;
 	gkrellm_dup_string(&krell_slider_expand, "none");
 
-	free_glist_and_data(&gkrellmrc_border_list);
-	free_glist_and_data(&gkrellmrc_integer_list);
-	free_glist_and_data(&gkrellmrc_string_list);
+	gkrellm_free_glist_and_data(&gkrellmrc_border_list);
+	gkrellm_free_glist_and_data(&gkrellmrc_integer_list);
+	gkrellm_free_glist_and_data(&gkrellmrc_string_list);
 	for (list = gkrellm_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmMonitor *) list->data;
@@ -1998,7 +1987,7 @@ static gchar	*default_theme[]	=
 	"StyleMeter sensors.alt_textcolor = #c8e4e2 #000000 shadow",
 
 	"StyleMeter fs.border = 4,4,3,1",
-	"StyleMeter fs.bottom_margin = 0",
+	"StyleMeter fs.bottom_margin = 1",
 	"StyleMeter fs.label_position = 0",
 	"StyleMeter fs.alt_font = normal_font",
 
@@ -2047,7 +2036,7 @@ static gchar	*default_theme[]	=
 	"spacer_top_height gkrellmms 3",
 	"spacer_bottom_height gkrellmms 3",
 	"set_image_border gkrellmms_bg_scroll 3,3,2,2",
-	"set_integer gkrellmms_scroll_margin 2",
+	"set_integer gkrellmms_scroll_margin 3",
 	"set_integer gkrellmms_scroll_top_margin 2",
 	"set_integer gkrellmms_scroll_bottom_margin 1",
 	/* GKrellMMS control (button) bar panel*/
@@ -2067,7 +2056,7 @@ static gchar	*default_theme[]	=
 
 	"set_string gkrellmms_eject_button_position \"17 12 0 0 c\"",
 	"set_string gkrellmms_led_position \"7 7 c\"",
-	"set_string gkrellmms_label_position \"-25 15 c\"",
+	"set_string gkrellmms_label_position \"-25 7 c\"",
 
 
 
@@ -2203,10 +2192,10 @@ gkrellm_using_default_theme(void)
 	}
 
 void
-gkrellmrc_theme_config(void)
+gkrellm_theme_config(void)
 	{
-	init_theme();
-	load_theme_config();
+	gkrellm_load_theme_config();
+	gkrellm_init_theme();
 
 	if (gkrellm_using_default_theme())
 		default_theme_config();
@@ -2219,7 +2208,7 @@ gkrellmrc_theme_config(void)
 			parse_gkrellmrc(_GK.theme_alternative);
 		}
 	cleanup_gkrellmrc();
-	set_theme_alternatives_label();
+	gkrellm_set_theme_alternatives_label();
 
 	/* Warn theme developers!
 	*/
@@ -2265,7 +2254,7 @@ struct	_config	user_config[] =
   |  config values.
   */
 void
-load_user_config(gboolean monitor_values)
+gkrellm_load_user_config(gboolean monitor_values)
 	{
 	FILE	*f;
 	gchar	*s, *ss, *config, *config_2_1_14 = NULL;
@@ -2292,14 +2281,6 @@ load_user_config(gboolean monitor_values)
 					GKRELLM_2_1_14_CONFIG);
 		f = fopen(config_2_1_14, "r");
 		}
-	if (!f)		/* Upgrade from 1.2 to 2.0 ? */
-		{
-		config = gkrellm_make_config_file_name(gkrellm_homedir(),
-					".gkrellm/user_config");
-		f = fopen(config, "r");
-		g_free(config_2_1_14);
-		config_2_1_14 = NULL;
-		}
 	if (f)
 		{
 		while (fgets(buf, sizeof(buf), f))
@@ -2346,14 +2327,14 @@ gkrellm_make_data_file_name(gchar *subdir, gchar *name)
 	gchar	*dir, *path, *s;
 
 	dir = gkrellm_make_config_file_name(NULL, GKRELLM_DATA_DIR);
-	make_home_subdir(dir, &path);
+	gkrellm_make_home_subdir(dir, &path);
 	if (subdir)
 		{
 		g_free(path);
 		s = g_strconcat(dir, G_DIR_SEPARATOR_S, subdir, NULL);
 		g_free(dir);
 		dir = s;
-		make_home_subdir(dir, &path);
+		gkrellm_make_home_subdir(dir, &path);
 		}
 	g_free(dir);
 	if (name)
@@ -2407,7 +2388,7 @@ gkrellm_make_config_file_name(gchar *dir, gchar *config)
 	}
 
 void
-save_user_config(void)
+gkrellm_save_user_config(void)
 	{
 	FILE	*f;
 	GList	*list;
