@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -55,7 +55,8 @@ typedef struct
 	GkrellmDecal	*sensor_decal,		/* temperature and possibly fan */
 					*fan_decal;			/* fan if separate draw mode	*/
 	gboolean		show_temperature,
-					show_fan;
+					show_fan,
+					new_text_format;
 	gint			save_label_position;  /* When toggling sensors on/off */
 
 	GkrellmAlert	*alert;
@@ -100,7 +101,7 @@ static gchar    *text_format,
 				*text_format_locale;
 
 static gint		n_cpus;
-gint			n_smp_cpus;
+static gint		n_smp_cpus;
 
 
 static gboolean
@@ -337,6 +338,9 @@ draw_cpu_extra(CpuMon *cpu)
 	if (!cp)
 		return;
 	format_cpu_data(cpu, text_format_locale, buf, sizeof(buf));
+	if (!cpu->new_text_format)
+		gkrellm_chart_reuse_text_format(cp);
+	cpu->new_text_format = FALSE;
 	gkrellm_draw_chart_text(cp, style_id, buf);
 	}
 
@@ -755,8 +759,8 @@ load_nice_data_piximages(void)
 	nice_color_string = gkrellm_get_gkrellmrc_string("cpu_nice_color");
 	nice_grid_color_string =
 				gkrellm_get_gkrellmrc_string("cpu_nice_grid_color");
-	map_color_string(nice_color_string, &nice_color);
-	map_color_string(nice_grid_color_string, &nice_grid_color);
+	gkrellm_map_color_string(nice_color_string, &nice_color);
+	gkrellm_map_color_string(nice_grid_color_string, &nice_grid_color);
 		
 	if (nice_data_piximage)
 		gkrellm_destroy_piximage(nice_data_piximage);
@@ -871,6 +875,7 @@ create_cpu(GtkWidget *vbox, gint first_create)
 
 		gkrellm_alloc_chartdata(cp);
 		enable_cpu_visibility(cpu);
+		cpu->new_text_format = TRUE;
 
 		if (first_create)
 			{
@@ -1162,12 +1167,17 @@ static void
 cb_text_format(GtkWidget *widget, gpointer data)
 	{
 	GList	*list;
+	CpuMon	*cpu;
 	gchar	*s;
 
 	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	for (list = cpu_mon_list; list; list = list->next)
-		refresh_cpu_chart((CpuMon *) list->data);
+		{
+		cpu = (CpuMon *) list->data;
+		cpu->new_text_format = TRUE;
+		refresh_cpu_chart(cpu);
+		}
 	}
 
 static void
@@ -1329,6 +1339,7 @@ create_cpu_tab(GtkWidget *tab_vbox)
 		list = g_list_append(list,
 			"\\ww\\C\\f$L\\D5\\f\\an\\.$n\\D2\\f\\au\\.$u\\D1\\f\\as\\.$s");
 	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
 			text_format);
@@ -1414,7 +1425,7 @@ static GkrellmMonitor	monitor_cpu =
 	};
 
 GkrellmMonitor *
-init_cpu_monitor(void)
+gkrellm_init_cpu_monitor(void)
 	{
 	GList	*list;
 
