@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -47,7 +47,12 @@ typedef struct
 	GtkWidget		*alert_config_read_button,
 					*alert_config_write_button;
 	gboolean		alert_uses_read,
-					alert_uses_write;
+					alert_uses_write,
+					new_text_format;
+
+	GkrellmDecal	*temp_decal;
+	gint			save_label_position;
+
 	gint			enabled;
 	gint			major,
 					minor,
@@ -391,7 +396,7 @@ format_disk_data(DiskMon *disk, gchar *src_string, gchar *buf, gint size)
 					}
 				}
 			if (blocks >= 0)
-				len = format_size_abbrev(buf, size, (gfloat) blocks,
+				len = gkrellm_format_size_abbrev(buf, size, (gfloat) blocks,
 						&disk_blocks_abbrev[0], tbl_size);
 			++s;
 			}
@@ -411,6 +416,9 @@ draw_disk_extra(DiskMon *disk)
 	if (!disk->extra_info)
 		return;
 	format_disk_data(disk, text_format_locale, buf, sizeof(buf));
+	if (!disk->new_text_format)
+		gkrellm_chart_reuse_text_format(disk->chart);
+	disk->new_text_format = FALSE;
 	gkrellm_draw_chart_text(disk->chart, style_id, buf);
 	}
 
@@ -429,6 +437,127 @@ draw_disk_chart(DiskMon *disk)
 	gkrellm_draw_chart_to_screen(disk->chart);
 	}
 
+static void
+cb_disk_temp_alert_trigger(GkrellmAlert *alert, DiskMon *disk)
+    {
+    GkrellmAlertdecal   *ad;
+    GkrellmDecal        *d;
+
+    if (alert && disk && disk->chart)
+        {
+        ad = &alert->ad;
+        d = disk->temp_decal;
+        if (d)
+            {
+            ad->x = d->x - 1;
+            ad->y = d->y - 1;
+            ad->w = d->w + 2;
+            ad->h = d->h + 2;
+            gkrellm_render_default_alert_decal(alert);
+            }
+        alert->panel = disk->chart->panel;
+        }
+    }
+
+gboolean
+gkrellm_disk_temperature_display(gpointer sr, gchar *id_name, gfloat t,
+			gchar units)
+	{
+	GList				*list;
+	DiskMon				*disk;
+	GkrellmPanel		*p;
+	GkrellmDecal		*decal;
+	gchar				*disk_name;
+	gint				len;
+	gboolean			display_done = FALSE, display_possible = FALSE;
+
+	if ((disk_name = strrchr(id_name, '/')) != NULL)
+		++disk_name;
+	else
+		disk_name = id_name;
+
+	len = strlen(disk_name);
+	for (list = disk_mon_list; list; list = list->next)
+		{
+		disk = (DiskMon *) list->data;
+		if (   strncmp(disk->name, disk_name, len)
+			|| (disk->name[len] != '\0' && !isdigit(disk->name[len]))
+		   )
+			continue;
+		if (!disk->enabled || !disk->chart)
+			{
+			display_possible |= disk->enabled;	/* Not created yet */
+			continue;
+			}
+		decal = disk->temp_decal;
+		p = disk->chart->panel;
+		if (display_done)
+			{
+			if (gkrellm_is_decal_visible(decal))
+				{
+				gkrellm_make_decal_invisible(p, decal);
+				p->label->position = disk->save_label_position;
+				gkrellm_draw_panel_label(p);
+				gkrellm_draw_panel_layers(p);
+				}
+			continue;
+			}
+		if (!gkrellm_is_decal_visible(decal))
+			{
+			gkrellm_make_decal_visible(p, decal);
+			disk->save_label_position = p->label->position;
+			if (p->label->position >= 0)
+				{
+				p->label->position = 0;
+				gkrellm_draw_panel_label(p);
+				}
+			gkrellm_sensor_alert_connect(sr, cb_disk_temp_alert_trigger, disk);
+			}
+		gkrellm_sensor_draw_temperature_decal(p, decal, t, units);
+		gkrellm_draw_panel_layers(p);
+		display_done = TRUE;
+		}
+	return (display_possible || display_done);
+	}
+
+void
+gkrellm_disk_temperature_remove(gchar *id_name)
+	{
+	GList			*list;
+	DiskMon			*disk;
+	GkrellmPanel	*p;
+	GkrellmDecal	*decal;
+	gchar			*disk_name;
+	gint			len;
+
+	if ((disk_name = strrchr(id_name, '/')) != NULL)
+		++disk_name;
+	else
+		disk_name = id_name;
+
+	len = strlen(disk_name);
+	for (list = disk_mon_list; list; list = list->next)
+		{
+		disk = (DiskMon *) list->data;
+		if (!disk->chart)
+			continue;
+		if (   strncmp(disk->name, disk_name, len)
+			|| (disk->name[len] != '\0' && !isdigit(disk->name[len]))
+		   )
+			continue;
+		p = disk->chart->panel;
+		decal = disk->temp_decal;
+		if (gkrellm_is_decal_visible(decal))
+			{
+			gkrellm_make_decal_invisible(p, decal);
+			p->label->position = disk->save_label_position;
+			gkrellm_draw_panel_label(p);
+			gkrellm_draw_panel_layers(p);
+			}
+		}
+	}
+
+
 static void
 update_disk(void)
 	{
@@ -573,9 +702,11 @@ cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
 static void
 create_disk_monitor(DiskMon *disk, gint first_create)
 	{
-	GkrellmPanel	*p;
-	GkrellmStyle	*style;
-	GkrellmChart	*cp;
+	GkrellmChart		*cp;
+	GkrellmPanel		*p;
+	GkrellmStyle		*style;
+	GkrellmTextstyle	*ts;
+	GkrellmMargin		*m;
 
 	if (first_create)
 		{
@@ -608,9 +739,18 @@ create_disk_monitor(DiskMon *disk, gint first_create)
 
 	setup_disk_scaling(cp->config, cp);
 
+	ts = gkrellm_panel_alt_textstyle(style_id);
+	m = gkrellm_get_style_margins(style);
+	disk->temp_decal = gkrellm_create_decal_text(p, "188.8F",
+					ts, style, -1, -1, 0);
+	disk->temp_decal->x =
+				gkrellm_chart_width() - m->right - disk->temp_decal->w;
+	gkrellm_make_decal_invisible(p, disk->temp_decal);
+
 	gkrellm_panel_configure(p, disk->label, style);
 	gkrellm_panel_create(disk->vbox, mon_disk, p);
 	disk->enabled = TRUE;
+	disk->new_text_format = TRUE;
 
 	if (first_create)
 		{
@@ -1144,12 +1284,17 @@ static void
 cb_text_format(GtkWidget *widget, gpointer data)
 	{
 	GList	*list;
+	DiskMon	*disk;
 	gchar	*s;
 
 	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	for (list = disk_mon_list; list; list = list->next)
-		draw_disk_chart((DiskMon *) list->data);
+		{
+		disk = (DiskMon *) list->data;
+		disk->new_text_format = TRUE;
+		draw_disk_chart(disk);
+		}
 	}
 
 
@@ -1251,6 +1396,7 @@ create_disk_tab(GtkWidget *tab_vbox)
 	list = g_list_append(list,
 					_("\\f\\ww\\r\\f$M\\D2\\f\\ar\\. $r\\D1\\f\\aw\\. $w"));
 	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
 			text_format);
@@ -1302,7 +1448,7 @@ static GkrellmMonitor	monitor_disk =
 	};
 
 GkrellmMonitor *
-init_disk_monitor(void)
+gkrellm_init_disk_monitor(void)
 	{
 	DiskMon	*disk;
 
