@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -80,7 +80,7 @@ typedef struct
 	gulong		krell_factor;		/* avoid krell math overflow */
 
 	gboolean	busy;
-	gulong		blocks,
+	glong		blocks,
 				bfree,
 				bavail,
 				bsize;
@@ -214,7 +214,7 @@ gkrellm_fs_add_to_fstab_list(gchar *dir, gchar *dev, gchar *type, gchar *opt)
 
 void
 gkrellm_fs_assign_fsusage_data(gpointer fspointer,
-			gulong blocks, gulong bavail, gulong bfree, gulong bsize)
+			glong blocks, glong bavail, glong bfree, glong bsize)
 	{
 	FSmon	*fs = (FSmon *) fspointer;
 
@@ -336,11 +336,11 @@ static GkrellmSizeAbbrev	fs_binary_abbrev[] =
 static gint
 format_fs_data(FSmon *fs, gchar *src_string, gchar *buf, gint size)
 	{
-	gulong		b, u, f;
+	glong		b, u, a;
 	gint		len;
 	gchar		*s;
-	gchar		tbuf[32], ubuf[32], fbuf[32];
-	gfloat		bsize;
+	gchar		tbuf[32], ubuf[32], abuf[32];
+	gfloat		bsize, val;
 	GkrellmSizeAbbrev *tbl;
 	size_t		tbl_size;
 
@@ -350,9 +350,10 @@ format_fs_data(FSmon *fs, gchar *src_string, gchar *buf, gint size)
 	*buf = '\0';
 	if (!src_string)
 		return -1;
+
 	b = fs->blocks;
-	u = fs->blocks - fs->bavail;
-	f = fs->bavail;
+	u = fs->blocks - fs->bfree;
+	a = fs->bavail;					/* Can be negative on BSD	*/
 	bsize = (gfloat) fs->bsize;
 
 	tbl = binary_units ? &fs_binary_abbrev[0] : &fs_decimal_abbrev[0];
@@ -360,11 +361,10 @@ format_fs_data(FSmon *fs, gchar *src_string, gchar *buf, gint size)
 			? (sizeof(fs_binary_abbrev) / sizeof(GkrellmSizeAbbrev))
 			: (sizeof(fs_decimal_abbrev) / sizeof(GkrellmSizeAbbrev));
 
-	format_size_abbrev(tbuf, sizeof(tbuf), (gfloat)b * bsize, tbl, tbl_size);
-	format_size_abbrev(ubuf, sizeof(ubuf), (gfloat)u * bsize, tbl, tbl_size);
-	format_size_abbrev(fbuf, sizeof(fbuf), (gfloat)f * bsize, tbl, tbl_size);
+	gkrellm_format_size_abbrev(tbuf, sizeof(tbuf), (gfloat) b * bsize, tbl, tbl_size);
+	gkrellm_format_size_abbrev(ubuf, sizeof(ubuf), (gfloat) u * bsize, tbl, tbl_size);
+	gkrellm_format_size_abbrev(abuf, sizeof(abuf), (gfloat) a * bsize, tbl, tbl_size);
 
-	f = b - u;
 	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
@@ -387,17 +387,21 @@ format_fs_data(FSmon *fs, gchar *src_string, gchar *buf, gint size)
 					len = snprintf(buf, size, "%s", ubuf);
 					break;
 				case 'U':
-					if (b > 0)
-						len = snprintf(buf, size, "%ld%%",
-							100 * (u >> 8) / (b >> 8));
+					if (u + a > 0)
+						val = 100.0 * (gfloat) u / (gfloat) (u + a);
+					else
+						val = 0;
+					len = snprintf(buf, size, "%.0f%%", val);
 					break;
 				case 'f':
-					len = snprintf(buf, size, "%s", fbuf);
+					len = snprintf(buf, size, "%s", abuf);
 					break;
 				case 'F':
-					if (b > 0)
-						len = snprintf(buf, size, "%ld%%",
-							100 * (f >> 8) / (b >> 8));
+					if (u + a > 0)
+						val = 100.0 * (gfloat) a / (gfloat) (u + a);
+					else
+						val = 0;
+					len = snprintf(buf, size, "%.0f%%", val);
 					break;
 				case 'H':
 					len = snprintf(buf, size, "%s",
@@ -428,17 +432,15 @@ format_fs_data(FSmon *fs, gchar *src_string, gchar *buf, gint size)
 static gint
 draw_fs_text_decal(FSmon *fs, gint value)
 	{
-	GkrellmDecal	*d;
-	GkrellmTextstyle ts_save;
-	gchar			buf[128];
-	gint			changed;
-	gint			w	= 0;
+	GkrellmDecal		*d;
+	GkrellmTextstyle	ts_save;
+	gchar				buf[128];
+	gint				x_off, w;
 
 	d = fs->text_decal;
-
 	if (value == 0)
 		{
-		d->x_off = 0;
+		gkrellm_decal_text_set_offset(d, 0, 0);
 		gkrellm_draw_decal_text(fs->panel, d, fs->label_shadow, 0);
 		}
 	else if (!fs->busy)
@@ -446,21 +448,15 @@ draw_fs_text_decal(FSmon *fs, gint value)
 		ts_save = d->text_style;
 		d->text_style = *gkrellm_meter_alt_textstyle(style_id);
 
-		changed = format_fs_data(fs, data_format_locale, buf, sizeof(buf));
-		w = gkrellm_gdk_string_width(d->text_style.font, buf);
-		if (w  > d->w)
-			{
-			d->x_off = d->w / 3 - x_scroll;
-			changed = x_scroll + 1;
-			}
+		format_fs_data(fs, data_format_locale, buf, sizeof(buf));
+		gkrellm_decal_scroll_text_set_markup(fs->panel, d, buf);
+		gkrellm_decal_scroll_text_get_size(d, &w, NULL);
+		if (w > d->w)
+			x_off = d->w / 3 - x_scroll;
 		else
-			d->x_off = 0;
+			x_off = 0;
+		gkrellm_decal_text_set_offset(d, x_off, 0);
 
-		/* Draw the decal, actual draws occur only if "changed" is different
-		|  from last call.  If scrolling, will be so each time, if not
-		|  will be so as format_fs_data() return value changes.
-		*/
-		gkrellm_draw_decal_text(fs->panel, d, buf, changed);
 		d->text_style = ts_save;
 		}
 	return w;
@@ -689,7 +685,7 @@ update_fs(void)
 	GkrellmPanel	*p;
 	GkrellmKrell	*k;
 	GList			*list;
-	gulong			l, used;
+	glong			used, avail;
 	gint			full_scale, index, w_scroll, w;
 	gboolean		fs_check, nfs_check, force_check, force_draw,
 					mounting_enabled;
@@ -774,20 +770,24 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 					g_thread_create(get_fsusage_thread, fs, FALSE, NULL);
 					}
 				fs->krell_factor = fs->blocks > 2097152 ? 1024 : 1;
-				}			
-			full_scale = (gint) (fs->blocks / fs->krell_factor);
-			gkrellm_set_krell_full_scale(k, full_scale, 1);
+				}
 
-			used = l = (fs->blocks - fs->bavail) / fs->krell_factor;
-			if (   (fs_in_motion && fs->label_is_data && x_moved)
-				|| fs->mouse_entered
-			   )
-				l = 0;
+			avail = fs->bavail >= 0 ? fs->bavail : 0;
+			used = fs->blocks - fs->bfree;
+			full_scale = (gint) (used + avail) / fs->krell_factor;
+			used /= fs->krell_factor;
+			gkrellm_set_krell_full_scale(k, full_scale, 1);
 
 			if (!fs->busy)
 				{
-				gkrellm_update_krell(p, k, l);
-				gkrellm_check_alert(fs->alert,
+				if (   (fs_in_motion && fs->label_is_data && x_moved)
+					|| fs->mouse_entered
+				   )
+					gkrellm_update_krell(p, k, 0);
+				else
+					gkrellm_update_krell(p, k, used);
+				if (full_scale > 0)
+					gkrellm_check_alert(fs->alert,
 							100.0 * (gfloat) used / (gfloat) full_scale);
 				}
 			else
@@ -1004,7 +1004,7 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 	GList			*list;
 	FSmon			*fs;
 	GkrellmDecal	*d;
-	GdkFont			*font;
+	PangoFontDescription *font_desc;
 	gchar			buf[128];
 	gint			w, x_delta	= 0;
 
@@ -1017,11 +1017,11 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 		fs_in_motion = NULL;
 		return FALSE;
 		}
-	font = gkrellm_meter_alt_textstyle(style_id)->font;
+	font_desc = gkrellm_meter_alt_textstyle(style_id)->font;
 
 	d = fs_in_motion->text_decal;
 	format_fs_data(fs_in_motion, data_format_locale, buf, sizeof(buf));
-	w = gkrellm_gdk_string_width(font, buf);
+	gkrellm_decal_scroll_text_get_size(d, &w, NULL);
 	if (w > d->w)
 		{
 		x_delta = ev->x - x_fs_motion;
@@ -1396,7 +1396,7 @@ load_fs_config(gchar *arg)
 		}
 	else
 		{
-		if ((label = cut_quoted_string(arg, &arg)) != NULL)
+		if ((label = gkrellm_cut_quoted_string(arg, &arg)) != NULL)
 			{
 			fs = g_new0(FSmon, 1);
 			gkrellm_locale_dup_string(&fs->label, label, &fs->label_shadow);
@@ -2051,6 +2051,16 @@ cb_data_format(GtkWidget *widget, gpointer data)
 	gchar	*s;
 
 	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(data_format_combo)->entry));
+
+	/* In case Pango markup tags, don't accept line unless valid markup.
+	|  Ie, markup like <span ...> xxx </span> or <b> xxx </b>
+	*/
+
+	if (   strchr(s, '<') != NULL
+		&& !pango_parse_markup(s, -1, 0, NULL, NULL, NULL, NULL)
+	   )
+		return;
+
 	if (gkrellm_locale_dup_string(&data_format, s, &data_format_locale))
 		for (list = fs_mon_list; list; list = list->next)
 			((FSmon *) list->data)->text_decal->value = -1;	/* Force redraw */
@@ -2186,6 +2196,7 @@ create_fs_panels_page(GtkWidget *vbox)
 		combo_list = g_list_append(combo_list,
 					((Mount *)list->data)->directory);
 	gtk_combo_set_popdown_strings( GTK_COMBO(dir_combo), combo_list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(dir_combo), TRUE);
 	g_list_free(combo_list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dir_combo)->entry), "");
 	g_signal_connect(G_OBJECT(GTK_COMBO(dir_combo)->entry),
@@ -2452,20 +2463,21 @@ create_fs_tab(GtkWidget *tab_vbox)
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 2);
 	data_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(hbox), data_format_combo, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), data_format_combo, TRUE, TRUE, 0);
 	list = NULL;
 	list = g_list_append(list, data_format);
 	list = g_list_append(list, DEFAULT_DATA_FORMAT);
 	list = g_list_append(list, _("$t - $u used"));
 	list = g_list_append(list, _("$t - $U"));
 	gtk_combo_set_popdown_strings(GTK_COMBO(data_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(data_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(data_format_combo)->entry),
 			data_format);
 	g_signal_connect(G_OBJECT(GTK_COMBO(data_format_combo)->entry), "changed",
 			G_CALLBACK(cb_data_format), NULL);
 	label = gtk_label_new(_("Format string for FS data display"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 2);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
 
 /* --Info tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
@@ -2504,7 +2516,7 @@ static GkrellmMonitor	monitor_fs =
 	};
 
 GkrellmMonitor *
-init_fs_monitor(void)
+gkrellm_init_fs_monitor(void)
 	{
 	monitor_fs.name = _(monitor_fs.name);
 	style_id = gkrellm_add_meter_style(&monitor_fs, FS_STYLE_NAME);
@@ -2520,4 +2532,3 @@ init_fs_monitor(void)
 		}
 	return NULL;
 	}
-
