@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -30,6 +30,8 @@
 #define	DEBUG_INET		0x800
 #define	DEBUG_CLIENT	0x1000
 #define DEBUG_GUI		0x2000
+#define	DEBUG_POSITION	0x4000
+#define	DEBUG_BATTERY	0x8000
 
 #define	GKRELLM_FLOAT_FACTOR	1000.0
 
@@ -46,6 +48,10 @@ enum GkrellmState
 	LOAD_CONFIG
 	};
 
+
+typedef	void (*GkrellmFunc)();
+
+
 /* Private global extern declarations and function prototypes.
 */
 
@@ -78,6 +84,7 @@ struct	GkrellmConfig
 				h_display,
 				y_position,
 				x_position;
+	gboolean	position_valid;
 	
 	gchar		*theme_path;
 	gchar		*config_suffix;		/* Overrides host_configs below */
@@ -235,7 +242,8 @@ struct	GkrellmConfig
 	GdkGC		*bit1_GC,		/* Depth 1 GCs		*/
 				*bit0_GC;
 
-	GdkFont		*large_font,
+	PangoFontDescription
+				*large_font,
 				*normal_font,
 				*small_font;
 
@@ -280,8 +288,6 @@ struct	GkrellmConfig
 	gint		theme_n_alternatives;
 	gint		theme_alternative;
 	gint		theme_scale;
-	gboolean	font_override,
-				use_fontset;
 	gint		m2;
 	gint		use_top_bottom_margins;
 
@@ -303,62 +309,52 @@ struct	GkrellmConfig
 extern struct GkrellmConfig  _GK;
 
 extern struct tm	gkrellm_current_tm;
-
-extern gint			n_smp_cpus;
 extern gint			gkrellm_w_display,
 					gkrellm_h_display,
 					gkrellm_y_position;
-extern gint			window_decorations;
-extern gint			n_system_plugins;
 
-extern GkrellmBorder zero_border;
-
-void load_plugin_monitors(void);
+void	gkrellm_plugins_load(void);
 
 void	gkrellm_record_state(enum GkrellmState state, GkrellmMonitor *mon);
-void	create_plugin_config(GtkWidget *);
-void	close_plugins_config(void);
+void	gkrellm_plugins_config_create(GtkWidget *);
+void	gkrellm_plugins_config_close(void);
 void	gkrellm_menu_popup(void);
 
-GkrellmMonitor *init_host_monitor(void);
-GkrellmMonitor *init_cal_monitor(void);
-GkrellmMonitor *init_clock_monitor(void);
-GkrellmMonitor *init_cpu_monitor(void);
-GkrellmMonitor *init_proc_monitor(void);
-GkrellmMonitor *init_sensor_monitor(void);
-GkrellmMonitor *init_disk_monitor(void);
-GkrellmMonitor *init_inet_monitor(void);
-GkrellmMonitor *init_net_monitor(void);
-GkrellmMonitor *init_timer_monitor(void);
-GkrellmMonitor *init_mem_monitor(void);
-GkrellmMonitor *init_swap_monitor(void);
-GkrellmMonitor *init_fs_monitor(void);
-GkrellmMonitor *init_mail_monitor(void);
-GkrellmMonitor *init_battery_monitor(void);
-GkrellmMonitor *init_uptime_monitor(void);
-GkrellmMonitor *init_sensors_config_monitor(void);
+GkrellmMonitor *gkrellm_init_host_monitor(void);
+GkrellmMonitor *gkrellm_init_cal_monitor(void);
+GkrellmMonitor *gkrellm_init_clock_monitor(void);
+GkrellmMonitor *gkrellm_init_cpu_monitor(void);
+GkrellmMonitor *gkrellm_init_proc_monitor(void);
+GkrellmMonitor *gkrellm_init_sensor_monitor(void);
+GkrellmMonitor *gkrellm_init_disk_monitor(void);
+GkrellmMonitor *gkrellm_init_inet_monitor(void);
+GkrellmMonitor *gkrellm_init_net_monitor(void);
+GkrellmMonitor *gkrellm_init_timer_monitor(void);
+GkrellmMonitor *gkrellm_init_mem_monitor(void);
+GkrellmMonitor *gkrellm_init_swap_monitor(void);
+GkrellmMonitor *gkrellm_init_fs_monitor(void);
+GkrellmMonitor *gkrellm_init_mail_monitor(void);
+GkrellmMonitor *gkrellm_init_battery_monitor(void);
+GkrellmMonitor *gkrellm_init_uptime_monitor(void);
+GkrellmMonitor *gkrellm_init_sensors_config_monitor(void);
 
 GkrellmMonitor *gkrellm_get_cpu_mon(void);
 GkrellmMonitor *gkrellm_get_proc_mon(void);
 GkrellmMonitor *gkrellm_get_sensors_mon(void);
 
-void	init_hostname_monitor(void);	/* XXX */
+void	gkrellm_init_hostname_monitor(void);	/* XXX */
 GkrellmMonitor	*gkrellm_mon_host(void);
-void			cb_gkrellmd_disconnect_alert(GtkWidget *b, gpointer data);
+void			gkrellm_gkrellmd_disconnect_cb(GtkWidget *b, gpointer data);
 
 GList	*gkrellm_get_chart_list(void);
 GList	*gkrellm_get_panel_list(void);
 
-void		update_alerts(void);
-void		reset_all_alerts(void);
+void		gkrellm_alert_update(void);
+void		gkrellm_alert_reset_all(void);
 GdkPixbuf	*gkrellm_alert_pixbuf(void);
 
-GkrellmMonitor	*lookup_monitor_from_style_name(gchar *);
-GkrellmMonitor	*lookup_monitor_from_id(gint);
-
-#if defined(__linux__)
-void	read_stat_swap(gulong *, gulong *);
-#endif
+GkrellmMonitor	*gkrellm_monitor_from_style_name(gchar *);
+GkrellmMonitor	*gkrellm_monitor_from_id(gint);
 
 gboolean gkrellm_render_spacer(GkrellmSpacer *spacer, gint y_src, gint h_src,
 			gint l_overlap, gint r_overlap);
@@ -367,56 +363,41 @@ void	gkrellm_spacers_hide(GkrellmMonitor *);
 void	gkrellm_spacers_show(GkrellmMonitor *);
 
 void	gkrellm_panel_button_signals_connect(GkrellmPanel *p);
-void	clean_out_panels(void);
-void	destroy_decal_button_list(void);
+void	gkrellm_panel_cleanup(void);
 
-void	insert_spacer(GtkWidget *);
+void	gkrellm_chart_setup(void);
 
-void	create_chart_item_factory(void);
-void	setup_charts(void);
-
-void	build_gkrellm(void);
-void	gkrellmrc_theme_config(void);
-void	load_user_config(gboolean);
-void	save_user_config(void);
-void	save_theme_config(void);
-void	load_theme_config(void);
-void	load_theme_piximages(void);
+void	gkrellm_build(void);
+void	gkrellm_theme_config(void);
+void	gkrellm_load_user_config(gboolean);
+void	gkrellm_save_user_config(void);
+void	gkrellm_save_theme_config(void);
+void	gkrellm_load_theme_config(void);
+void	gkrellm_load_theme_piximages(void);
 void	gkrellm_read_theme_event(GtkSettings *settings);
-void	make_themes_list(void);
-gchar	*get_large_font_override(void);
-gchar	*get_normal_font_override(void);
-gchar	*get_small_font_override(void);
+void	gkrellm_make_themes_list(void);
+gchar	*gkrellm_get_large_font_string(void);
+gchar	*gkrellm_get_normal_font_string(void);
+gchar	*gkrellm_get_small_font_string(void);
 
-gint	x_label_position(gint, gint, gint, gint);
+gint	gkrellm_label_position(gint, gint, gint, gint);
 
-void	load_inet_data(void);
-void	save_inet_data(void);
+void	gkrellm_inet_load_data(void);
+void	gkrellm_inet_save_data(void);
 
-void	save_net_data(void);
+void	gkrellm_net_save_data(void);
 
-gint	effect_string_value(gchar *);
+gint	gkrellm_effect_string_value(gchar *);
 
-void	map_color_string(gchar *, GdkColor *);
+void	gkrellm_map_color_string(gchar *, GdkColor *);
 
-void	add_plugin_config_page(GkrellmMonitor *);
-void	remove_plugin_config_page(GkrellmMonitor *);
+void	gkrellm_add_plugin_config_page(GkrellmMonitor *);
+void	gkrellm_remove_plugin_config_page(GkrellmMonitor *);
 
-void	make_spin_button(GtkWidget *, GtkWidget **, gfloat, gfloat, gfloat,
-				gfloat, gfloat, gint, gint, void (*func)(), gpointer, gchar *);
+void	gkrellm_set_theme_alternatives_label(void);
+void	gkrellm_start_timer(gint);
 
-void	make_check_button(GtkWidget *, GtkWidget **, gint, gint, gchar *);
-void	make_check_button_shrink(GtkWidget *, GtkWidget **,
-				gint, gint, gchar *);
-void	insert_expanded_filler(GtkWidget *);
-void	common_labels_intro(GtkWidget *);
-
-
-void	set_theme_alternatives_label(void);
-
-void	start_timer(gint);
-
-GtkItemFactory *create_item_factory_popup(void);
+GtkItemFactory *gkrellm_create_item_factory_popup(void);
 
 void	gkrellm_apply_hostname_config(void);
 gboolean gkrellm_hostname_can_shorten(void);
@@ -429,24 +410,16 @@ gchar	*gkrellm_cpu_get_sensor_panel_label(gint n);
 gboolean gkrellm_cpu_set_sensor(gpointer sr, gint type, gint n);
 void	gkrellm_cpu_draw_sensors(gpointer sr);
 
+
 /* utils.c */
-gint	strcmp_basename(gchar *, gchar *);
-gboolean dup_string(gchar **, gchar *);
-gchar	*cut_quoted_string(gchar *, gchar **);
-gboolean getline_from_gstring(GString **, gchar *, gint);
-
-void	free_glist_and_data(GList **);
-GList	*basename_in_list(GList *, gchar *);
-GList	*string_in_list(GList *, gchar *);
-gint	string_position_in_list(GList *list, gchar *s);
-
-gboolean make_home_subdir(gchar *, gchar **);
-void	dup_home_pathname(gchar **, gchar *, gchar *);
-gchar	*entry_get_alpha_text(GtkWidget **);
-
-gint	format_size_abbrev(gchar *, size_t, gfloat,
+gchar	*gkrellm_cut_quoted_string(gchar *, gchar **);
+gboolean gkrellm_getline_from_gstring(GString **, gchar *, gint);
+void	gkrellm_free_glist_and_data(GList **);
+GList	*gkrellm_string_in_list(GList *, gchar *);
+gint	gkrellm_string_position_in_list(GList *list, gchar *s);
+gboolean gkrellm_make_home_subdir(gchar *, gchar **);
+gint	gkrellm_format_size_abbrev(gchar *, size_t, gfloat,
 						GkrellmSizeAbbrev *, size_t);
-gfloat	locale_float_fix(gchar *);
 
 /* sensors.c */
 void		gkrellm_sensors_create_decals(GkrellmPanel *, gint,
@@ -469,14 +442,13 @@ void		gkrellm_border_adjust(GkrellmBorder *border,
 
 
 /* winops */
-void	gkrellm_winop_text_extents(GdkFont *font, gchar *text,
-				gint text_length, gint *lbearing, gint *rbearing,
-				gint *width, gint *ascent, gint *descent);
 void	gkrellm_winop_reset(void);
 void	gkrellm_winop_options(gint, gchar **);
 void	gkrellm_winop_place_gkrellm(gchar *);
 void	gkrellm_winop_flush_motion_events(void);
 gboolean gkrellm_winop_updated_background(void);
+void	gkrellm_winop_update_struts(void);
+void	gkrellm_winop_withdrawn(void);
 gboolean gkrellm_winop_draw_rootpixmap_onto_transparent_panel(GkrellmPanel *);
 gboolean gkrellm_winop_draw_rootpixmap_onto_transparent_chart(GkrellmChart *);
 void	gkrellm_winop_apply_rootpixmap_transparency(void);
@@ -492,9 +464,3 @@ struct tm	*gkrellm_client_server_time(void);
 gint		gkrellm_client_server_connect_state(void);
 void		gkrellm_client_mode_connect_thread(void);
 
-/* deprecated */
-gint	gkrellm_gdk_string_width(GdkFont *, gchar *);
-void	gkrellm_gdk_draw_string(GdkDrawable *drawable, GdkFont *font,
-				GdkGC *gc, gint x, gint y, gchar *string);
-void	gkrellm_gdk_draw_text(GdkDrawable *drawable, GdkFont *font,
-			GdkGC *gc, gint x, gint y, gchar *string, gint len);
