@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -53,6 +53,7 @@ void		gkrellm_draw_chart_to_screen(GkrellmChart *);
 gint		gkrellm_draw_chart_label(GkrellmChart *, GkrellmTextstyle *,
 						gint, gint,gchar *);
 void		gkrellm_draw_chart_text(GkrellmChart *, gint, gchar *);
+void		gkrellm_chart_reuse_text_format(GkrellmChart *cp);
 void		gkrellm_reset_chart(GkrellmChart *);
 void		gkrellm_reset_and_draw_chart(GkrellmChart *);
 void		gkrellm_refresh_chart(GkrellmChart *);
@@ -182,19 +183,35 @@ GkrellmDecal *gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
 						gint x, gint y, gint w);
 GkrellmDecal *gkrellm_create_decal_text_with_height(GkrellmPanel *p,
 						GkrellmTextstyle *ts, GkrellmStyle *style,
-						gint x, gint y, gint w, gint h, gint y_baseline);
+						gint x, gint y, gint w, gint h, gint y_pad);
 GkrellmDecal *gkrellm_create_decal_pixmap(GkrellmPanel *, GdkPixmap *,
 						GdkBitmap *, gint, GkrellmStyle *, gint, gint);
 GkrellmDecal *gkrellm_make_scaled_decal_pixmap(GkrellmPanel *p,
 						GkrellmPiximage *im, GkrellmStyle *style, gint depth,
 						gint x, gint y, gint w, gint h);
- 
+void		gkrellm_decal_get_size(GkrellmDecal *d, gint *w, gint *h);
 void		gkrellm_draw_decal_pixmap(GkrellmPanel *, GkrellmDecal *, gint);
 void		gkrellm_draw_decal_text(GkrellmPanel *, GkrellmDecal *, gchar *,
 						gint);
+void		gkrellm_draw_decal_markup(GkrellmPanel *, GkrellmDecal *, gchar *);
 void		gkrellm_decal_text_clear(GkrellmDecal *d);
 void		gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s,
 						GkrellmTextstyle *ts, gint x_off, gint y_off);
+void		gkrellm_decal_text_set_offset(GkrellmDecal *d, gint x, gint y);
+void		gkrellm_decal_text_get_offset(GkrellmDecal *d, gint *x, gint *y);
+void		gkrellm_decal_scroll_text_align_center(GkrellmDecal *d,
+						gboolean center);
+void		gkrellm_decal_scroll_text_horizontal_loop(GkrellmDecal *d,
+						gboolean loop);
+void		gkrellm_decal_scroll_text_vertical_loop(GkrellmDecal *d,
+						gboolean loop);
+
+void		gkrellm_decal_scroll_text_get_size(GkrellmDecal *d,
+						gint *w, gint *h);
+void		gkrellm_decal_scroll_text_set_markup(GkrellmPanel *p,
+						GkrellmDecal *d, gchar *text);
+void		gkrellm_decal_scroll_text_set_text(GkrellmPanel *p,
+						GkrellmDecal *d, gchar *text);
 
 void		gkrellm_draw_decal_on_chart(GkrellmChart *, GkrellmDecal *,
 						gint, gint);
@@ -324,6 +341,7 @@ void		gkrellm_spacers_set_types(GkrellmMonitor *mon, gint top, gint bot);
 					gkrellm_message_dialog(title, message)
 #define	gkrellm_config_message_window(title, message, widget) \
 					gkrellm_config_message_dialog(title, message)
+gchar		*gkrellm_get_hostname(void);
 
 
   /* Alerts
@@ -370,9 +388,10 @@ void		gkrellm_alert_commands_config(GkrellmAlert *alert,
   */
 GkrellmAlertPlugin	*gkrellm_alert_plugin_add(GkrellmMonitor *mon,
 						gchar *name);
-void		gkrellm_alert_plugin_connect(GkrellmAlertPlugin *gap,
+void		gkrellm_alert_plugin_alert_connect(GkrellmAlertPlugin *gap,
             			void (*alarm_func)(), void (*warn_func)(),
-						void (*update_func)(), void (*destroy_func)());
+						void (*update_func)(), void (*check_func)(),
+						void (*destroy_func)());
 void		gkrellm_alert_plugin_config_connect(GkrellmAlertPlugin *gap,
 						gchar *tab_name,
             			void (*config_create_func)(), void (*config_done_func),
@@ -434,7 +453,7 @@ GdkBitmap	*gkrellm_decal_misc_mask(void);
   */
 GdkGC		*gkrellm_draw_GC(gint);
 GdkGC		*gkrellm_bit_GC(gint);
-GdkFont		*gkrellm_default_font(gint);
+PangoFontDescription *gkrellm_default_font(gint);
 GdkColor	*gkrellm_white_color(void);
 GdkColor	*gkrellm_black_color(void);
 GdkColor	*gkrellm_in_color(void);
@@ -507,12 +526,19 @@ gint		gkrellm_smp_cpus(void);
 gboolean	gkrellm_cpu_stats(gint n, gulong *, gulong *, gulong *, gulong *);
 
 
+  /* Functions exported by disk.c
+  */
+gboolean	gkrellm_disk_temperature_display(gpointer sr, gchar *disk_name,
+						gfloat t, gchar units);
+void		gkrellm_disk_temperature_remove(gchar *disk_name);
+
+ 
   /* Functions exported by net.c
   */
 gint		gkrellm_net_routes(void);
 gboolean	gkrellm_net_stats(gint n, gchar *, gulong *, gulong *);
 void		gkrellm_net_led_positions(gint *x_rx_led, gint *y_rx_led,
-						gint *x_tx_led, gint *y_tx_led);
+					gint *x_tx_led, gint *y_tx_led);
 
 
   /* Functions exported by the Mail monitor - see bottom of mail.c
@@ -523,3 +549,27 @@ void		gkrellm_destroy_external_mbox(gpointer);
 void		gkrellm_set_external_mbox_counts(gpointer, gint, gint);
 void		gkrellm_set_external_mbox_tooltip(gpointer, gchar *);
 
+
+
+/* -------- Client mode functions ----------- */
+gboolean	gkrellm_client_mode(void);
+void		gkrellm_client_plugin_get_setup(gchar *key_name,
+					void (*setup_func_cb)(gchar *str));
+void		gkrellm_client_plugin_serve_data_connect(GkrellmMonitor *mon,
+					gchar *key_name, void (*func_cb)(gchar *line));
+void		gkrellm_client_plugin_reconnect_connect(gchar *key_name,
+					void (*func_cb)());
+
+
+/* ------- Some aids for using PangoFontDescription -------*/
+void	gkrellm_text_extents(PangoFontDescription *font_desc, gchar *text,
+					gint len, gint *width, gint *height, gint *baseline,
+					gint *y_ink);
+
+gint	gkrellm_gdk_string_width(PangoFontDescription *, gchar *);
+void	gkrellm_gdk_draw_string(GdkDrawable *drawable,
+					PangoFontDescription *font,
+					GdkGC *gc, gint x, gint y, gchar *string);
+void	gkrellm_gdk_draw_text(GdkDrawable *drawable,
+					PangoFontDescription *font,
+					GdkGC *gc, gint x, gint y, gchar *string, gint len);
