@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -447,15 +447,6 @@ gkrellm_setup_decal_launcher(GkrellmPanel *p, GkrellmLauncher *launch,
 		}
 	}
 
-void
-insert_expanded_filler(GtkWidget *box)
-	{
-	GtkWidget	*label;
-
-	label = gtk_label_new("");
-	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
-	}
-
 void
 gkrellm_gtk_check_button(GtkWidget *box, GtkWidget **button, gboolean active,
 			gboolean expand, gint pad, gchar *string)
@@ -730,7 +721,7 @@ create_about_tab(GtkWidget *vbox)
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 
 	buf = g_strdup_printf(_("GKrellM %d.%d.%d%s\nGNU Krell Monitors\n\n"
-				"Copyright (c) 1999-2003 by Bill Wilson\n"
+				"Copyright (c) 1999-2004 by Bill Wilson\n"
 				"bill@gkrellm.net\n"
 				"http://gkrellm.net\n\n"
 				"Released under the GNU General Public License"),
@@ -813,7 +804,7 @@ static void
 cb_width_spin(GtkWidget *widget, GtkSpinButton *spin)
 	{
 	_GK.chart_width = gtk_spin_button_get_value_as_int(spin);
-	build_gkrellm();
+	gkrellm_build();
 	}
 
 static void
@@ -824,7 +815,7 @@ cb_HZ_spin(GtkWidget *widget, GtkSpinButton *spin)
 	n = _GK.update_HZ;
 	_GK.update_HZ = gtk_spin_button_get_value_as_int(spin);
 	if (n != _GK.update_HZ)
-		start_timer(_GK.update_HZ);
+		gkrellm_start_timer(_GK.update_HZ);
 	}
 
 static void
@@ -975,6 +966,8 @@ N_("\tMany monitors can be configured to launch commands.  Just enter the\n"
    "\twant a tooltip to appear for the command.  After a command is entered,\n"
    "\tfor a monitor, a button for launching it will become visible when you\n"
    "\tmove the mouse into the panel area of the monitor.\n\n"),
+N_("See the README or do a \"man gkrellm\" for more information.\n"),
+"\n",
 
 N_("<h>\nMouse Button Actions:\n"),
 N_("<b>\tLeft "),
@@ -1045,7 +1038,7 @@ _("Make gkrellm a topmost window (restart gkrellm for this to take effect)."));
 		hbox = gtk_hbox_new(FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 6);
 		gkrellm_gtk_alert_button(hbox, NULL, FALSE, FALSE, 4, TRUE,
-					cb_gkrellmd_disconnect_alert, NULL);
+					gkrellm_gkrellmd_disconnect_cb, NULL);
 		label = gtk_label_new(_("gkrellmd server disconnect"));
 		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 4);
 		}
@@ -1163,7 +1156,7 @@ typedef struct
 	GtkWidget				*browse_button;
 	gchar					*name;
 	gchar					*string;
-	GdkFont					*font;
+//	PangoFontDescription	*font_desc;
 	}
 	AltFontSelect;
 
@@ -1173,19 +1166,19 @@ static AltFontSelect
 			small_font;
 
 gchar *
-get_large_font_override(void)
+gkrellm_get_large_font_string(void)
 	{
 	return large_font.string;
 	}
 
 gchar *
-get_normal_font_override(void)
+gkrellm_get_normal_font_string(void)
 	{
 	return normal_font.string;
 	}
 
 gchar *
-get_small_font_override(void)
+gkrellm_get_small_font_string(void)
 	{
 	return small_font.string;
 	}
@@ -1216,7 +1209,7 @@ cb_font_dialog_ok(GtkWidget *w, AltFontSelect *afs)
 	gtk_widget_destroy(GTK_WIDGET(afs->fontseldlg));
 	theme_modified = TRUE;
 	get_font_entries();
-	build_gkrellm();
+	gkrellm_build();
 	}
 
 static void
@@ -1354,7 +1347,7 @@ theme_compare(Theme *th1, Theme *th2)
 	}
 
 void
-make_themes_list(void)
+gkrellm_make_themes_list(void)
 	{
 	GList	*list;
 	Theme	*theme;
@@ -1366,7 +1359,7 @@ make_themes_list(void)
 		g_free(theme->path);
 		g_free(theme->name);
 		}
-	free_glist_and_data(&themes_list);
+	gkrellm_free_glist_and_data(&themes_list);
 
 	theme = g_new0(Theme, 1);
 	theme->path = g_strdup("Default");
@@ -1411,7 +1404,7 @@ theme_create_model(void)
 	GList			*list;
 	Theme			*theme;
 
-	make_themes_list();
+	gkrellm_make_themes_list();
 	store = gtk_list_store_new(N_THEME_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
 	for (list = themes_list; list; list = list->next)
 		{
@@ -1442,7 +1435,7 @@ cb_theme_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 		{
 		find_theme_position_in_list();
 		theme_modified = TRUE;
-		build_gkrellm();
+		gkrellm_build();
 		}
 	}
 
@@ -1461,7 +1454,7 @@ cb_theme_alternative_changed(GtkWidget *widget, GtkSpinButton *spin)
 		{
 		_GK.theme_alternative = i;
 		theme_modified = TRUE;
-		build_gkrellm();
+		gkrellm_build();
 		}
 	}
 
@@ -1475,12 +1468,12 @@ cb_theme_scale_changed(GtkWidget *widget, GtkSpinButton *spin)
 		{
 		_GK.theme_scale = i;
 		theme_modified = TRUE;
-		build_gkrellm();
+		gkrellm_build();
 		}
 	}
 
 void
-set_theme_alternatives_label(void)
+gkrellm_set_theme_alternatives_label(void)
 	{
 	GtkSpinButton	*spin;
 	gchar			buf[64];
@@ -1495,7 +1488,7 @@ set_theme_alternatives_label(void)
 	}
 
 void
-save_theme_config(void)
+gkrellm_save_theme_config(void)
 	{
 	FILE	*f;
 	gchar	*path;
@@ -1512,8 +1505,7 @@ save_theme_config(void)
 	if ((f = fopen(path, "w")) != NULL)
 		{
 		fprintf(f, "%s\n", _GK.theme_path);
-		fprintf(f, "%d %d %d\n",
-				_GK.theme_alternative, _GK.font_override, _GK.use_fontset);
+		fprintf(f, "%d\n", _GK.theme_alternative);
 		fprintf(f, "%s\n", large_font.string);
 		fprintf(f, "%s\n", normal_font.string);
 		fprintf(f, "%s\n", small_font.string);
@@ -1525,7 +1517,7 @@ save_theme_config(void)
 	}
 
 void
-load_theme_config(void)
+gkrellm_load_theme_config(void)
 	{
 	FILE	*f;
 	gchar	*path, *s;
@@ -1535,10 +1527,8 @@ load_theme_config(void)
 	/* Need to load the theme from ~/.gkrellm/theme_config only at startup
 	|  or if re-reading because of theme_event - these are only times 
 	|  _GK.theme_path will be NULL.  Note: _GK.theme_path will not be NULL
-	|  at startup if there is a command line theme so won't use alternate
-	|  fonts in that case.  This will be right for theme makers testing
-	|  their themes, but not using fontsets may not be right ...
-	|  Likewise, no theme scaling if using command line theme.
+	|  at startup if there is a command line theme, so no theme scaling if
+	|  using command line theme.
 	*/
 	if (!_GK.theme_path)
 		{
@@ -1550,6 +1540,8 @@ load_theme_config(void)
 			{
 			if ((s = strchr(buf, (gint) '\n')) != NULL)
 				*s = '\0';
+			if (_GK.debug_level & DEBUG_GUI)
+				printf("gkrellm_load_theme_config: %s\n", buf);
 			s = buf;
 			if (s && *s != '#' && *s != '\0' && strcmp(s, "Default"))
 				{
@@ -1563,14 +1555,15 @@ load_theme_config(void)
 				{
 				if ((s = strchr(buf, (gint) '\n')) != NULL)
 					*s = '\0';
+				if (_GK.debug_level & DEBUG_GUI)
+					printf("gkrellm_load_theme_config: %s\n", buf);
 				if (i == 0)
-					sscanf(buf, "%d %d %d", &_GK.theme_alternative,
-							&_GK.font_override, &_GK.use_fontset);
-				if (i == 1)
+					sscanf(buf, "%d", &_GK.theme_alternative);
+				if (i == 1 && !strstr(buf, "*-*"))	/* XXX Trap out GdkFont */
 					gkrellm_dup_string(&large_font.string, buf);
-				if (i == 2)
+				if (i == 2 && !strstr(buf, "*-*"))
 					gkrellm_dup_string(&normal_font.string, buf);
-				if (i == 3)
+				if (i == 3 && !strstr(buf, "*-*"))
 					gkrellm_dup_string(&small_font.string, buf);
 				if (i == 4)
 					sscanf(buf, "%d", &_GK.theme_scale);
@@ -1582,14 +1575,11 @@ load_theme_config(void)
 	if (!_GK.theme_path || !g_file_test(_GK.theme_path, G_FILE_TEST_IS_DIR))
 		gkrellm_dup_string(&_GK.theme_path, "");
 	if (!large_font.string)
-		gkrellm_dup_string(&large_font.string,
-				"-b&h-lucida-bold-r-normal-*-*-140-*-*-*-*-iso8859-*");
+		gkrellm_dup_string(&large_font.string, "Serif 11");
 	if (!normal_font.string)
-		gkrellm_dup_string(&normal_font.string,
-				"-b&h-lucida-medium-r-normal-*-*-120-*-*-*-*-iso8859-*");
+		gkrellm_dup_string(&normal_font.string, "Serif 9");
 	if (!small_font.string)
-		gkrellm_dup_string(&small_font.string,
-				"-adobe-helvetica-medium-r-*-*-*-100-*-*-*-*-iso8859-*");
+		gkrellm_dup_string(&small_font.string, "Serif 8");
 	}
 
 
@@ -1630,8 +1620,8 @@ gkrellm_read_theme_event(GtkSettings  *settings)
 				{
 				_GK.theme_alternative = alt;
 				theme_modified = TRUE;
-				save_theme_config();
-				build_gkrellm();
+				gkrellm_save_theme_config();
+				gkrellm_build();
 				}
 			}
 		}
@@ -1640,7 +1630,7 @@ gkrellm_read_theme_event(GtkSettings  *settings)
 		{
 		g_free(_GK.theme_path);
 		_GK.theme_path = NULL;	/* Forces reread of GKRELLM_THEME_CONFIG */
-		build_gkrellm();
+		gkrellm_build();
 		}
 	}
 
@@ -1657,7 +1647,7 @@ cb_load_theme(gpointer data, guint action, GtkWidget *widget)
 	if (_GK.no_config)
 		return;
 	if (!themes_list)
-		make_themes_list();
+		gkrellm_make_themes_list();
 	if (action == THEME_ALT_NEXT || action == THEME_ALT_PREV)
 		{
 		_GK.theme_alternative += ((action == THEME_ALT_NEXT) ? 1 : -1);
@@ -1697,7 +1687,7 @@ cb_load_theme(gpointer data, guint action, GtkWidget *widget)
 			gtk_tree_model_iter_nth_child(model, &iter, NULL, row);
 			path = gtk_tree_model_get_path(model, &iter);
 			gtk_tree_view_set_cursor(theme_treeview, path, NULL, FALSE);
-			return;		/* cb_theme_tree_selection_changed -> build_gkrellm()*/
+			return;		/* cb_theme_tree_selection_changed -> gkrellm_build()*/
 			}
 		theme = (Theme *) theme_position_in_list->data;
 		gkrellm_dup_string(&_GK.theme_path,
@@ -1714,7 +1704,7 @@ cb_load_theme(gpointer data, guint action, GtkWidget *widget)
 		_GK.theme_scale -= 20;
 		theme_modified = TRUE;
 		}
-	build_gkrellm();
+	gkrellm_build();
 	}
 
 static void
@@ -1734,43 +1724,13 @@ close_theme_config(gint from_close)
 	destroy_font_dialogs();
 	}
 
-static void
-cb_use_fontset(GtkWidget *widget, gpointer *data)
-	{
-	_GK.use_fontset = GTK_TOGGLE_BUTTON(widget)->active;
-	theme_modified = TRUE;
-	build_gkrellm();
-	}
-
-static void
-set_font_override_sensitivities(gboolean sensitive)
-	{
-	gtk_widget_set_sensitive(large_font.entry, sensitive);
-	gtk_widget_set_sensitive(large_font.browse_button, sensitive);
-	gtk_widget_set_sensitive(normal_font.entry, sensitive);
-	gtk_widget_set_sensitive(normal_font.browse_button, sensitive);
-	gtk_widget_set_sensitive(small_font.entry, sensitive);
-	gtk_widget_set_sensitive(small_font.browse_button, sensitive);
-	}
-
-static void
-cb_font_override(GtkToggleButton *button, gpointer *data)
-	{
-	_GK.font_override = button->active;
-	build_gkrellm();
-	set_font_override_sensitivities(_GK.font_override);
-	theme_modified = TRUE;
-	if (!_GK.font_override)
-		destroy_font_dialogs();
-	}
-
 static void
 cb_font_entry_activate(GtkWidget *widget, gpointer *data)
 	{
 	if (!get_font_entries())
 		return;
 	theme_modified = TRUE;
-	build_gkrellm();
+	gkrellm_build();
 	}
 
 static void
@@ -1866,7 +1826,7 @@ create_theme_tab(GtkWidget *tabs_vbox)
 	theme_alt_label = gtk_label_new("");
 	gtk_box_pack_start (GTK_BOX (hbox), theme_alt_label, TRUE, TRUE, 4);
 	gtk_misc_set_alignment(GTK_MISC(theme_alt_label), 0, 0.5);
-	set_theme_alternatives_label();
+	gkrellm_set_theme_alternatives_label();
 
 
 /* -- Options tab */
@@ -1950,17 +1910,6 @@ create_theme_tab(GtkWidget *tabs_vbox)
 	gkrellm_gtk_button_connected(hbox, &small_font.browse_button, FALSE, FALSE,
 			0, cb_font_dialog, &small_font, _("Browse"));
 
-	gkrellm_gtk_check_button_connected(vbox, NULL,
-			_GK.font_override, FALSE, FALSE, 2,
-			cb_font_override, NULL,
-			_("Use above alternate fonts instead of theme specified fonts"));
-	set_font_override_sensitivities(_GK.font_override);
-
-	gkrellm_gtk_check_button_connected(vbox, NULL,
-			_GK.use_fontset, FALSE, FALSE, 2,
-			cb_use_fontset, NULL,
-			_("Load fonts as fontsets"));
-
 	/* --Info tab*/
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
 	s = g_strdup_printf(_("Untar your theme tar files in %s/%s"),
@@ -2023,12 +1972,12 @@ close_config(gpointer data)
 	gtk_widget_destroy(config_window);
 	config_window = NULL;
 
-	close_plugins_config();
+	gkrellm_plugins_config_close();
 	close_theme_config(from_close);
 	if (_GK.config_modified)
 		{
-		save_theme_config();
-		save_user_config();
+		gkrellm_save_theme_config();
+		gkrellm_save_user_config();
 		}
 	}
 
@@ -2104,7 +2053,7 @@ real_create_config(GkrellmMonitor *mon)
 	}
 
 void
-add_plugin_config_page(GkrellmMonitor *mon)
+gkrellm_add_plugin_config_page(GkrellmMonitor *mon)
 	{
 	GtkTreeIter		iter, plugin_iter;
 	GtkTreePath		*path;
@@ -2130,7 +2079,7 @@ add_plugin_config_page(GkrellmMonitor *mon)
 	}
 
 void
-remove_plugin_config_page(GkrellmMonitor *mon)
+gkrellm_remove_plugin_config_page(GkrellmMonitor *mon)
 	{
 	GtkTreePath		*path;
 	GtkTreeIter		iter;
@@ -2315,7 +2264,7 @@ create_config_window(void)
 	gtk_tree_store_append(model, &iter, NULL);
 	gtk_tree_store_set(model, &iter, NAME_COLUMN, _("Plugins"), -1);
 	vbox = create_config_page(NULL, model, &iter, config_notebook);
-	create_plugin_config(vbox);
+	gkrellm_plugins_config_create(vbox);
 
 	gtk_tree_store_append(model, &iter, NULL);
 	gtk_tree_store_set(model, &iter, NAME_COLUMN, _("Themes"), -1);
@@ -2338,7 +2287,7 @@ create_config_window(void)
 			|| MONITOR_ID(mon) != MON_PLUGIN
 		   )
 			continue;
-		add_plugin_config_page(mon);
+		gkrellm_add_plugin_config_page(mon);
 		}
 
 	/* Create the tree view and don't unref the model because need to modify
@@ -2450,7 +2399,7 @@ static GtkItemFactory	*debug_factory;
 
 
 GtkItemFactory *
-create_item_factory_popup(void)
+gkrellm_create_item_factory_popup(void)
 	{
 	GtkWidget		*top_win;
 	GtkItemFactory	*item_factory;
