@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -22,8 +22,8 @@
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 
-GkrellmBorder	zero_border;
-GkrellmMargin	zero_margin;
+static GkrellmBorder	zero_border;
+static GkrellmMargin	zero_margin;
 
   /* Smooth the krell response with an exponential moving average.
   |  Eponential MA is quicker responding than pure moving average.
@@ -159,19 +159,100 @@ gkrellm_update_krell(GkrellmPanel *p, GkrellmKrell *k, gulong value)
 	}
 
 void
-gkrellm_draw_decal_on_chart(GkrellmChart *cp, GkrellmDecal *d, gint x, gint y)
+gkrellm_decal_get_size(GkrellmDecal *d, gint *w, gint *h)
 	{
+	if (!d)
+		return;
+	if (w)
+		*w = d->w;
+	if (h)
+		*h = d->h;
+	}
+
+void
+gkrellm_draw_decal_on_chart(GkrellmChart *cp, GkrellmDecal *d,
+			gint xd, gint yd)
+	{
+	PangoLayout			*layout;
+	PangoRectangle		ink;
+	GList				*list;
+	GdkRectangle		rect, *r;
+	GkrellmTextstyle	*ts;
+	GkrellmText			*tx;
+	gint				x, y, max;
+
 	if (!cp || !d || d->state != DS_VISIBLE)
 		return;
-	d->x = x;
-	d->y = y;
-	gdk_gc_set_clip_mask(_GK.text_GC, d->stencil);
-	gdk_gc_set_clip_origin(_GK.text_GC, x, y);
-	gdk_draw_drawable(cp->pixmap, _GK.text_GC, d->pixmap,
-				0, d->y_src, x, y, d->w, d->h);
+	d->x = xd;
+	d->y = yd;
 
-	gdk_gc_set_clip_mask(_GK.text_GC, NULL);
-	gdk_gc_set_clip_origin(_GK.text_GC, 0, 0);
+	if (d->text_list)
+		{
+		r = &d->ink;
+		if (d->modified || d->chart_sequence_id != cp->bg_sequence_id)
+			{
+			gdk_draw_drawable(cp->bg_text_pixmap, _GK.draw1_GC, cp->bg_pixmap,
+						d->x, d->y, d->x, d->y, d->w, d->h);
+			layout = gtk_widget_create_pango_layout(
+						gkrellm_get_top_window(), NULL);
+			memset(r, 0, sizeof(GdkRectangle));
+			rect.x = d->x;
+			rect.y = d->y;
+			rect.width = d->w;
+			rect.height = d->h;
+			gdk_gc_set_clip_rectangle(_GK.text_GC, &rect);
+			for (list = d->text_list; list; list = list->next)
+				{
+				tx = (GkrellmText *) list->data;
+				ts = &tx->text_style;
+				pango_layout_set_font_description(layout, ts->font);
+				if (d->flags & DF_TEXT_USE_MARKUP)
+					pango_layout_set_markup(layout, tx->text,strlen(tx->text));
+				else
+					pango_layout_set_text(layout, tx->text, strlen(tx->text));
+				x = d->x + tx->x_off;
+				y = d->y + tx->y_off;
+				if (ts->effect)
+					{
+					gdk_gc_set_foreground(_GK.text_GC, &ts->shadow_color);
+					gdk_draw_layout_with_colors(cp->bg_text_pixmap,
+							_GK.text_GC,
+							x + 1, y + 1, layout,
+							&ts->shadow_color, NULL);
+					}
+				gdk_gc_set_foreground(_GK.text_GC, &ts->color);
+				gdk_draw_layout(cp->bg_text_pixmap, _GK.text_GC, x, y, layout);
+
+				pango_layout_get_pixel_extents(layout, &ink, NULL);
+				if (r->x == 0 || ink.x + x < r->x)	/* Shadow ??? */
+					r->x = ink.x + x;
+				if (r->y == 0 || ink.y + y < r->y)
+					r->y = ink.y + y;
+				max = ink.x + x + ink.width;
+				if (r->x + r->width < max)
+					r->width = max - r->x;
+				max = ink.y + y + ink.height;
+				if (r->y + r->height < max)
+					r->height = max - r->y;
+				}
+			gdk_gc_set_clip_rectangle(_GK.text_GC, NULL);
+			g_object_unref(layout);
+			d->chart_sequence_id = cp->bg_sequence_id;
+			d->modified = FALSE;
+			}
+		gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_text_pixmap,
+					r->x, r->y, r->x, r->y, r->width, r->height);
+		}
+	else
+		{
+		gdk_gc_set_clip_mask(_GK.text_GC, d->stencil);
+		gdk_gc_set_clip_origin(_GK.text_GC, xd, yd);
+		gdk_draw_drawable(cp->pixmap, _GK.text_GC, d->pixmap,
+					0, d->y_src, xd, yd, d->w, d->h);
+
+		gdk_gc_set_clip_mask(_GK.text_GC, NULL);
+		gdk_gc_set_clip_origin(_GK.text_GC, 0, 0);
+		}
 	}
 
 void
@@ -198,56 +279,412 @@ gkrellm_draw_decal_pixmap(GkrellmPanel *p, GkrellmDecal *d, gint index)
 	d->value = index;
 	}
 
-void
-gkrellm_draw_decal_text(GkrellmPanel *p, GkrellmDecal *d, gchar *s, gint value)
+static void
+decal_text_list_free(GList **text_list)
 	{
-	GkrellmTextstyle	*ts;
-	gint				x, y;
+	GList	*list;
+
+	if (!text_list || !*text_list)
+		return;
+	for (list = *text_list; list; list = list->next)
+		g_free(((GkrellmText *) list->data)->text);
+	gkrellm_free_glist_and_data(text_list);
+	}
+
+
+static void
+gkrellm_draw_decal_internal(GkrellmPanel *p, GkrellmDecal *d, gchar *s,
+			gint value)
+	{
+	GkrellmText			*tx;
+	GList				*list;
 
 	if (!s || !d || d->state == DS_INVISIBLE)
 		return;
-	if (d->value != value || value == -1)
-		{
-		ts = &d->text_style;
-		x = d->x_off;
-		y = d->y_baseline;
-		gkrellm_draw_string(d->pixmap, ts, x, y, s);
-		gdk_draw_rectangle(d->stencil, _GK.bit0_GC, TRUE, 0, 0, d->w, d->h);
-		gkrellm_gdk_draw_string(d->stencil, ts->font, _GK.bit1_GC, x, y, s);
-		if (ts->effect)
-			gkrellm_gdk_draw_string(d->stencil, ts->font, _GK.bit1_GC,
-						x + 1, y + 1, s);
+	if (!d->text_list)
+		d->text_list = g_list_append(d->text_list, g_new0(GkrellmText, 1));
+	list = d->text_list;
+	tx = (GkrellmText *) list->data;
+	if (   gkrellm_dup_string(&tx->text, s)
+		|| tx->x_off_prev != d->x_off
+		|| tx->text_style.font != d->text_style.font
+		|| tx->text_style.color.pixel != d->text_style.color.pixel
+	   )
 		d->modified = TRUE;
-		}
+	tx->x_off = d->x_off;
+	tx->x_off_prev = d->x_off;
+	tx->y_off = d->y_off - d->y_ink;
+	tx->text_style = d->text_style;
+	if (list->next)
+		decal_text_list_free(&list->next);
 	d->value = value;
 	}
 
+void
+gkrellm_draw_decal_text(GkrellmPanel *p, GkrellmDecal *d, gchar *s, gint value)
+	{
+	d->flags &= ~DF_TEXT_USE_MARKUP;
+	gkrellm_draw_decal_internal(p, d, s, value);
+	}
+
+void
+gkrellm_draw_decal_markup(GkrellmPanel *p, GkrellmDecal *d, gchar *s)
+	{
+	d->flags |= DF_TEXT_USE_MARKUP;
+	gkrellm_draw_decal_internal(p, d, s, 0);
+	}
+
 void
 gkrellm_decal_text_clear(GkrellmDecal *d)
 	{
-	gdk_draw_rectangle(d->stencil, _GK.bit0_GC, TRUE, 0, 0, d->w, d->h);
+	GList		*list;
+	GkrellmText	*tx;
+
+	if (!d || ((list = d->text_list) == NULL))
+		return;
+	tx = (GkrellmText *) list->data;
+
+	if (list->next)
+		decal_text_list_free(&list->next);
+
+	gkrellm_dup_string(&tx->text, "");
+	tx->text_style = d->text_style;
+
+	if (d->scroll_text)
+		g_free(d->scroll_text);
+	d->scroll_text = NULL;
+
+	d->modified = TRUE;
 	}
 
 void
 gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts,
 		gint x_off, gint y_off)
 	{
-	gint				x, y;
+	GkrellmText	*tx = NULL;
 
 	if (!s || !d || d->state == DS_INVISIBLE)
 		return;
 	if (!ts)
 		ts = &d->text_style;
-	x = x_off;
-	y = d->y_baseline + y_off;
-	gkrellm_draw_string(d->pixmap, ts, x, y, s);
-	gkrellm_gdk_draw_string(d->stencil, ts->font, _GK.bit1_GC, x, y, s);
-	if (ts->effect)
-		gkrellm_gdk_draw_string(d->stencil, ts->font, _GK.bit1_GC,
-					x + 1, y + 1, s);
+	if (   d->text_list
+		&& !*((GkrellmText *) d->text_list->data)->text
+	   )
+		tx = (GkrellmText *) d->text_list->data;
+
+	if (!tx)
+		{
+		tx = g_new0(GkrellmText, 1);
+		d->text_list = g_list_append(d->text_list, tx);
+		}
+
+	gkrellm_dup_string(&tx->text, s);
+	tx->x_off = x_off;
+	tx->x_off_prev = x_off;
+	tx->y_off = y_off - d->y_ink;
+	tx->text_style = *ts;
+
 	d->modified = TRUE;
 	}
 
+void
+gkrellm_decal_text_get_offset(GkrellmDecal *d, gint *x, gint *y)
+	{
+	if (!d)
+		return;
+	if (x)
+		*x = d->x_off;
+	if (y)
+		*y = d->y_off;
+	}
+
+void
+gkrellm_decal_text_set_offset(GkrellmDecal *d, gint x, gint y)
+	{
+	GList		*list;
+	GkrellmText	*tx;
+
+	if (!d || (d->x_off == x && d->y_off == y))
+		return;
+
+	d->x_off = x;
+	d->y_off = y;
+	for (list = d->text_list; list; list = list->next)
+		{
+		tx = (GkrellmText *) list->data;
+		tx->x_off = x;
+		tx->x_off_prev = x;
+		tx->y_off = y - d->y_ink;;
+		}
+	d->modified = TRUE;
+	}
+
+void
+gkrellm_decal_scroll_text_align_center(GkrellmDecal *d, gboolean center)
+	{
+	gint	prev_center;
+
+	if (!d)
+		return;
+	prev_center = d->flags & DF_SCROLL_TEXT_CENTER;
+	if ((center && prev_center) || (!center && !prev_center))
+		return;
+
+	if (center)
+		d->flags |= DF_SCROLL_TEXT_CENTER;
+	else
+		d->flags &= ~DF_SCROLL_TEXT_CENTER;
+	d->modified = TRUE;
+	}
+
+void
+gkrellm_decal_scroll_text_horizontal_loop(GkrellmDecal *d, gboolean loop)
+	{
+	gint	prev_loop;
+
+	if (!d)
+		return;
+	prev_loop = d->flags & DF_SCROLL_TEXT_H_LOOP;
+	if ((loop && prev_loop) || (!loop && !prev_loop))
+		return;
+
+	if (loop)
+		d->flags |= DF_SCROLL_TEXT_H_LOOP;
+	else
+		d->flags &= ~DF_SCROLL_TEXT_H_LOOP;
+	d->modified = TRUE;
+	}
+
+void
+gkrellm_decal_scroll_text_vertical_loop(GkrellmDecal *d, gboolean loop)
+	{
+	gint	prev_loop;
+
+	if (!d)
+		return;
+	prev_loop = d->flags & DF_SCROLL_TEXT_V_LOOP;
+	if ((loop && prev_loop) || (!loop && !prev_loop))
+		return;
+
+	if (loop)
+		d->flags |= DF_SCROLL_TEXT_V_LOOP;
+	else
+		d->flags &= ~DF_SCROLL_TEXT_V_LOOP;
+	d->modified = TRUE;
+	}
+
+void
+gkrellm_decal_scroll_text_get_size(GkrellmDecal *d, gint *w, gint *h)
+	{
+	if (!d)
+		return;
+	if (w)
+		*w = d->scroll_width;
+	if (h)
+		*h = d->scroll_height
+					+ ((d->flags & DF_SCROLL_TEXT_V_LOOP) ? d->y_ink : 0);
+	}
+
+static PangoLayout *
+decal_scroll_text_layout(GkrellmDecal *d, gchar *text, gint *yink)
+	{
+	PangoLayout			*layout;
+	PangoRectangle		ink, logical;
+	GkrellmTextstyle	*ts;
+	gint				y_ink;
+
+	ts = &d->text_style;
+	layout = gtk_widget_create_pango_layout(gkrellm_get_top_window(), NULL);
+	pango_layout_set_spacing(layout, 0);
+	if (d->flags & DF_SCROLL_TEXT_CENTER)
+		pango_layout_set_alignment(layout, PANGO_ALIGN_CENTER);
+	pango_layout_set_font_description(layout, ts->font);
+	if (d->flags & DF_TEXT_USE_MARKUP)
+		pango_layout_set_markup(layout, text, strlen(text));
+	else
+		pango_layout_set_text(layout, text, strlen(text));
+
+	pango_layout_get_pixel_extents(layout, &ink, &logical);
+	d->scroll_width = logical.width + ts->effect;	/* incl trailing spaces */
+	d->scroll_height = ink.height + ts->effect;
+	y_ink = ink.y - logical.y;
+
+	/* y_ink change can happen if markup up text sizes differently from sizing
+	|  at decal creation.  Adjust the decal y_ink so text doesn't get bumped
+	|  upward which clips top of text string.  This means the decal height
+	|  should probably also be changed because now descenders might get
+	|  clipped when switching back out of scrolling.
+	|  But this is much less noticeable... later.
+	*/
+	if (d->y_ink > y_ink)
+		d->y_ink = y_ink;
+
+	if (yink)
+		*yink = y_ink;
+	return layout;
+	}
+
+
+  /* Draw Pango rendered text onto a pixmap which can be scrolled across
+  |  a decal's extents.  So when scrolling, a slow Pango redraw at each scroll
+  |  step can be avoided if the text string has not changed.
+  |  There are a couple of limitations because it's impossible to generate a
+  |  stencil bitmask of Pango rendered text (Pango doesn't allow writting to
+  |  depth 1 and it looks at the background pixmap colors as it renders):
+  |    1) A scroll text decal is not transparent and will hide any objects
+  |       underneath it, so when this condition is detected, scroll drawing
+  |       is diverted to Pango at each step gkrellm_draw_decal_text().
+  |    2) The scroll text must be rendered onto a background which for many
+  |       themes won't exactly blend with the panel background as the scroll
+  |       pixmap is scrolled.  It should be at worst only slightly noticeable
+  |       for most themes, but might be a problem for themes with high gradient
+  |       panel backgrounds.
+  */
+static void
+gkrellm_decal_scroll_text_set_internal(GkrellmPanel *p, GkrellmDecal *d,
+			gchar *text)
+	{
+	GtkWidget			*top_win = gkrellm_get_top_window();
+	PangoLayout			*layout;
+	GdkPixbuf			*pixbuf;
+	GList				*list;
+	GkrellmDecal		*dcheck;
+	GkrellmBorder		*b;
+	GkrellmTextstyle	*ts;
+	gint				dx, y_ink;
+	gboolean			new_text, no_scroll_caching;
+
+	if (!p || !text || !d || d->state == DS_INVISIBLE)
+		return;
+
+	new_text = gkrellm_dup_string(&d->scroll_text, text);
+	no_scroll_caching = (p->transparency || p->scroll_text_cache_off);
+
+	for (list = p->decal_list; list; list = list->next)
+		{
+		dcheck = (GkrellmDecal *) list->data;
+
+		/* Scroll text underneath is OK */
+		if (dcheck == d && !no_scroll_caching)
+			break;
+
+		/* But scroll text overlapping and on top of any other decal requires
+		|  scroll text to be pushed.
+		*/
+		if (   no_scroll_caching
+			|| (   d->x + d->w > dcheck->x
+				&& d->x < dcheck->x + dcheck->w
+				&& d->y + d->h > dcheck->y
+				&& d->y < dcheck->y + dcheck->h
+			   )
+		   )
+			{
+			d->flags |= (DF_TEXT_OVERLAPS | DF_SCROLL_TEXT_DIVERTED);
+			if (new_text)
+				{
+				layout = decal_scroll_text_layout(d, text, NULL);
+				g_object_unref(G_OBJECT(layout));
+				}
+			gkrellm_draw_decal_text(p, d, text, -1);
+			return;
+			}
+		}
+	if (d->text_list)	/* Erase any previous gkrellm_draw_decal_text()	*/
+		{
+		if (d->flags & DF_MOVED)
+			{
+			gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
+						p->bg_pixmap,
+						d->x_old, d->y_old, d->x_old, d->y_old, d->w, d->h);
+			d->flags &= ~DF_MOVED;
+			}
+		else
+			gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
+						p->bg_pixmap,
+						d->x, d->y,  d->x, d->y,   d->w, d->h);
+
+		decal_text_list_free(&d->text_list);
+		d->value = -1;
+		}
+	d->modified = TRUE;
+
+	if (!new_text && !(d->flags & DF_SCROLL_TEXT_DIVERTED))
+		return;
+
+	layout = decal_scroll_text_layout(d, text, &y_ink);
+	gkrellm_free_pixmap(&d->pixmap);
+	d->pixmap = gdk_pixmap_new(top_win->window,
+					d->scroll_width, d->scroll_height, -1);
+
+	if (!d->pixmap)		/* too wide maybe? */
+		{
+		d->flags |= (DF_TEXT_OVERLAPS | DF_SCROLL_TEXT_DIVERTED);
+		g_object_unref(G_OBJECT(layout));
+		gkrellm_draw_decal_text(p, d, text, -1);
+		return;
+		}
+	d->flags &= ~(DF_TEXT_OVERLAPS | DF_SCROLL_TEXT_DIVERTED);
+
+	dx = d->w - d->scroll_width;
+
+	if (dx >= 0)
+		pixbuf = gdk_pixbuf_get_from_drawable(NULL, p->bg_pixmap, NULL,
+				d->x + ((d->flags & DF_SCROLL_TEXT_CENTER) ? dx / 2 : 0), d->y,
+				0, 0, d->scroll_width, d->h);
+	else
+		{
+		gint	xfudge, yfudge;	/* Many themes have sloppy margins XXX */
+
+		xfudge = d->w > 20 ? d->w / 6 : 0;	/* and gradient compensate a bit */
+		yfudge = d->h > 5 ? d->h / 5 : 0;
+		pixbuf = gdk_pixbuf_get_from_drawable(NULL, p->bg_pixmap, NULL,
+				d->x + xfudge, d->y + yfudge, 0, 0,
+				d->w - 2 * xfudge, d->h - 2 * yfudge);
+		}
+
+	if (pixbuf)
+		gkrellm_paste_pixbuf(pixbuf, d->pixmap, 0, 0,
+					d->scroll_width, d->scroll_height);
+	else
+		{
+		b = &p->bg_piximage->border;
+		gkrellm_paste_pixbuf(p->bg_piximage->pixbuf, d->pixmap,
+					b->left, b->top,
+					d->scroll_width + b->left + b->right,
+					d->scroll_height + b->top + b->bottom); /* XXX */
+		}
+
+	ts = &d->text_style;
+	if (ts->effect)
+		{
+		gdk_gc_set_foreground(_GK.text_GC, &ts->shadow_color);
+		gdk_draw_layout_with_colors(d->pixmap, _GK.text_GC,
+					1, -y_ink + 1, layout,
+					&ts->shadow_color, NULL);
+		}
+	gdk_gc_set_foreground(_GK.text_GC, &ts->color);
+	gdk_draw_layout(d->pixmap, _GK.text_GC, 0, -y_ink, layout);
+
+	g_object_unref(G_OBJECT(layout));
+	}
+
+void
+gkrellm_decal_scroll_text_set_markup(GkrellmPanel *p, GkrellmDecal *d,
+			gchar *text)
+	{
+	d->flags |= DF_TEXT_USE_MARKUP;
+	gkrellm_decal_scroll_text_set_internal(p, d, text);
+	}
+
+void
+gkrellm_decal_scroll_text_set_text(GkrellmPanel *p, GkrellmDecal *d,
+			gchar *text)
+	{
+	d->flags &= ~DF_TEXT_USE_MARKUP;
+	gkrellm_decal_scroll_text_set_internal(p, d, text);
+	}
+
+
 void
 gkrellm_move_krell_yoff(GkrellmPanel *p, GkrellmKrell *k, gint y)
 	{
@@ -531,17 +968,24 @@ gkrellm_move_decal(GkrellmPanel *p, GkrellmDecal *d, gint x, gint y)
 	d->y = y;
 	d->flags |= DF_MOVED;
 	d->modified = TRUE;
+
+	if (p)
+		p->need_decal_overlap_check = TRUE;
 	}
 
 void
 gkrellm_decal_on_top_layer(GkrellmDecal *d, gboolean top)
 	{
+	GkrellmPanel	*p = (GkrellmPanel *) d->panel;
 	if (!d)
 		return;
 	if (top)
 		d->flags |= DF_TOP_LAYER;
 	else
 		d->flags &= ~DF_TOP_LAYER;
+	if (p)
+		p->need_decal_overlap_check = TRUE;
+
 	d->modified = TRUE;
 	}
 
@@ -554,6 +998,13 @@ _destroy_decal(GkrellmDecal *d)
 		g_object_unref(G_OBJECT(d->pixmap));
 	if (d->stencil)
 		g_object_unref(G_OBJECT(d->stencil));
+	if (d->text_list)
+		decal_text_list_free(&d->text_list);
+	if (d->scroll_text)
+		g_free(d->scroll_text);
+	if (d->panel)
+		((GkrellmPanel *)d->panel)->need_decal_overlap_check = TRUE;
+
 	g_free(d);
 	}
 
@@ -563,6 +1014,7 @@ _remove_decal(GkrellmPanel *p, GkrellmDecal *d)
 	if (!g_list_find(p->decal_list, d))
 		return;
 	p->decal_list = g_list_remove(p->decal_list, d);
+	p->need_decal_overlap_check = TRUE;
 	d->panel = NULL;
 	if (p->pixmap && p->bg_pixmap)
 		{
@@ -612,6 +1064,7 @@ gkrellm_insert_decal(GkrellmPanel *p, GkrellmDecal *d, gboolean append)
 	else
 		p->decal_list = g_list_prepend(p->decal_list, d);
 	d->panel = (gpointer) p;
+	p->need_decal_overlap_check = TRUE;
 	gkrellm_draw_panel_layers_force(p);
 	}
 
@@ -622,6 +1075,7 @@ gkrellm_insert_decal_nth(GkrellmPanel *p, GkrellmDecal *d, gint n)
 		return;
 	p->decal_list = g_list_insert(p->decal_list, d, n);
 	d->panel = (gpointer) p;
+	p->need_decal_overlap_check = TRUE;
 	gkrellm_draw_panel_layers_force(p);
 	}
 
@@ -660,7 +1114,10 @@ gkrellm_create_decal_pixmap(GkrellmPanel *p,
 	top_win = gkrellm_get_top_window();
 	d = (GkrellmDecal *) g_new0(GkrellmDecal, 1);
 	if (p)
+		{
 		p->decal_list = g_list_append(p->decal_list, d);
+		p->need_decal_overlap_check = TRUE;
+		}
 	d->panel = (gpointer) p;
 
 	gdk_drawable_get_size(pixmap, &d->w, &d->h);
@@ -697,33 +1154,40 @@ gkrellm_create_decal_pixmap(GkrellmPanel *p,
 static GkrellmDecal *
 _create_decal_text(GkrellmPanel *p, gchar *string,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
-				gint x, gint y, gint w, gint h, gint y_base)
+				gint x, gint y, gint w, gint h, gint y_ink)
 	{
 	GtkWidget		*top_win;
 	GkrellmMargin	*m;
 	GkrellmDecal	*d;
-	gint			ll, rr, ww, aa, dd;
+	GkrellmText		*tx;
+	gint			width, height, baseline;
 
 	top_win = gkrellm_get_top_window();
 	d = (GkrellmDecal *) g_new0(GkrellmDecal, 1);
 	if (p)
+		{
 		p->decal_list = g_list_append(p->decal_list, d);
+		p->need_decal_overlap_check = TRUE;
+		}
 	d->panel = (gpointer) p;
 
-	d->text_style = *ts;
+	d->text_style = ts ? *ts : *gkrellm_meter_textstyle(0);
+	tx = g_new0(GkrellmText, 1);
+	d->text_list = g_list_append(d->text_list, tx);
+	gkrellm_dup_string(&tx->text, "");
+	tx->text_style = d->text_style;
 
 	if (string && *string)
 		{
-		gkrellm_winop_text_extents(d->text_style.font, string, strlen(string),
-				&ll, &rr, &ww, &aa, &dd);
-		d->h = aa + dd + d->text_style.effect;
-		d->y_baseline = aa;
+		gkrellm_text_extents(d->text_style.font, string, strlen(string),
+				&width, &height, &baseline, &d->y_ink);
+		d->h = height + d->text_style.effect;
 		}
 	else
 		{
-		ww = 2;
+		width = 2;
 		d->h = h;
-		d->y_baseline = y_base;
+		d->y_ink = y_ink;
 		}
 	if (d->h <= 0)
 		d->h = 2;
@@ -737,7 +1201,7 @@ _create_decal_text(GkrellmPanel *p, gchar *string,
 		if (w < 0)
 			d->w = _GK.chart_width - m->left - m->right;
 		else if (w == 0)
-			d->w = ww + d->text_style.effect;
+			d->w = width + d->text_style.effect;
 		else
 			d->w = w;
 
@@ -752,7 +1216,7 @@ _create_decal_text(GkrellmPanel *p, gchar *string,
 		if (w < 0)
 			d->w = _GK.chart_width;
 		else if (w == 0)
-			d->w = ww;
+			d->w = width;
 		else
 			d->w = w;
 		d->x = x;
@@ -776,19 +1240,15 @@ gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
 				gint x, gint y, gint w)
 	{
-	if (!ts || !string)
-		return NULL;
 	return _create_decal_text(p, string, ts, style, x, y, w, 0, 0);
 	}
 
 GkrellmDecal *
 gkrellm_create_decal_text_with_height(GkrellmPanel *p,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
-				gint x, gint y, gint w, gint h, gint y_baseline)
+				gint x, gint y, gint w, gint h, gint y_ink)
 	{
-	if (!ts)
-		return NULL;
-	return _create_decal_text(p, NULL, ts, style, x, y, w, h, y_baseline);
+	return _create_decal_text(p, NULL, ts, style, x, y, w, h, y_ink);
 	}
 
 void
@@ -796,10 +1256,10 @@ gkrellm_make_decal_invisible(GkrellmPanel *p, GkrellmDecal *d)
 	{
 	if (!p || !d || d->state == DS_INVISIBLE)
 		return;
-	d->state = DS_ERASING;
-	d->modified = TRUE;
-	gkrellm_draw_panel_layers(p);
 	d->state = DS_INVISIBLE;
+	d->modified = TRUE;
+	p->need_decal_overlap_check = TRUE;
+	gkrellm_draw_panel_layers(p);
 	}
 
 void
@@ -809,6 +1269,7 @@ gkrellm_make_decal_visible(GkrellmPanel *p, GkrellmDecal *d)
 		return;
 	d->state = DS_VISIBLE;
 	d->modified = TRUE;
+	p->need_decal_overlap_check = TRUE;
 	gkrellm_draw_panel_layers(p);
 	}
 
@@ -951,11 +1412,11 @@ cb_decal_button_leave(GtkWidget *widget, GdkEventButton *ev, GkrellmPanel *p)
 			{
 			if (b->sensitive)
 				{
-				b->decal->state = DS_ERASING;
 				set_button_index(b, 0, 0);
-				b->decal->modified = TRUE;
-				gkrellm_draw_panel_layers(b->panel);
 				b->decal->state = DS_INVISIBLE;
+				b->decal->modified = TRUE;
+				p->need_decal_overlap_check = TRUE;
+				gkrellm_draw_panel_layers(p);
 				}
 			}
 		else
@@ -981,6 +1442,7 @@ cb_decal_button_enter(GtkWidget *widget, GdkEventButton *ev, GkrellmPanel *p)
 		b->decal->state = DS_VISIBLE;
 		set_button_index(b, 0, 0);
 		b->decal->modified = TRUE;
+		p->need_decal_overlap_check = TRUE;
 		gkrellm_draw_panel_layers(b->panel);
 		}
 	return FALSE;
@@ -1110,6 +1572,7 @@ gkrellm_make_decal_button(GkrellmPanel *p, GkrellmDecal *d, void (*func)(),
 
 	p->button_list = g_list_append(p->button_list, b);
 	gkrellm_panel_button_signals_connect(p);
+	p->need_decal_overlap_check = TRUE;
 
 	return b;
 	}
@@ -1303,9 +1766,9 @@ put_label_in_button(GkrellmPanel *p, void (*func)(), void *data, gint type,
 	else
 		{
 		x = lbl->x_panel - fr_border->left - pad;
-		y = lbl->y_baseline - lbl->ascent - fr_border->top;
+		y = lbl->y_panel - fr_border->top;
 		w = lbl->width + fr_border->left + fr_border->right + 2 * pad;
-		h = lbl->ascent + lbl->descent + p->textstyle->effect
+		h = lbl->height + (p->textstyle->effect ? 1 : 0)
 				+ fr_border->top + fr_border->bottom;
 		}
 	b = gkrellm_make_overlay_button(p, func, data, x, y, w, h,
@@ -1417,7 +1880,10 @@ gkrellm_make_scaled_decal_pixmap(GkrellmPanel *p, GkrellmPiximage *im,
 	top_win = gkrellm_get_top_window();
 	d = (GkrellmDecal *) g_new0(GkrellmDecal, 1);
 	if (p)
+		{
 		p->decal_list = g_list_append(p->decal_list, d);
+		p->need_decal_overlap_check = TRUE;
+		}
 	d->panel = (gpointer) p;
 
 	if (depth < 1)
