@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -83,7 +83,8 @@ static gint		check_rootpixmap_transparency;
 
 static gboolean	no_transparency,
 				do_intro,
-				decorated;
+				decorated,
+				configure_position_lock;
 
 static void		apply_frame_transparency(gboolean force);
 
@@ -92,22 +93,22 @@ static void		apply_frame_transparency(gboolean force);
 
 static gchar	*fail_large_font[N_FALLBACK_FONTS] =
 {
-"-b&h-lucida-bold-r-normal-*-*-120-*-*-*-*-iso8859-*",
-"-adobe-courier-medium-r-*-*-*-120-*-*-*-*-iso8859-*",
+"Serif 10",
+"Ariel 10",
 "fixed"
 };
 
 static gchar	*fail_normal_font[N_FALLBACK_FONTS] =
 {
-"-b&h-lucida-medium-r-normal-*-*-100-*-*-*-*-iso8859-*",
-"-adobe-courier-medium-r-*-*-*-100-*-*-*-*-iso8859-*",
+"Serif 9",
+"Ariel 9",
 "fixed"
 };
 
 static gchar	*fail_small_font[N_FALLBACK_FONTS] =
 {
-"-adobe-helvetica-medium-r-normal-*-*-80-*-*-*-*-iso8859-*",
-"-adobe-courier-medium-r-*-*-*-80-*-*-*-*-iso8859-*",
+"Serif 8",
+"Ariel 8",
 "fixed"
 };
 
@@ -120,33 +121,30 @@ N_(	"You can configure your monitors by right clicking on\n"
 
 
 static void
-load_font(gchar *font_string, GdkFont **gk_font, gchar **fallback_fonts)
+load_font(gchar *font_string, PangoFontDescription **gk_font,
+				gchar **fallback_fonts)
 	{
-	PangoFontDescription	*font_desc;
-	GdkFont		*(*font_load_func)();
-	GdkFont		*font = NULL;
+	PangoFontDescription	*font_desc = NULL;
 	gint		i;
 
-	if (_GK.use_fontset)
-		font_load_func = gdk_fontset_load;
-	else
-		font_load_func = gdk_font_load;
+	if (font_string)
+		font_desc = pango_font_description_from_string(font_string);
+	if (_GK.debug_level & DEBUG_GUI)
+		printf("load_font: %s %p\n", font_string, font_desc);
 
-	font_desc = pango_font_description_from_string(font_string);
-	if (font_desc)
+	if (!font_desc)
 		{
-		font = gdk_font_from_description(font_desc);
-		pango_font_description_free(font_desc);
-		}
-	if (!font)
-		{
-		font = (*font_load_func)(font_string);
-		for (i = 0; !font && i < N_FALLBACK_FONTS; ++i)
-			font = (*font_load_func)(fallback_fonts[i]);		
+		for (i = 0; !font_desc && i < N_FALLBACK_FONTS; ++i)
+			{
+			font_desc = pango_font_description_from_string(fallback_fonts[i]);
+			if (_GK.debug_level & DEBUG_GUI)
+				printf("load_font trying fallback: %s\n",
+							fallback_fonts[i]);
+			}
 		}
 	if (*gk_font)
-		gdk_font_unref(*gk_font);
-	*gk_font = font;
+		pango_font_description_free(*gk_font);
+	*gk_font = font_desc;
 	}
 
 static void
@@ -164,7 +162,7 @@ setup_fonts()
 	}
 
 void
-map_color_string(gchar *color_string, GdkColor *color)
+gkrellm_map_color_string(gchar *color_string, GdkColor *color)
 	{
 	static GdkColormap	*colormap;
 
@@ -183,15 +181,14 @@ setup_colors()
 	{
 	GtkWidget	*win	= top_window;
 
-	map_color_string(_GK.chart_in_color, &_GK.in_color);
-	map_color_string(_GK.chart_in_color_grid, &_GK.in_color_grid);
+	gkrellm_map_color_string(_GK.chart_in_color, &_GK.in_color);
+	gkrellm_map_color_string(_GK.chart_in_color_grid, &_GK.in_color_grid);
 
-	map_color_string(_GK.chart_out_color, &_GK.out_color);
-	map_color_string(_GK.chart_out_color_grid, &_GK.out_color_grid);
+	gkrellm_map_color_string(_GK.chart_out_color, &_GK.out_color);
+	gkrellm_map_color_string(_GK.chart_out_color_grid, &_GK.out_color_grid);
 
-	map_color_string("#000000", &_GK.background_color);
-
-	map_color_string("#FFFFFF", &_GK.white_color);
+	gkrellm_map_color_string("#000000", &_GK.background_color);
+	gkrellm_map_color_string("#FFFFFF", &_GK.white_color);
 
 	if (_GK.draw1_GC == NULL)
 		{
@@ -253,6 +250,15 @@ set_or_save_position(gint save)
 	path = gkrellm_make_data_file_name(NULL, "startup_position");
 	if (save)
 		{
+		if (!configure_position_lock)
+			{	/* Most recent configure event may have happened before
+				|  gkrellm has removed it's lock, so get gdk's cached values.
+				|  Eg. gkrellm is moved < 2 sec after startup by the window
+				|  manager.  See cb_configure_notify().
+				*/
+			gdk_window_get_position(top_window->window,
+					&_GK.x_position, &_GK.y_position);
+			}
 		if (   (_GK.x_position != x_last || _GK.y_position != y_last)
 			&& (f = fopen(path, "w")) != NULL
 		   )
@@ -261,6 +267,8 @@ set_or_save_position(gint save)
 			y_last = _GK.y_position;
 			fprintf(f, "%d %d\n", _GK.x_position, _GK.y_position);
 			fclose(f);
+			if (_GK.debug_level & DEBUG_POSITION)
+				printf("save_position: %d %d\n", x_last, y_last);
 			}
 		save_position_countdown = 0;
 		}
@@ -278,7 +286,10 @@ set_or_save_position(gint save)
 				{
 				_GK.x_position = x_last = x;
 				_GK.y_position = y_last = y;
+				_GK.position_valid = TRUE;
 				gdk_window_move(gtree.window->window, x, y);
+				if (_GK.debug_level & DEBUG_POSITION)
+					printf("startup_position moveto %d %d (valid)\n", x, y);
 				}
 			}
 		}
@@ -325,7 +336,7 @@ update_monitors()
 		GK.day_tick = FALSE;
 		}
 
-	update_alerts();
+	gkrellm_alert_update();
 	for (list = gkrellm_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmMonitor *) list->data;
@@ -338,6 +349,9 @@ update_monitors()
 
 	if (save_position_countdown > 0 && --save_position_countdown == 0)
 		set_or_save_position(1);
+	if (configure_position_lock && (GK.timer_ticks / _GK.update_HZ) > 1)
+		configure_position_lock = FALSE;
+
 
 	/* Update if background has changed */
 	if (   GK.second_tick && !no_transparency
@@ -362,13 +376,13 @@ update_monitors()
 	if (   GK.minute_tick && _GK.config_modified
 		&& !gkrellm_config_window_shown()
 	   )
-		save_user_config();
+		gkrellm_save_user_config();
 
 	return TRUE;			/* restarts timeout */
 	}
 
 void
-start_timer(gint Hz)
+gkrellm_start_timer(gint Hz)
 	{
 	static gint	timeout_id	= 0;
 	gint		interval;
@@ -409,7 +423,7 @@ gkrellm_get_timer_ticks(void)
   |  callers to increment height fields to allow for the offset shadow draw.
   */
 gint
-effect_string_value(gchar *effect_string)
+gkrellm_effect_string_value(gchar *effect_string)
 	{
 	if (effect_string)
 		return (strcmp(effect_string, "shadow") == 0) ? 1 : 0;
@@ -498,7 +512,7 @@ gkrellm_draw_text(GdkDrawable *drawable, GkrellmTextstyle *ts, gint x, gint y,
 	}
 
 gint
-x_label_position(gint position, gint w_field, gint w_label, gint margin)
+gkrellm_label_position(gint position, gint w_field, gint w_label, gint margin)
 	{
 	gint	x;
 
@@ -560,9 +574,10 @@ gkrellm_motion(GtkWidget *widget, GdkEventMotion *ev, gpointer data)
 
 		/* Moves to x,y position relative to root.
 		*/
-		gdk_window_move(top_window->window, x, y);
 		_GK.y_position = y;
 		_GK.x_position = x;
+		_GK.position_valid = TRUE;
+		gdk_window_move(top_window->window, x, y);
 		}
 	}
 
@@ -579,6 +594,9 @@ top_frame_button_release(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	if (!no_transparency)
 		gkrellm_winop_apply_rootpixmap_transparency();
 	moving_gkrellm = FALSE;
+	if (_GK.debug_level & DEBUG_POSITION)
+		printf("gkrellm moveto: x_pos=%d y_pos=%d\n",
+				_GK.x_position, _GK.y_position);
 	}
 
 static gboolean
@@ -600,7 +618,7 @@ top_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	*/
 	time(&time_check);
 	if (time_check > _GK.time_now + 2 || time_check < _GK.time_now)
-		start_timer(_GK.update_HZ);
+		gkrellm_start_timer(_GK.update_HZ);
 
 	if (ev->button == 3)
 		{
@@ -626,6 +644,7 @@ top_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 			y_press_event += _GK.frame_top_height;
 			}
 		moving_gkrellm = TRUE;
+		configure_position_lock = FALSE;
 		}
 	return FALSE;
 	}
@@ -1104,7 +1123,7 @@ gkrellm_pack_side_frames(void)
 	static GtkWidget	*lf_image, *rf_image;
 	gint				x_gkrell, y_gkrell, y_bottom;
 	gint				w, h;
-	gint				was_on_bottom;
+	gboolean			was_on_bottom;
 
 	if (   ! _GK.initialized
 		|| (   monitor_previous_height == _GK.monitor_height
@@ -1120,6 +1139,7 @@ gkrellm_pack_side_frames(void)
 		_GK.frame_right_width = 0;
 		return;
 		}
+
 	gdk_window_get_origin(gtree.window->window, &x_gkrell, &y_gkrell);
 	gdk_drawable_get_size(gtree.window->window, &w, &h);
 	was_on_bottom = (y_gkrell + h >= _GK.h_display) ? TRUE : FALSE;
@@ -1163,9 +1183,14 @@ gkrellm_pack_side_frames(void)
 	y_bottom = _GK.h_display - (_GK.y_position + h);
 	if (y_bottom >= 0)
 		{
-		if (_GK.y_position != y_gkrell)
+		if (_GK.y_position != y_gkrell && _GK.position_valid)
+			{
 			gdk_window_move(gtree.window->window,
 							_GK.x_position, _GK.y_position);
+			if (_GK.debug_level & DEBUG_POSITION)
+				printf("pack moveto %d %d=y_position (y_gkrell=%d y_bot=%d)\n",
+						_GK.x_position, _GK.y_position, y_gkrell, y_bottom);
+			}
 		y_pack = -1;
 		}
 	else	/* Otherwise, do a y move to maximize visibility */
@@ -1178,7 +1203,13 @@ gkrellm_pack_side_frames(void)
 			{
 			if ((y_pack = y_gkrell + y_bottom) < 0)
 				y_pack = 0;
-			gdk_window_move(gtree.window->window, _GK.x_position, y_pack);
+			if (_GK.position_valid)
+				{
+				gdk_window_move(gtree.window->window, _GK.x_position, y_pack);
+				if (_GK.debug_level & DEBUG_POSITION)
+					printf("pack moveto %d %d=y_pack (y_gkrell=%d y_bot=%d)\n",
+							_GK.x_position, y_pack, y_gkrell, y_bottom);
+				}
 			}
 		/* If GKrellM bottom edge was <= screen bottom, then move to make
 		|  resized GKrellM still on bottom whether window has shrunk or grown.
@@ -1187,7 +1218,13 @@ gkrellm_pack_side_frames(void)
 			{
 			if ((y_pack = _GK.h_display - h) < 0)
 				y_pack = 0;
-			gdk_window_move(gtree.window->window, _GK.x_position, y_pack);
+			if (_GK.position_valid)
+				{
+				gdk_window_move(gtree.window->window, _GK.x_position, y_pack);
+				if (_GK.debug_level & DEBUG_POSITION)
+					printf("pack moveto %d %d=y_pack (y_gkrell=%d on_bottom)\n",
+							_GK.x_position, y_pack, y_gkrell);
+				}
 			}
 		}
 	monitor_previous_height = _GK.monitor_height;
@@ -1214,6 +1251,8 @@ fix_edges()
 	{
 	gint	x, y, w, h, x0, y0, y_bottom;
 
+	if (!_GK.position_valid)
+		return;
 	gdk_window_get_origin(gtree.window->window, &x, &y);
 	w = _GK.chart_width + _GK.frame_left_width + _GK.frame_right_width;
 	h = _GK.monitor_height + _GK.total_frame_height;
@@ -1242,6 +1281,8 @@ fix_edges()
 		gdk_window_move(gtree.window->window, x, y);
 		if (y != _GK.y_position)
 			y_pack = y;
+		if (_GK.debug_level & DEBUG_POSITION)
+			printf("fix_edges: %d %d (y_pos=%d)\n", x, y, _GK.y_position);
 		}
 	}
 
@@ -1600,10 +1641,27 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 	w_prev = w;
 	h_prev = h;
 
-	if (x >= 0 && x < _GK.w_display - 10)
-		_GK.x_position = x;
-	if (y >= 0 && y < _GK.h_display - 5 && y != y_pack)
-		_GK.y_position = y;
+	/* At startup can be a race for configure events to be reporting correct
+	|  move to position and pack_side_frames() first call.  Initial configure
+	|  events will be intermediate x,y values as intial mapping happens, so
+	|  using configure event position values is delayed.
+	*/
+	if (!configure_position_lock)
+		{
+		if (x >= 0 && x < _GK.w_display - 10)
+			_GK.x_position = x;
+		if (y >= 0 && y < _GK.h_display - 5 && y != y_pack)
+			_GK.y_position = y;
+		_GK.position_valid = TRUE;
+		if (!moving_gkrellm && (_GK.debug_level & DEBUG_POSITION))
+			printf("configure-event: x_pos=%d y_pos=%d x=%d y=%d y_pack=%d\n",
+						_GK.x_position, _GK.y_position, x, y, y_pack);
+		}
+	else if (_GK.debug_level & DEBUG_POSITION)
+			printf("locked configure-event: x=%d y=%d\n", x, y);
+
+	if (size_change || position_change)
+		gkrellm_winop_update_struts();
 
 	if (do_intro)
 		{
@@ -1623,13 +1681,13 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 void
 gkrellm_save_all()
 	{
-	save_net_data();
-	save_inet_data();
+	gkrellm_net_save_data();
+	gkrellm_inet_save_data();
 	if (_GK.save_position)
 		set_or_save_position(1);
 	if (_GK.config_modified)
-		save_user_config();
-	save_theme_config();
+		gkrellm_save_user_config();
+	gkrellm_save_theme_config();
 	}
 
 static void
@@ -1637,12 +1695,12 @@ check_gkrellm_directories(void)
 	{
 	gchar	*t, *u;
 
-	if (make_home_subdir(GKRELLM_DIR, NULL))
+	if (gkrellm_make_home_subdir(GKRELLM_DIR, NULL))
 		do_intro = TRUE;	/* Defer it until main window realizes */
 
-	make_home_subdir(GKRELLM_PLUGINS_DIR, NULL);
-	make_home_subdir(GKRELLM_THEMES_DIR, NULL);
-	make_home_subdir(GKRELLM_DATA_DIR, NULL);
+	gkrellm_make_home_subdir(GKRELLM_PLUGINS_DIR, NULL);
+	gkrellm_make_home_subdir(GKRELLM_THEMES_DIR, NULL);
+	gkrellm_make_home_subdir(GKRELLM_DATA_DIR, NULL);
 
 	/* If no user_config specified, force a check for host-specific config
 	*/
@@ -1734,7 +1792,7 @@ gkrellm_monitor_vbox(void)
 	}
 
 void
-build_gkrellm()
+gkrellm_build()
 	{
 	GkrellmMonitor		*mon;
 	GList		*list;
@@ -1745,20 +1803,20 @@ build_gkrellm()
 	_GK.total_frame_height = 0;
 	_GK.any_transparency = FALSE;
 	gkrellm_winop_reset();
-	start_timer(0);
+	gkrellm_start_timer(0);
 
 	if (!first_create)
 		edge_record();
-	reset_all_alerts();
-	clean_out_panels();
+	gkrellm_alert_reset_all();
+	gkrellm_panel_cleanup();
 
-	gkrellmrc_theme_config();
+	gkrellm_theme_config();
 	if (first_create)
-		load_user_config(TRUE);
+		gkrellm_load_user_config(TRUE);
 	setup_colors();
 	setup_fonts();
-	load_theme_piximages();
-	setup_charts();
+	gkrellm_load_theme_piximages();
+	gkrellm_chart_setup();
 	gkrellm_freeze_side_frame_packing();
 
 	create_frame_top(gtree.top0_vbox);
@@ -1801,7 +1859,7 @@ build_gkrellm()
 
 	_GK.initialized = TRUE;
 	gkrellm_thaw_side_frame_packing();
-	start_timer(_GK.update_HZ);
+	gkrellm_start_timer(_GK.update_HZ);
 	if (!first_create)
 		{
 		fix_edges();
@@ -1839,23 +1897,23 @@ load_builtin_monitors()
 	|  have an apply which needs to be called before the cpu or proc apply.
 	|  So just put sensors config first.
 	*/
-	add_builtin(init_sensors_config_monitor());
-	add_builtin(init_host_monitor());
-	add_builtin(init_cal_monitor());
-	add_builtin(init_clock_monitor());
-	add_builtin(init_cpu_monitor());
-	add_builtin(init_proc_monitor());
-	add_builtin(init_sensor_monitor());
-	add_builtin(init_disk_monitor());
-	add_builtin(init_inet_monitor());
-	add_builtin(init_net_monitor());
-	add_builtin(init_timer_monitor());
-	add_builtin(init_mem_monitor());
-	add_builtin(init_swap_monitor());
-	add_builtin(init_fs_monitor());
-	add_builtin(init_mail_monitor());
-	add_builtin(init_battery_monitor());
-	add_builtin(init_uptime_monitor());
+	add_builtin(gkrellm_init_sensors_config_monitor());
+	add_builtin(gkrellm_init_host_monitor());
+	add_builtin(gkrellm_init_cal_monitor());
+	add_builtin(gkrellm_init_clock_monitor());
+	add_builtin(gkrellm_init_cpu_monitor());
+	add_builtin(gkrellm_init_proc_monitor());
+	add_builtin(gkrellm_init_sensor_monitor());
+	add_builtin(gkrellm_init_disk_monitor());
+	add_builtin(gkrellm_init_inet_monitor());
+	add_builtin(gkrellm_init_net_monitor());
+	add_builtin(gkrellm_init_timer_monitor());
+	add_builtin(gkrellm_init_mem_monitor());
+	add_builtin(gkrellm_init_swap_monitor());
+	add_builtin(gkrellm_init_fs_monitor());
+	add_builtin(gkrellm_init_mail_monitor());
+	add_builtin(gkrellm_init_battery_monitor());
+	add_builtin(gkrellm_init_uptime_monitor());
 	}
 
 
@@ -2095,7 +2153,7 @@ main(gint argc, gchar **argv)
 		exit(0);
 
 	check_gkrellm_directories();
-	load_user_config(FALSE);		
+	gkrellm_load_user_config(FALSE);		
 	decorated = (_GK.command_line_decorated || _GK.decorated);
 	if (   _GK.command_line_plugin || _GK.command_line_theme
 		|| _GK.debug_level > 0 || _GK.debug > 0 || _GK.nolock
@@ -2107,10 +2165,10 @@ main(gint argc, gchar **argv)
 	top_window = gtree.window;
 
 	load_builtin_monitors();
-	load_plugin_monitors();
+	gkrellm_plugins_load();
 
-	build_gkrellm();
-	make_themes_list();
+	gkrellm_build();
+	gkrellm_make_themes_list();
 
 	if ((_GK.gtk_settings = gtk_settings_get_default()) != NULL)
 		{
@@ -2151,20 +2209,23 @@ main(gint argc, gchar **argv)
 	g_signal_connect(G_OBJECT(gtree.right_event_box), "button_press_event",
 				G_CALLBACK(side_frame_button_press), NULL );
 
-	item_factory = create_item_factory_popup();
+	item_factory = gkrellm_create_item_factory_popup();
 
 	if (_GK.sticky_state)
 		gtk_window_stick(GTK_WINDOW(top_window));
 	gkrellm_winop_options(argc, argv);
 	gtk_widget_show(gtree.window);
+	gkrellm_winop_withdrawn();
+
+	if (geometry || _GK.save_position)
+		configure_position_lock = TRUE;		/* see cb_configure_notify */
 
 	if (geometry)		/* Command line placement overrides */
 		gkrellm_winop_place_gkrellm(geometry);
 	else if (_GK.save_position)
 		set_or_save_position(0);
 
-
-	start_timer(_GK.update_HZ);
+	gkrellm_start_timer(_GK.update_HZ);
 	setup_signal_handler();
 	gtk_main ();
 	gkrellm_save_all();
