@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -275,14 +275,13 @@ draw_decal_label(MeminfoMeter *mm, gint draw_to_screen)
 	{
 	GkrellmDecal		*d;
 	GkrellmTextstyle	ts_save;
-	gchar		buf[128];
-	gint		changed;
-	gint		w	= 0;
+	gchar				buf[128];
+	gint				x_off, w;
 
 	d = mm->decal_label;
 	if (! mm->label_is_data)
 		{
-		d->x_off = mm->x_label;
+		gkrellm_decal_text_set_offset(d, mm->x_label, 0);
 		gkrellm_draw_decal_text(mm->panel, d, mm->label, 0);
 		}
     else
@@ -290,23 +289,15 @@ draw_decal_label(MeminfoMeter *mm, gint draw_to_screen)
 		ts_save = d->text_style;
 		d->text_style = *gkrellm_meter_alt_textstyle(mm->style_id);
 
-		changed = format_meminfo_data(mm, mm->data_format_shadow,
-					buf, sizeof(buf));
-		w = gkrellm_gdk_string_width(d->text_style.font, buf);
-
+		format_meminfo_data(mm, mm->data_format_shadow, buf, sizeof(buf));
+		gkrellm_decal_scroll_text_set_markup(mm->panel, d, buf);
+		gkrellm_decal_scroll_text_get_size(d, &w, NULL);
 		if (w  > d->w)
-			{
-			d->x_off = d->w / 3 - x_scroll;
-			changed = x_scroll + 1;
-			}
+			x_off = d->w / 3 - x_scroll;
 		else
-			d->x_off = 0;
+			x_off = 0;
+		gkrellm_decal_text_set_offset(d, x_off, 0);
 
-		/* Draw the decal, actual draws occur only if "changed" is different
-		|  from last call.  If scrolling, will be so each time, if not
-		|  will be so as format_meminfo_data() return value changes.
-		*/
-		gkrellm_draw_decal_text(mm->panel, d, buf, changed);
 		d->text_style = ts_save;
 		}
 	if (draw_to_screen)
@@ -382,7 +373,7 @@ format_chart_text(MeminfoChart *mc, gchar *buf, gint size)
 					}
 				}
 			if (blocks >= 0)
-				len = format_size_abbrev(buf, size, (gfloat) blocks,
+				len = gkrellm_format_size_abbrev(buf, size, (gfloat) blocks,
 						&swap_blocks_abbrev[0], tbl_size);
 			++s;
 			}
@@ -608,7 +599,7 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 	{
 	GdkModifierType	state;
 	GkrellmDecal	*d;
-	GdkFont			*font;
+	PangoFontDescription *font_desc;
 	gchar			buf[128];
 	gint			w, x_delta;
 
@@ -622,11 +613,11 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 		return FALSE;
 		}
 	d = mon_in_motion->decal_label;
-	font = gkrellm_meter_alt_textstyle(mon_in_motion->style_id)->font;
+	font_desc = gkrellm_meter_alt_textstyle(mon_in_motion->style_id)->font;
 
 	format_meminfo_data(mon_in_motion, mon_in_motion->data_format_shadow,
 				buf, sizeof(buf));
-	w = gkrellm_gdk_string_width(font, buf);
+	w = gkrellm_gdk_string_width(font_desc, buf);
 	if (w > d->w)
 		{
 		x_delta = ev->x - x_mon_motion;
@@ -746,7 +737,7 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 	GkrellmPanel	*p;
 	GkrellmTextstyle *ts;
 	GkrellmStyle	*s;
-	gchar			*expand, buf[64];
+	gchar			*expand, buf[64], *u;
 	gint			w_label;
 
 	mm = &mem;
@@ -808,7 +799,13 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 	if (s->label_position == GKRELLM_LABEL_NONE)
 		mem.label = g_strdup("");
 	else
-		mem.label = g_locale_from_utf8(_("Mem"), -1, NULL, NULL, NULL);
+		mem.label = g_strdup(_("Mem"));
+	if (!g_utf8_validate(mem.label, -1, NULL))
+		{
+		u = g_locale_to_utf8(mem.label, -1, NULL, NULL, NULL);
+		g_free(mem.label);
+		mem.label = u;
+		}
 
 	snprintf(buf, sizeof(buf), "%sMemfj8", mem.label);
 	mm->decal_label = gkrellm_create_decal_text(p, buf,
@@ -818,7 +815,7 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 
 	ts = &mm->decal_label->text_style;
 	w_label = gkrellm_gdk_string_width(ts->font, mm->label) + ts->effect;
-	mm->x_label = x_label_position(p->label->position, mm->decal_label->w,
+	mm->x_label = gkrellm_label_position(p->label->position, mm->decal_label->w,
 				w_label, 0);
 	draw_decal_label(mm, 0);
 
@@ -845,7 +842,7 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 	GkrellmPanel	*p;
 	GkrellmStyle	*style, *panel_style;
 	GkrellmTextstyle *ts;
-	gchar			buf[64];
+	gchar			buf[64], *u;
 	gint			w_label;
 
 	mm = &swap;
@@ -874,7 +871,13 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 	if (style->label_position == GKRELLM_LABEL_NONE)
 		swap.label = g_strdup("");
 	else
-		swap.label = g_locale_from_utf8(_("Swap"), -1, NULL, NULL, NULL);
+		swap.label = g_strdup(_("Swap"));
+	if (!g_utf8_validate(swap.label, -1, NULL))
+		{
+		u = g_locale_to_utf8(swap.label, -1, NULL, NULL, NULL);
+		g_free(swap.label);
+		swap.label = u;
+		}
 
 	snprintf(buf, sizeof(buf), "%sMemfj8", swap.label);
 	mm->decal_label = gkrellm_create_decal_text(p, buf,
@@ -885,7 +888,7 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 
 	ts = &mm->decal_label->text_style;
 	w_label = gkrellm_gdk_string_width(ts->font, mm->label) + ts->effect;
-	mm->x_label = x_label_position(p->label->position, mm->decal_label->w,
+	mm->x_label = gkrellm_label_position(p->label->position, mm->decal_label->w,
 				w_label, 0);
 	draw_decal_label(mm, 0);
 
@@ -1160,6 +1163,15 @@ cb_mem_format(GtkWidget *widget, gpointer data)
 	gchar	*s;
 
 	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(mem_format_combo)->entry));
+
+	/* In case Pango markup tags, don't accept line unless valid markup.
+	|  Ie, markup like <span ...> xxx </span> or <b> xxx </b>
+	*/
+	if (   strchr(s, '<') != NULL
+		&& !pango_parse_markup(s, -1, 0, NULL, NULL, NULL, NULL)
+	   )
+		return;
+
 	if (gkrellm_locale_dup_string(&mem.data_format, s,
 				&mem.data_format_shadow))
 		mem.decal_label->value = -1;	/* Force redraw */
@@ -1171,6 +1183,12 @@ cb_swap_format(GtkWidget *widget, gpointer data)
 	gchar	*s;
 
 	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(swap_format_combo)->entry));
+
+	if (   strchr(s, '<') != NULL
+		&& !pango_parse_markup(s, -1, 0, NULL, NULL, NULL, NULL)
+	   )
+		return;
+
 	if (gkrellm_locale_dup_string(&swap.data_format, s,
 				&swap.data_format_shadow))
 		swap.decal_label->value = -1;
@@ -1300,6 +1318,7 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	list = g_list_append(list,
 				"\\ww\\C\\f$M\\D3\\f\\ai\\.$i\\D0\\f\\ao\\.$o");
 	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
 			text_format);
@@ -1319,6 +1338,7 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	list = g_list_append(list, _("$t - $U"));
 	list = g_list_append(list, _("$t - $u used  $s sh  $b bf  $c ca"));
 	gtk_combo_set_popdown_strings(GTK_COMBO(mem_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(mem_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mem_format_combo)->entry),
 			mem.data_format);
@@ -1337,6 +1357,7 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	list = g_list_append(list, _("$t - $u used"));
 	list = g_list_append(list, _("$t - $U"));
 	gtk_combo_set_popdown_strings(GTK_COMBO(swap_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(swap_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(swap_format_combo)->entry),
 			swap.data_format);
@@ -1403,7 +1424,7 @@ static GkrellmMonitor	monitor_mem =
 	};
 
 GkrellmMonitor *
-init_mem_monitor(void)
+gkrellm_init_mem_monitor(void)
 	{
 	monitor_mem.name = _(monitor_mem.name);
 	mon_mem = &monitor_mem;
@@ -1439,7 +1460,7 @@ static GkrellmMonitor	monitor_swap =
 	};
 
 GkrellmMonitor *
-init_swap_monitor(void)
+gkrellm_init_swap_monitor(void)
 	{
 	mon_swap = &monitor_swap;
 	swap.style_id = gkrellm_add_meter_style(mon_swap, SWAP_STYLE_NAME);
