@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -83,7 +83,8 @@ typedef struct
 						totals_shown,
 						reset_button_in,
 						stats_button_in,
-						mouse_in_chart;
+						mouse_in_chart,
+						new_text_format;
 	GkrellmLauncher		launch;
 	GtkWidget			*launch_entry,
 						*tooltip_entry;
@@ -527,7 +528,7 @@ add_net_stat(GtkWidget *table, gdouble v, gint c0, gint c1, gint r0, gint r1)
 	GtkWidget	*label;
 	gchar		buf[64];
 
-	format_size_abbrev(buf, sizeof(buf), (gfloat) v,
+	gkrellm_format_size_abbrev(buf, sizeof(buf), (gfloat) v,
 				&stats_bytes_abbrev[0],
 				sizeof(stats_bytes_abbrev) / sizeof(GkrellmSizeAbbrev));
 	label = gtk_label_new(buf);
@@ -901,7 +902,7 @@ timed_net_visibility(void)
 		gkrellm_chart_hide(net->chart, FALSE);  /* Don't hide the panel */
 
 	if (!net_is_up && chart_is_visible)
-		save_net_data();
+		gkrellm_net_save_data();
 	}
 
 static gint
@@ -1136,12 +1137,12 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 					}
 				}
 			if (bytes >= 0)
-				len = format_size_abbrev(buf, size, (gfloat) bytes,
+				len = gkrellm_format_size_abbrev(buf, size, (gfloat) bytes,
 					&current_bytes_abbrev[0],
 					sizeof(current_bytes_abbrev) / sizeof(GkrellmSizeAbbrev));
 			else if (fbytes >= 0)
 				{
-				len = format_size_abbrev(buf, size, (gfloat) fbytes,
+				len = gkrellm_format_size_abbrev(buf, size, (gfloat) fbytes,
 					&total_bytes_abbrev[0],
 					sizeof(total_bytes_abbrev) /sizeof(GkrellmSizeAbbrev));
 				if (!day && !week && !month)
@@ -1168,6 +1169,9 @@ draw_net_chart_labels(NetMon *net)
 	if (!net->chart_labels)
 		return;
 	format_net_data(net, text_format_locale, buf, sizeof(buf));
+	if (!net->new_text_format)
+		gkrellm_chart_reuse_text_format(cp);
+	net->new_text_format = FALSE;
 	gkrellm_draw_chart_text(cp, net_style_id, buf);
 	}
 
@@ -1397,6 +1401,7 @@ create_net_monitor(GtkWidget *vbox, NetMon *net, gint first_create)
 
 	setup_net_scaling(cp->config, net);
 
+	net->new_text_format = TRUE;
 	if (first_create)
 		{
 		g_signal_connect(G_OBJECT (cp->drawing_area), "expose_event",
@@ -1464,7 +1469,7 @@ net_accounting_month_new(struct tm *tm)
 	}
 
 void
-save_net_data(void)
+gkrellm_net_save_data(void)
 	{
 	FILE		*f;
 	GList		*list;
@@ -1659,8 +1664,7 @@ create_net_timer(GtkWidget *vbox, gint first_create)
 			time_decal->y += (button_decal->h - time_decal->h) / 2;
 		}
 
-	seconds_decal->y = time_decal->y + time_decal->y_baseline
-							- seconds_decal->y_baseline;
+	seconds_decal->y = time_decal->y + time_decal->h - seconds_decal->h;
 	gkrellm_panel_create(vbox, mon_timer, p);
 
 	gkrellm_move_decal(p, seconds_decal,
@@ -1686,6 +1690,9 @@ create_net_timer(GtkWidget *vbox, gint first_create)
 		y = time_decal->y - tb->top;
 		gkrellm_paste_piximage(bg_timer_piximage, p->pixmap, x, y, w, h);
 		gkrellm_paste_piximage(bg_timer_piximage, p->bg_pixmap, x, y, w, h);
+		gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC, p->bg_pixmap,
+                0, 0,  0, 0,  p->w, p->h);
+
 		}
 
 	set_timer_button_state(timer_button_state);
@@ -1837,7 +1844,7 @@ update_net(void)
 		{
 		tm = gkrellm_get_current_time();
 		if ((tm->tm_hour % 6) == 0)
-			save_net_data();
+			gkrellm_net_save_data();
 		}
 	}
 
@@ -2269,12 +2276,17 @@ static void
 cb_text_format(GtkWidget *widget, gpointer data)
 	{
 	GList   *list;
+	NetMon	*net;
 	gchar   *s;
 
 	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	for (list = net_mon_list; list; list = list->next)
-		refresh_net_chart((NetMon *) list->data);
+		{
+		net = (NetMon *) list->data;
+		net->new_text_format = TRUE;
+		refresh_net_chart(net);
+		}
 	}
 
 static void
@@ -2556,8 +2568,7 @@ create_net_tab(GtkWidget *tab_vbox)
 	if (!_GK.client_mode)
 		{
 		timer_iface_combo = gtk_combo_new();
-		gtk_box_pack_start(GTK_BOX(hbox), timer_iface_combo,
-				FALSE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(hbox), timer_iface_combo, FALSE, TRUE, 0);
 		list = NULL;
 		list = g_list_append(list, _("none"));
 		for (tlist = timer_defaults_list; tlist; tlist = tlist->next)
@@ -2565,11 +2576,10 @@ create_net_tab(GtkWidget *tab_vbox)
 			tt = (TimerType *) tlist->data;
 			list = g_list_append(list, tt->name);
 			}
-		gtk_combo_set_popdown_strings(
-					GTK_COMBO(timer_iface_combo), list);
+		gtk_combo_set_popdown_strings(GTK_COMBO(timer_iface_combo), list);
+		gtk_combo_set_case_sensitive(GTK_COMBO(timer_iface_combo), TRUE);
 		g_list_free(list);
-		gtk_entry_set_text(
-					GTK_ENTRY(GTK_COMBO(timer_iface_combo)->entry),
+		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(timer_iface_combo)->entry),
 					timer_button_iface);
 		g_signal_connect(G_OBJECT(GTK_COMBO(timer_iface_combo)->entry),
 					"changed", G_CALLBACK(cb_timer_iface), NULL);
@@ -2676,6 +2686,7 @@ create_net_tab(GtkWidget *tab_vbox)
 	list = g_list_append(list,
 		_("\\f\\ww\\c\\f$M\\D2\\f\\ar\\.$r\\D1\\f\\at\\.$t\\b\\c\\f$L"));
 	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
 			text_format);
@@ -2726,7 +2737,7 @@ static GkrellmMonitor	monitor_net =
 	};
 
 GkrellmMonitor *
-init_net_monitor(void)
+gkrellm_init_net_monitor(void)
 	{
 	TimerType	*tt;
 
@@ -2782,7 +2793,7 @@ static GkrellmMonitor	monitor_timer =
 	};
 
 GkrellmMonitor *
-init_timer_monitor(void)
+gkrellm_init_timer_monitor(void)
 	{
 	if (!mon_net)
 		return NULL;
