@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -65,6 +65,7 @@ setup_panel_style(GkrellmPanel *p, GkrellmStyle *style)
 		return;
 	lbl->position = style->label_position;
 	p->transparency = style->transparency;
+	p->scroll_text_cache_off = style->scroll_text_cache_off;
 	_GK.any_transparency |= p->transparency;
 	if (!p->textstyle)
 		{
@@ -93,8 +94,7 @@ gkrellm_panel_configure(GkrellmPanel *p, gchar *string, GkrellmStyle *style)
 	GkrellmDecal	*d;
 	GkrellmLabel	*lbl;
 	GkrellmTextstyle *ts = NULL;
-	gchar			*buf;
-	gint			y, h_panel, h;
+	gint			y, h_panel, h, baseline;
 	gint			top_margin = 0, bottom_margin = 0;
 
 	if (!p)
@@ -107,29 +107,27 @@ gkrellm_panel_configure(GkrellmPanel *p, gchar *string, GkrellmStyle *style)
 
 	ts = p->textstyle;
 
-	if (string && g_utf8_validate(string, -1, NULL))
+	if (string)
 		{
-		buf = g_locale_from_utf8(string, -1, NULL, NULL, NULL);
-		gkrellm_dup_string(&lbl->string, buf);
-		g_free(buf);
+		if (g_utf8_validate(string, -1, NULL))
+			gkrellm_dup_string(&lbl->string, string);
+		else
+			lbl->string = g_locale_to_utf8(string, -1, NULL, NULL, NULL);
 		}
 	else
 		gkrellm_dup_string(&lbl->string, string);
 
 	if (lbl->string && ts && lbl->position >= 0)
-		gkrellm_winop_text_extents(ts->font, string, strlen(string),
-				&lbl->lbearing, &lbl->rbearing,
-				&lbl->width, &lbl->ascent, &lbl->descent);
+		gkrellm_text_extents(ts->font, string, strlen(string),
+				&lbl->width, &lbl->height, &baseline, NULL);
 	else
 		{
-		lbl->lbearing = 0;
-		lbl->rbearing = 0;
 		lbl->width = 0;
-		lbl->ascent = 0;
-		lbl->descent = 0;
+		lbl->height = 0;
+		baseline = 0;
 		}
 	h_panel = top_margin + bottom_margin;
-	h_panel += lbl->ascent + lbl->descent + (ts ? ts->effect : 0);
+	h_panel += lbl->height + (ts ? ts->effect : 0);
 
 	/* If krell_yoff is -1, then in gkrellm_create_krell() k->y0 was put
 	|  at the top margin.  If krell_yoff is < -1, then here I will bottom
@@ -175,7 +173,7 @@ gkrellm_panel_configure(GkrellmPanel *p, gchar *string, GkrellmStyle *style)
 	if (h_panel <= 0)
 		h_panel = 1;
 
-	lbl->y_baseline = top_margin + lbl->ascent;
+	lbl->y_panel = top_margin - (baseline - lbl->height);
 	p->h_configure = h_panel;
 	}
 
@@ -195,11 +193,11 @@ draw_panel_label(GkrellmPanel *p, gboolean to_bg)
 		{
 		m = gkrellm_get_style_margins(p->style);
 		lbl->width = gkrellm_gdk_string_width(ts->font, s) + ts->effect;
-		xdst = x_label_position(lbl->position, p->w, lbl->width, m->left);
+		xdst = gkrellm_label_position(lbl->position, p->w, lbl->width, m->left);
 		lbl->x_panel = xdst;
-		gkrellm_draw_string(p->pixmap, ts, xdst, lbl->y_baseline, s);
+		gkrellm_draw_string(p->pixmap, ts, xdst, lbl->y_panel, s);
 		if (to_bg)
-			gkrellm_draw_string(p->bg_pixmap, ts, xdst, lbl->y_baseline, s);
+			gkrellm_draw_string(p->bg_pixmap, ts, xdst, lbl->y_panel, s);
 		}
 	}
 
@@ -213,6 +211,10 @@ draw_panel(GkrellmPanel *p, gint to_screen)
 	gdk_draw_drawable(p->pixmap, _GK.draw1_GC, p->bg_pixmap,
 					0, 0, 0, 0, p->w, p->h);
 	draw_panel_label(p, TRUE);
+
+	gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC, p->bg_pixmap,
+					0, 0, 0, 0, p->w, p->h);
+
 	if (p->drawing_area->window && to_screen)
 		{
 		gdk_draw_drawable(p->drawing_area->window, _GK.draw1_GC, p->pixmap,
@@ -252,12 +254,16 @@ gkrellm_panel_destroy(GkrellmPanel *p)
 			g_free(p->label->string);
 		g_free(p->label);
 		}
+	if (p->layout)
+		g_object_unref(G_OBJECT(p->layout));
 	if (p->textstyle && p->textstyle->internal)
 		g_free(p->textstyle);
 	if (p->pixmap)
 		g_object_unref(G_OBJECT(p->pixmap));
 	if (p->bg_pixmap)
 		g_object_unref(G_OBJECT(p->bg_pixmap));
+	if (p->bg_text_layer_pixmap)
+		g_object_unref(G_OBJECT(p->bg_text_layer_pixmap));
 	if (p->bg_clean_pixmap)
 		g_object_unref(G_OBJECT(p->bg_clean_pixmap));
 	if (p->bg_mask)
@@ -345,6 +351,7 @@ gkrellm_panel_create(GtkWidget *vbox, GkrellmMonitor *mon, GkrellmPanel *p)
 		gtk_container_add (GTK_CONTAINER(vbox), hbox);
 		p->hbox = hbox;
 		p->drawing_area = gtk_drawing_area_new();
+		p->layout = gtk_widget_create_pango_layout(top_win, NULL);
 		gtk_widget_set_events (p->drawing_area, GDK_EXPOSURE_MASK
 				| GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK
 				| GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK
@@ -387,6 +394,9 @@ gkrellm_panel_create(GtkWidget *vbox, GkrellmMonitor *mon, GkrellmPanel *p)
 //		gkrellm_scale_piximage_to_pixmap(p->bg_piximage, &p->bg_clean_pixmap,
 //			&p->bg_mask, p->w, p->h);
 
+	if (p->bg_text_layer_pixmap)
+		g_object_unref(G_OBJECT(p->bg_text_layer_pixmap));
+	p->bg_text_layer_pixmap = gdk_pixmap_new(top_win->window, p->w, p->h, -1);
 	if (p->bg_pixmap)
 		g_object_unref(G_OBJECT(p->bg_pixmap));
 	p->bg_pixmap = gdk_pixmap_new(top_win->window, p->w, p->h, -1);
@@ -399,6 +409,7 @@ gkrellm_panel_create(GtkWidget *vbox, GkrellmMonitor *mon, GkrellmPanel *p)
 		gkrellm_monitor_height_adjust(p->h);
 		gkrellm_pack_side_frames();
 		}
+	p->need_decal_overlap_check = TRUE;
 	draw_panel(p, FALSE);
 	gkrellm_panel_button_signals_connect(p);
 	}
@@ -461,7 +472,7 @@ gkrellm_panel_enable_visibility(GkrellmPanel *p, gboolean new_vis,
   |  the new gkrellm_panel_create() functions.
   */
 void
-clean_out_panels(void)
+gkrellm_panel_cleanup(void)
 	{
 	GList	*list, *list1;
 	GkrellmPanel	*p;
@@ -475,7 +486,7 @@ clean_out_panels(void)
 			gkrellm_destroy_decal_list(p);	/* Also destroys buttons */
 			if (p->krell_list || p->decal_list || p->button_list)
 				fprintf(stderr,
-					"GKrellM: clean_out_panels krell=%p decal=%p button=%p\n",
+					"GKrellM: gkrellm_panel_cleanup krell=%p decal=%p button=%p\n",
 						p->krell_list, p->decal_list, p->button_list);
 			}
 		else
@@ -492,6 +503,176 @@ clean_out_panels(void)
 	}
 
 
+  /* Check text_list decals for overlap of other decals and set flag to force
+  |  drawing in the push function instead of the normal text pixmap layer
+  */
+static void
+panel_decal_check_text_overlap(GkrellmDecal *d, GList *decal_list)
+	{
+	GList			*list;
+	GkrellmDecal	*dcheck;
+
+	if (!d || (d->state == DS_INVISIBLE))
+		return;
+	for (list = decal_list; list; list = list->next)
+		{
+		dcheck = (GkrellmDecal *) list->data;
+		if (dcheck->state == DS_INVISIBLE)
+			continue;
+
+		if (   d->x + d->w > dcheck->x
+			&& d->x < dcheck->x + dcheck->w
+			&& d->y + d->h > dcheck->y
+			&& d->y < dcheck->y + dcheck->h
+		   )
+			{
+			/* text_list decals need push if overlapping any other decal.
+			*/
+			if (d->text_list)
+				d->flags |= DF_TEXT_OVERLAPS;
+			if (dcheck->text_list)
+				dcheck->flags |= DF_TEXT_OVERLAPS;
+			}
+		}
+	}
+
+
+  /* Do the Pango text rendering.  If called when pixmap is bg_text_layer,
+  |  it is a background draw.  But if called from push_decal_pixmaps() it is
+  |  considered a push operation because it is being done whenever any panel
+  |  decals or krells are modified.  In this case, Pango does the pixel pushing
+  |  instead of gkrellm as was done in pre 2.2.0.
+  */
+static void
+panel_draw_decal_text_list(GdkPixmap *pixmap, GkrellmDecal *d)
+	{
+	PangoLayout			*layout;
+	GdkRectangle		rect;
+	GList				*list;
+	GkrellmText			*tx;
+	GkrellmTextstyle	*ts;
+	gchar				*s;
+	gint				x, y;
+
+	layout = gtk_widget_create_pango_layout(gkrellm_get_top_window(), NULL);
+	rect.x = d->x;
+	rect.y = d->y;
+	rect.width = d->w;
+	rect.height = d->h;
+	gdk_gc_set_clip_rectangle(_GK.text_GC, &rect);
+	for (list = d->text_list; list; list = list->next)
+		{
+		tx = (GkrellmText *) list->data;
+		if (!*tx->text)
+			continue;
+		ts = &tx->text_style;
+
+		pango_layout_set_font_description(layout, ts->font);
+		x = tx->x_off;
+		y = tx->y_off;
+		if (d->flags & DF_SCROLL_TEXT_DIVERTED)
+			{
+			if (d->flags & DF_SCROLL_TEXT_CENTER)
+				pango_layout_set_alignment(layout, PANGO_ALIGN_CENTER);
+			if (d->flags & DF_SCROLL_TEXT_H_LOOP)
+				{
+				x %= d->scroll_width;
+				if (x > 0)
+					x -= d->scroll_width;
+				s = g_strconcat(tx->text, tx->text, NULL);
+				if (d->flags & DF_TEXT_USE_MARKUP)
+					pango_layout_set_markup(layout, s, strlen(s));
+				else
+					pango_layout_set_text(layout, s, strlen(s));
+				g_free(s);
+				}
+			else if (d->flags & DF_SCROLL_TEXT_V_LOOP)
+				{
+				y %= d->scroll_height + d->y_ink;
+				if (y > 0)
+					y -= d->scroll_height + d->y_ink;
+				s = g_strconcat(tx->text, "\n", tx->text, NULL);
+				if (d->flags & DF_TEXT_USE_MARKUP)
+					pango_layout_set_markup(layout, s, strlen(s));
+				else
+					pango_layout_set_text(layout, s, strlen(s));
+				g_free(s);
+				}
+			else
+				{
+				if (d->flags & DF_TEXT_USE_MARKUP)
+					pango_layout_set_markup(layout, tx->text,strlen(tx->text));
+				else
+					pango_layout_set_text(layout, tx->text, strlen(tx->text));
+				}
+			}
+		else
+			{
+			if (d->flags & DF_TEXT_USE_MARKUP)
+				pango_layout_set_markup(layout, tx->text, strlen(tx->text));
+			else
+				pango_layout_set_text(layout, tx->text, strlen(tx->text));
+			}
+		x += d->x;
+		y += d->y;
+
+		if (ts->effect)
+			{
+			gdk_gc_set_foreground(_GK.text_GC, &ts->shadow_color);
+			gdk_draw_layout_with_colors(pixmap, _GK.text_GC,
+						x + 1, y + 1, layout,
+						&ts->shadow_color, NULL);
+			}
+		gdk_gc_set_foreground(_GK.text_GC, &ts->color);
+		gdk_draw_layout(pixmap, _GK.text_GC, x, y, layout);
+		}
+	gdk_gc_set_clip_rectangle(_GK.text_GC, NULL);
+	g_object_unref(layout);
+	}
+
+  /* Draw text decals on a cache text layer to avoid high overhead Pango
+  |  drawing of them each time gkrellm_draw_panel_layers() is called.
+  |  May draw here if a text decal was found to not overlap any other decal
+  |  and if it's not flagged to be drawn on the top layer.
+  |  The text layer is then used as the background for push decal and krell
+  |  draws.
+  */
+static void
+panel_draw_decal_text_layer(GkrellmPanel *p)
+	{
+	GList			*list;
+	GkrellmDecal	*d;
+
+	for (list = p->decal_list; list; list = list->next)
+		{
+		d = (GkrellmDecal *) list->data;
+
+		if (   !d->modified
+			|| !d->text_list
+			|| (d->flags & DF_TOP_LAYER)
+			|| (d->flags & DF_TEXT_OVERLAPS)
+		   )
+			continue;
+		if (d->state != DS_INVISIBLE)
+			{
+			if (d->flags & DF_MOVED)
+				{
+				gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
+					p->bg_pixmap,
+					d->x_old, d->y_old, d->x_old, d->y_old, d->w, d->h);
+				d->flags &= ~DF_MOVED;
+				}
+			else
+				gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
+					p->bg_pixmap,
+					d->x, d->y,  d->x, d->y,   d->w, d->h);
+
+			panel_draw_decal_text_list(p->bg_text_layer_pixmap, d);
+			}
+		p->modified = TRUE;
+		}
+	}
+
   /* Push decal pixmaps through their stencils onto a Panel expose pixmap
   */
 static gboolean
@@ -499,8 +680,9 @@ push_decal_pixmaps(GkrellmPanel *p, gboolean top_layer)
 	{
 	GList			*list;
 	GkrellmDecal	*d;
-	gboolean		on_top, do_top_layer = FALSE;
-	gboolean		restore_gc = FALSE;
+	gint			x, y, w, h;
+	gboolean		on_top, is_non_overlapping_text_decal;
+	gboolean		restore_gc = FALSE, do_top_layer = FALSE;
 
 	if (!p)
 		return FALSE;
@@ -508,23 +690,72 @@ push_decal_pixmaps(GkrellmPanel *p, gboolean top_layer)
 		{
 		d = (GkrellmDecal *) list->data;
 		on_top = (d->flags & DF_TOP_LAYER);
+		is_non_overlapping_text_decal
+				= (d->text_list && !(d->flags & DF_TEXT_OVERLAPS));
+
 		if (on_top && !top_layer)
 			do_top_layer = TRUE;
 
 		if (   d->state != DS_VISIBLE
-			|| (on_top && !top_layer) || (!on_top && top_layer)
+			|| (top_layer && !on_top)
+			|| (!top_layer && (on_top || is_non_overlapping_text_decal))
 		   )
 			continue;
-		gdk_gc_set_clip_mask(_GK.text_GC, d->stencil);
-		gdk_gc_set_clip_origin(_GK.text_GC, d->x, d->y);
-		gdk_draw_drawable(p->pixmap, _GK.text_GC, d->pixmap,
-					0, d->y_src, d->x, d->y, d->w, d->h);
-		restore_gc = TRUE;
+		if (d->text_list)
+			panel_draw_decal_text_list(p->pixmap, d);
+		else if (d->scroll_text)
+			{
+			x = d->x_off;
+			y = d->y_off;
+
+			gdk_draw_drawable(p->pixmap, _GK.draw1_GC, d->pixmap,
+						-x, -y, d->x, d->y, d->w, d->h);
+			if (d->flags & DF_SCROLL_TEXT_H_LOOP)
+				{
+				x %= d->scroll_width;
+				if (x > 0)
+					{
+					gdk_draw_drawable(p->pixmap, _GK.draw1_GC, d->pixmap,
+							d->scroll_width - x, -y,
+							d->x, d->y, x, d->h);
+					}
+				else if (   x <= 0
+						 && (w = d->scroll_width + x) < d->w
+						)
+					{
+					gdk_draw_drawable(p->pixmap, _GK.draw1_GC, d->pixmap,
+							0, -y,
+							d->x + w, d->y, d->w - w, d->h);
+					}
+				}
+			if (d->flags & DF_SCROLL_TEXT_V_LOOP)
+				{
+				y %= d->scroll_height + d->y_ink;
+				if (y > 0)
+					gdk_draw_drawable(p->pixmap, _GK.draw1_GC, d->pixmap,
+							-x, d->scroll_height + d->y_ink - y,
+							d->x, d->y, d->w, y);
+				else if (   y <= 0
+						 && (h = d->scroll_height + d->y_ink + y) < d->h
+						)
+					gdk_draw_drawable(p->pixmap, _GK.draw1_GC, d->pixmap,
+							-x, 0,
+							d->x, d->y + h, d->w, d->h - h);
+				}
+			}
+		else
+			{
+			gdk_gc_set_clip_mask(_GK.draw3_GC, d->stencil);
+			gdk_gc_set_clip_origin(_GK.draw3_GC, d->x, d->y);
+			gdk_draw_drawable(p->pixmap, _GK.draw3_GC, d->pixmap,
+						0, d->y_src, d->x, d->y, d->w, d->h);
+			restore_gc = TRUE;
+			}
 		}
 	if (restore_gc)
 		{
-		gdk_gc_set_clip_mask(_GK.text_GC, NULL);
-		gdk_gc_set_clip_origin(_GK.text_GC, 0, 0);
+		gdk_gc_set_clip_mask(_GK.draw3_GC, NULL);
+		gdk_gc_set_clip_origin(_GK.draw3_GC, 0, 0);
 		}
 
 	return do_top_layer;
@@ -566,55 +797,62 @@ push_krell_pixmaps(GkrellmPanel *p)
 void
 gkrellm_draw_panel_layers(GkrellmPanel *p)
 	{
-	GdkDrawable		*drawable;
 	GList			*list;
 	GkrellmKrell	*k;
 	GkrellmDecal	*d;
-	GkrellmDrawrec	*dr;
 	gboolean		do_top_layer_decals;
 
 	if (!p || !p->drawing_area)
 		return;
-	/* Restore background of panel where objects have changed.
-	|  I should test for size of krell, and if large enough, just do this:
-	|		gdk_draw_drawable(p->pixmap, _GK.draw1_GC, p->bg_pixmap,
-	|					0, 0,   0, 0,   p->w, p->h);
-	|
-	*/
-	/* Restore background under old decal positions */
+
+	if (p->need_decal_overlap_check)
+		{
+		gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
+					p->bg_pixmap,
+					0, 0,  0, 0,  p->w, p->h);
+		for (list = p->decal_list; list; list = list->next)
+			{
+			d = (GkrellmDecal *) list->data;
+			if (!(d->flags & DF_SCROLL_TEXT_DIVERTED))
+				d->flags &= ~DF_TEXT_OVERLAPS;
+			d->modified = TRUE;
+			}
+		for (list = p->decal_list; list; list = list->next)
+			{
+			d = (GkrellmDecal *) list->data;
+			panel_decal_check_text_overlap(d, list->next);
+			}
+		}
+	p->need_decal_overlap_check = FALSE;
+
+	panel_draw_decal_text_layer(p);
+
 	for (list = p->decal_list; list; list = list->next)
 		{
 		d = (GkrellmDecal *) list->data;
 		if (d->modified)
 			{
-			if (d->state != DS_INVISIBLE)
-				{
-				if (d->flags & DF_MOVED)
-					gdk_draw_drawable(p->pixmap, _GK.draw1_GC, p->bg_pixmap,
-						d->x_old, d->y_old, d->x_old, d->y_old, d->w, d->h);
-				else
-					gdk_draw_drawable(p->pixmap, _GK.draw1_GC, p->bg_pixmap,
-						d->x, d->y,  d->x, d->y,   d->w, d->h);
-				}
+			d->modified = FALSE;
 			p->modified = TRUE;
 			}
 		}
-	/* Restore background under old krell positions */
 	for (list = p->krell_list; list; list = list->next)
 		{
 		k = (GkrellmKrell *) list->data;
 		if (k->modified)
 			{
-			dr = &k->old_draw;
-			gdk_draw_drawable(p->pixmap, _GK.draw1_GC, p->bg_pixmap,
-					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
+			k->modified = FALSE;
 			p->modified = TRUE;
 			}
 		}
+
 	/* For each layer, push new layer image onto the expose pixmap.
 	*/
 	if (p->modified)
 		{
+		gdk_draw_drawable(p->pixmap, _GK.draw1_GC, p->bg_text_layer_pixmap,
+					0, 0,  0, 0,  p->w, p->h);
+
 		do_top_layer_decals = push_decal_pixmaps(p, FALSE);
 		if (p->label_on_top_of_decals)
 			draw_panel_label(p, FALSE);
@@ -622,52 +860,14 @@ gkrellm_draw_panel_layers(GkrellmPanel *p)
 		if (do_top_layer_decals)
 			push_decal_pixmaps(p, TRUE);
 
-		/* Draw regions from the expose pixmap onto the screen.  Draw the
-		|  Old krell region and the new krell region.
-		|  I should test here for region sizes to minimize screen draws.
-		*/
-#if 0
-		if (regions_to_draw_are_large_enough)	/* Blast expose pixmap */
+		if (p->drawing_area->window)
 			gdk_draw_drawable(p->drawing_area->window, _GK.draw1_GC, p->pixmap,
-					0, 0,   0, 0,   p->w, p->h);
-		else
-#endif
-
-		drawable = p->drawing_area->window;
-		for (list = p->decal_list; list; list = list->next)
-			{
-			d = (GkrellmDecal *) list->data;
-			if (d->state != DS_INVISIBLE && drawable)
-				{
-				if (d->flags & DF_MOVED)
-					{
-					gdk_draw_drawable(drawable, _GK.draw1_GC, p->pixmap,
-						d->x_old, d->y_old, d->x_old, d->y_old, d->w, d->h);
-					d->flags &= ~DF_MOVED;
-					}
-				gdk_draw_drawable(drawable, _GK.draw1_GC, p->pixmap,
-						d->x, d->y,  d->x, d->y,   d->w, d->h);
-				}
-			d->modified = FALSE;
-			}
-		for (list = p->krell_list; list; list = list->next)
-			{
-			k = (GkrellmKrell *) list->data;
-			if (drawable)
-				{
-				dr = &k->old_draw;
-				gdk_draw_drawable(drawable, _GK.draw1_GC, p->pixmap,
-					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
-				dr = &k->draw;
-				gdk_draw_drawable(drawable, _GK.draw1_GC, p->pixmap,
-					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
-				}
-			k->modified = FALSE;
-			}
+						0, 0,   0, 0,   p->w, p->h);
 		}
 	p->modified = FALSE;
 	}
 
+
 void
 gkrellm_draw_panel_layers_force(GkrellmPanel *p)
 	{
