@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -21,6 +21,7 @@
 
 #include "gkrellm.h"
 #include "gkrellm-private.h"
+#include "gkrellm-sysdeps.h"
 
 #include <gmodule.h>
 
@@ -32,6 +33,18 @@
 /* Plugin interface to GKrellM.
 */
 
+gchar *
+gkrellm_get_hostname(void)
+	{
+	gchar	*hostname;
+
+	if (_GK.client_mode)
+		hostname = _GK.server_hostname;
+	else
+		hostname = gkrellm_sys_get_host_name();
+	return hostname;
+	}
+
 gchar *
 gkrellm_get_theme_path(void)
 	{
@@ -189,14 +202,14 @@ get_style_from_list_by_name(GList *name_list, GList *style_list, gchar *name)
 	strncpy(buf, name, sizeof(buf));
 	buf[sizeof(buf) - 1] = '\0';
 	if (   (p = strchr(buf, '.')) != NULL
-		&& (n = string_position_in_list(_GK.custom_name_list, name)) >= 0
+		&& (n = gkrellm_string_position_in_list(_GK.custom_name_list, name)) >= 0
 	   )
 		list = _GK.custom_style_list;
 	else
 		{
 		if (p)
 			*p = '\0';
-		if ((n = string_position_in_list(name_list, buf)) < 0)
+		if ((n = gkrellm_string_position_in_list(name_list, buf)) < 0)
 			n = 0;
 		list = style_list;
 		}
@@ -483,7 +496,7 @@ gkrellm_meter_alt_textstyle(gint n)
 	return ts;
 	}
 
-GdkFont *
+PangoFontDescription *
 gkrellm_default_font(gint n)
 	{
 	if (n == 0)
@@ -584,7 +597,7 @@ gchar				*plugin_install_log;
 
 
 GkrellmMonitor *
-lookup_monitor_from_id(gint id)
+gkrellm_monitor_from_id(gint id)
 	{
 	GList			*list;
 	GkrellmMonitor	*mon;
@@ -616,7 +629,7 @@ lookup_monitor_from_name(gchar *name)
 	}
 
 GkrellmMonitor *
-lookup_monitor_from_style_name(gchar *style_name)
+gkrellm_monitor_from_style_name(gchar *style_name)
 	{
 	GkrellmMonitor	*mon;
 	GList			*list;
@@ -668,8 +681,8 @@ user_placement(GkrellmMonitor *plugin)
 	for (list = plugins_place_list; list; list = list->next)
 		{
 		line = g_strconcat((gchar *) list->data, NULL);
-		plugin_name = cut_quoted_string(line, &s);
-		mon_name = cut_quoted_string(s, &s);
+		plugin_name = gkrellm_cut_quoted_string(line, &s);
+		mon_name = gkrellm_cut_quoted_string(s, &s);
 		n = sscanf(s, "%d %d", &after, &gravity);
 		if (n == 2)
 			{
@@ -896,8 +909,8 @@ disable_plugin(GkrellmMonitor *plugin)
 	mp->enabled = FALSE;
 	if (mp->cb_disable_plugin)
 		(*mp->cb_disable_plugin)();
-	build_gkrellm();
-	remove_plugin_config_page(plugin);
+	gkrellm_build();
+	gkrellm_remove_plugin_config_page(plugin);
 	plugin_enable_list_modified = TRUE;
 	}
 
@@ -911,8 +924,8 @@ enable_plugin(GkrellmMonitor *plugin)
 	gtk_widget_show(mp->vbox);
 	gtk_widget_show(mp->main_vbox);
 	mp->enabled = TRUE;
-	build_gkrellm();
-	add_plugin_config_page(plugin);
+	gkrellm_build();
+	gkrellm_add_plugin_config_page(plugin);
 	plugin_enable_list_modified = TRUE;
 	}
 
@@ -922,7 +935,7 @@ load_plugins_placement_file(void)
 	FILE	*f;
 	gchar	*path, *s, buf[256];
 
-	free_glist_and_data(&plugins_place_list);
+	gkrellm_free_glist_and_data(&plugins_place_list);
 	path = gkrellm_make_config_file_name(gkrellm_homedir(),
 				PLUGIN_PLACEMENT_FILE);
 	if ((f = fopen(path, "r")) != NULL)
@@ -965,7 +978,7 @@ save_plugins_placement_file(void)
 				   )
 			   )
 				{
-				builtin = lookup_monitor_from_id(mp->insert_before_id);
+				builtin = gkrellm_monitor_from_id(mp->insert_before_id);
 				if (!builtin)
 					continue;
 				fprintf(f, "\"%s\" \"%s\" %d %d\n", plugin->name,
@@ -984,7 +997,7 @@ load_plugins_enable_file(void)
 	FILE	*f;
 	gchar	*path, *s, buf[256];
 
-	free_glist_and_data(&plugins_enable_list);
+	gkrellm_free_glist_and_data(&plugins_enable_list);
 	path = gkrellm_make_config_file_name(gkrellm_homedir(),PLUGIN_ENABLE_FILE);
 	if ((f = fopen(path, "r")) != NULL)
 		{
@@ -1057,6 +1070,12 @@ scan_for_plugins(gchar *path)
 		return;
 	while ((name = (gchar *) g_dir_read_name(dir)) != NULL)
 		{
+		if (   !string_suffix(name, "so")
+			&& !string_suffix(name, "la")
+			&& !string_suffix(name, "dll")
+		   )
+			continue;
+
 		/* If there's a libtool .la archive, won't want to load this .so
 		*/
 		if (   !string_suffix(name, "la")
@@ -1093,7 +1112,7 @@ scan_for_plugins(gchar *path)
 			{
 			plugins_list = g_list_append(plugins_list, m);
 			s = g_path_get_basename(m->path);
-			if (! string_in_list(plugins_enable_list, s))
+			if (! gkrellm_string_in_list(plugins_enable_list, s))
 				m->privat->enabled = FALSE;
 			}
 		g_free(s);
@@ -1102,7 +1121,7 @@ scan_for_plugins(gchar *path)
 	}
 
 void
-load_plugin_monitors(void)
+gkrellm_plugins_load(void)
 	{
 	GkrellmMonitor	*m;
 	gchar			*path;
@@ -1142,6 +1161,35 @@ load_plugin_monitors(void)
 #endif
 	}
 
+#if 0
+void
+shuffle_test(void)
+	{
+	GList			*list;
+	GkrellmMonitor	*mon;
+	gint			i;
+	static gboolean	srand_done;
+	extern GtkWidget *gkrellm_monitor_vbox();
+
+	if (!srand_done)
+		{
+		srandom((unsigned int) time(0));
+		srand_done = TRUE;
+		}
+	for (list = gkrellm_monitor_list; list; list = list->next)
+		{
+		i = (gint) random();
+		i = (i % (g_list_length(gkrellm_monitor_list) - 2)) + 1;
+		mon = (GkrellmMonitor *) list->data;
+		if (mon->privat->main_vbox && mon != gkrellm_mon_host())
+			{
+			gtk_box_reorder_child(GTK_BOX(gkrellm_monitor_vbox()),
+						mon->privat->main_vbox, i);
+			}
+		}
+	}
+#endif
+
 static void
 replace_plugins()
 	{
@@ -1250,7 +1298,7 @@ cb_place_default(GtkWidget *widget, gpointer data)
 				TRUE);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gravity_spin_button),
 			(gfloat) PLUGIN_GRAVITY(place_plugin));
-	mon = lookup_monitor_from_id(PLUGIN_INSERT_BEFORE_ID(place_plugin));
+	mon = gkrellm_monitor_from_id(PLUGIN_INSERT_BEFORE_ID(place_plugin));
 	if (mon && mon->privat->button_id >= 0)
 		gtk_toggle_button_set_active(
 			GTK_TOGGLE_BUTTON(builtin_button[mon->privat->button_id]), TRUE);
@@ -1291,7 +1339,7 @@ place_button_sensitivity(GkrellmMonitor *plugin, gboolean state)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(before_button), TRUE);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gravity_spin_button),
 			(gfloat) plugin->privat->gravity);
-	mon = lookup_monitor_from_id(plugin->privat->insert_before_id);
+	mon = gkrellm_monitor_from_id(plugin->privat->insert_before_id);
 	if (mon && mon->privat->button_id >= 0)
 		gtk_toggle_button_set_active(
 			GTK_TOGGLE_BUTTON(builtin_button[mon->privat->button_id]), TRUE);
@@ -1488,7 +1536,7 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	}
 
 void
-close_plugins_config(void)
+gkrellm_plugins_config_close(void)
 	{
 	save_plugins_enable_file();
 	save_plugins_placement_file();
@@ -1527,7 +1575,7 @@ create_model(void)
 	}
 
 void
-create_plugin_config(GtkWidget *tab_vbox)
+gkrellm_plugins_config_create(GtkWidget *tab_vbox)
 	{
 	GtkWidget			*tabs;
 	GtkWidget			*vbox;
