@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -49,6 +49,7 @@ typedef struct _sensor
 	gint		inter;
 
 	gint		enabled;
+	gint		group;
 	gint		location;		/* default, Proc panel, or cpu panel */
 	gfloat		factor,			/* Scale sensor reading		*/
 				offset;			/* Add to sensor reading	*/
@@ -112,8 +113,16 @@ setup_sensor_interface(void)
 	return get_temperature ? TRUE : FALSE;
 	}
 
-
 void
+gkrellm_sensors_set_group(gpointer sr, gint group)
+	{
+	Sensor	*sensor = (Sensor *) sr;
+
+	if (sensor)
+		sensor->group = group;
+	}
+
+gpointer
 gkrellm_sensors_add_sensor(gint type, gchar *sensor_path, gchar *id_name,
 		gint id, gint iodev, gint inter,
 		gfloat factor, gfloat offset, gchar *vref, gchar *default_label)
@@ -122,7 +131,7 @@ gkrellm_sensors_add_sensor(gint type, gchar *sensor_path, gchar *id_name,
 	gchar	*r;
 
 	if (!id_name || !*id_name || type < 0 || type > 2)
-		return;
+		return NULL;
 
 	sensor = g_new0(Sensor, 1);
 	sensor->id_name = g_strdup(id_name);
@@ -149,9 +158,11 @@ gkrellm_sensors_add_sensor(gint type, gchar *sensor_path, gchar *id_name,
 	if (type == SENSOR_VOLTAGE && vref)
 		sensor->vref_name = g_strdup(vref);
 	sensor_list = g_list_append(sensor_list, sensor);
+	return (gpointer) sensor;
 	}
 
 
+
 /* ======================================================================== */
 static gboolean	use_threads,
 				thread_data_valid,
@@ -339,7 +350,7 @@ gkrellm_sensor_draw_fan_decal(GkrellmPanel *p, GkrellmDecal *d, gfloat f)
 	d->x_off = d->w - w;
 	if (d->x_off < 0)
 		d->x_off = 0;
-	gkrellm_draw_decal_text(p, d, buf, atoi(buf) /* draw when changes */);
+	gkrellm_draw_decal_text(p, d, buf, 0);
 	}
 
 void
@@ -347,7 +358,7 @@ gkrellm_sensor_draw_temperature_decal(GkrellmPanel *p, GkrellmDecal *d,
 				gfloat t, gchar units)
 	{
 	gchar	*s, buf[8];
-	gint	w, value = 0;
+	gint	w;
 
 	if (!p || !d)
 		return;
@@ -361,15 +372,12 @@ gkrellm_sensor_draw_temperature_decal(GkrellmPanel *p, GkrellmDecal *d,
 		snprintf(buf, sizeof(buf), "%.0f%c", t, units);
 		w = gkrellm_gdk_string_width(d->text_style.font, buf)
 				+ d->text_style.effect;
-		value = atoi(buf);
 		}
-	else
-		value = 10 * atoi(buf) + (s ? atoi(s+1) : 0);
 	
 	d->x_off = d->w - w;
 	if (d->x_off < 0)
 		d->x_off = 0;
-	gkrellm_draw_decal_text(p, d, buf, value /* draw when changes */);
+	gkrellm_draw_decal_text(p, d, buf, 0 /* no longer used */);
 	}
 
 static Sensor *
@@ -480,6 +488,7 @@ gkrellm_sensor_alert_connect(gpointer sr, void (*cb_func)(), gpointer data)
 	gkrellm_alert_trigger_connect(sensor->alert, cb_func, data);
 	gkrellm_alert_command_process_connect(sensor->alert,
 				cb_command_process, sensor);
+	gkrellm_reset_alert_soft(sensor->alert);
 	}
 
 static gboolean
@@ -602,12 +611,17 @@ gkrellm_sensor_read_voltage(gpointer sr, gfloat *voltage)
 
 #define	SENSOR_STYLE_NAME	"sensors"
 
-/* Temperature and fan sensors can be located on different panels
+/* Temperature and fan sensors can be located on different panels depending
+|  on the sensor group.
 */
 #define	SENSOR_PANEL_LOCATION	0
-#define	PROC_PANEL_LOCATION		1
+
+#define	PROC_PANEL_LOCATION		1	/* SENSOR_GROUP_MAINBOARD */
 #define	CPU_PANEL_LOCATION		2	/* cpu0 if smp */
 
+#define	DISK_PANEL_LOCATION		1	/* SENSOR_GROUP_DISK	*/
+
+
 #define DO_TEMP	1
 #define	DO_FAN	1
 #define	DO_VOLT	1
@@ -628,7 +642,8 @@ static GtkWidget
 
 static GList	*volt_list,
 				*temperature_list,
-				*fan_list;
+				*fan_list,
+				*disk_temperature_list;
 
 static GkrellmPanel
 				*pVolt,
@@ -641,6 +656,7 @@ static gint		volt_mon_width,
 				volt_name_width,
 				volt_bezel_width;
 
+
   /* Display modes */
 #define	DIGITAL_WITH_LABELS	0
 #define	DIGITAL_NO_LABELS	1
@@ -677,7 +693,9 @@ static void
 add_sensor_monitor(Sensor *sr, GkrellmPanel **p, GList **smon_list)
 	{
 	SensorMon		*smon;
-	gboolean		set_loc;
+	gfloat			t;
+	gchar			units;
+	gboolean		set_loc = FALSE;
 
 	sr->smon = NULL;
 	if (!sr->enabled)
@@ -685,11 +703,28 @@ add_sensor_monitor(Sensor *sr, GkrellmPanel **p, GList **smon_list)
 
 	if (sr->location != SENSOR_PANEL_LOCATION)
 		{
-		if (sr->location == PROC_PANEL_LOCATION)
-			set_loc = gkrellm_proc_set_sensor(sr, sr->type);
-		else
-			set_loc = gkrellm_cpu_set_sensor(sr, sr->type,
-						sr->location - CPU_PANEL_LOCATION);
+		if (sr->group == SENSOR_GROUP_MAINBOARD)
+			{
+			if (sr->location == PROC_PANEL_LOCATION)
+				set_loc = gkrellm_proc_set_sensor(sr, sr->type);
+			else
+				set_loc = gkrellm_cpu_set_sensor(sr, sr->type,
+							sr->location - CPU_PANEL_LOCATION);
+			}
+		else if (sr->group == SENSOR_GROUP_DISK)
+			{
+			if (sr->location == DISK_PANEL_LOCATION)
+				{
+				gkrellm_freeze_alert(sr->alert);
+				sensor_read_temperature(sr, &t, &units);
+				gkrellm_thaw_alert(sr->alert);
+				set_loc = gkrellm_disk_temperature_display((gpointer) sr,
+							sr->id_name, t, units);
+				if (set_loc)
+					disk_temperature_list =
+							g_list_append(disk_temperature_list, sr);
+				}
+			}
 		if (set_loc)
 			return;
 		sr->location = SENSOR_PANEL_LOCATION;
@@ -708,11 +743,15 @@ make_sensor_monitor_lists(gboolean do_temp, gboolean do_fan, gboolean do_volt)
 	Sensor	*sr;
 
 	if (do_temp)
-		free_glist_and_data(&temperature_list);
+		{
+		gkrellm_free_glist_and_data(&temperature_list);
+		g_list_free(disk_temperature_list);
+		disk_temperature_list = NULL;
+		}
 	if (do_fan)
-		free_glist_and_data(&fan_list);
+		gkrellm_free_glist_and_data(&fan_list);
 	if (do_volt)
-		free_glist_and_data(&volt_list);
+		gkrellm_free_glist_and_data(&volt_list);
 
 	for (list = sensor_list; list; list = list->next)
 		{
@@ -780,8 +819,11 @@ draw_bezels(GkrellmPanel *p, GList *smon_list, gint w, gint h, gint x_adjust)
 		gkrellm_paste_piximage(bezel_piximage, p->pixmap,
 				x - b->left, dv->y - b->top, w, h);
 		}
+	gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC, p->bg_pixmap,
+				0, 0,  0, 0,  p->w, p->h);
 	}
 
+
 static gboolean
 any_negative_volts(void)
 	{
@@ -869,6 +911,7 @@ make_volt_decals(GkrellmPanel *p, GkrellmStyle *style)
 		gkrellm_alert_trigger_connect(sensor->alert, cb_alert_trigger, volt);
 		gkrellm_alert_command_process_connect(sensor->alert,
 					cb_command_process, sensor);
+		gkrellm_reset_alert_soft(sensor->alert);
 		}
 	pixel_grub = minus_width ? 1 : 0;
 	volt_bezel_width = b->left + w_volt + b->right - pixel_grub;
@@ -933,6 +976,23 @@ layout_volt_decals(GkrellmPanel *p, GkrellmStyle *style)
 		}
 	}
 
+static void
+update_disk_temperatures(void)
+	{
+	GList	*list;
+	Sensor	*sr;
+	gfloat	t;
+	gchar	units;
+
+	for (list = disk_temperature_list; list; list = list->next)
+		{
+		sr = (Sensor *) list->data;
+		sensor_read_temperature(sr, &t, &units);
+		gkrellm_disk_temperature_display((gpointer) sr, sr->id_name, t, units);
+		}
+	}
+
+
 static void
 draw_temperatures(gboolean draw_name)
 	{
@@ -1034,7 +1094,9 @@ draw_voltages(Sensor *s, gint do_names)
 static void
 update_sensors(void)
 	{
-	if (!GK.five_second_tick)
+	static gboolean		first_time_done;
+
+	if (!GK.five_second_tick && first_time_done)
 		return;
 	if (use_threads)
 		{
@@ -1044,6 +1106,8 @@ update_sensors(void)
 	draw_temperatures(FALSE);
 	draw_fans(FALSE);
 	draw_voltages(NULL, FALSE);
+	update_disk_temperatures();
+	first_time_done = TRUE;
 	}
 
 static gint
@@ -1156,6 +1220,7 @@ make_temperature_panel(GtkWidget *vbox, gint first_create)
 		gkrellm_alert_trigger_connect(sensor->alert, cb_alert_trigger, smon);
 		gkrellm_alert_command_process_connect(sensor->alert,
 					cb_command_process, sensor);
+		gkrellm_reset_alert_soft(sensor->alert);
 		}
 	g_free(ts_name);
 	g_free(ts_sensor);
@@ -1214,6 +1279,7 @@ make_fan_panel(GtkWidget *vbox, gint first_create)
 		gkrellm_alert_trigger_connect(sensor->alert, cb_alert_trigger, smon);
 		gkrellm_alert_command_process_connect(sensor->alert,
 					cb_command_process, sensor);
+		gkrellm_reset_alert_soft(sensor->alert);
 		}
 	g_free(ts_name);
 	g_free(ts_sensor);
@@ -1398,7 +1464,7 @@ static GkrellmMonitor	monitor_sensors =
 	};
 
 GkrellmMonitor *
-init_sensor_monitor(void)
+gkrellm_init_sensor_monitor(void)
 	{
 	if (!sensor_list)
 		return NULL;
@@ -1629,8 +1695,9 @@ static GtkTreeRowReference	*row_reference;
 static GtkTreeSelection		*selection;
 
 
-static GtkWidget	*optionmenu;
-//static GtkWidget	*use_threads_button;
+static GtkWidget	*optionmenu,
+					*optionmenu_menu;
+
 static GtkWidget	*display_mode_button[2];
 static GtkWidget	*factor_spin_button,
 					*offset_spin_button;
@@ -1904,6 +1971,135 @@ cb_drag_end(GtkWidget *widget, GdkDragContext *context, gpointer data)
 	return FALSE;
 	}
 
+static void
+cb_location_menu(GtkOptionMenu *om, gpointer data)
+	{
+	GList		*list;
+	Sensor		*sr, *s;
+	gchar		*msg, *pname = NULL;
+	gint		location;
+
+	location = gtk_option_menu_get_history(om);
+	sr = get_referenced_sensor();
+	if (!sr || !sr->enabled || sr->location == location)
+		return;
+
+	/* If trying to relocate, get a dst panel name so can report failures.
+	*/
+	if (location != SENSOR_PANEL_LOCATION)
+		{
+		if (sr->group == SENSOR_GROUP_MAINBOARD)
+			pname = (location == PROC_PANEL_LOCATION)
+				? gkrellm_proc_get_sensor_panel_label()
+				: gkrellm_cpu_get_sensor_panel_label(
+							location - CPU_PANEL_LOCATION);
+		else if (sr->group == SENSOR_GROUP_DISK)
+			pname = _("Disk");
+		}
+
+	/* If moving off some other panel, reset that panel.
+	*/
+	if (sr->group == SENSOR_GROUP_MAINBOARD)
+		{
+		if (sr->location == PROC_PANEL_LOCATION)
+			gkrellm_proc_set_sensor(NULL, sr->type);
+		else if (sr->location >= CPU_PANEL_LOCATION)
+			gkrellm_cpu_set_sensor(NULL, sr->type,
+						sr->location - CPU_PANEL_LOCATION);
+		}
+	else if (sr->group == SENSOR_GROUP_DISK)
+		{
+		if (sr->location == DISK_PANEL_LOCATION)
+			gkrellm_disk_temperature_remove(sr->id_name);
+		}
+
+	/* For mainboard sensor group, if relocating to a panel with some other
+	|  sensor of same type on it, auto restore the other sensor to the sensor
+	|  panel.  Disk sensor group should never conflict.
+	*/
+	sr->location = location;
+	if (   location != SENSOR_PANEL_LOCATION
+		&& sr->group == SENSOR_GROUP_MAINBOARD
+	   )
+		for (list = sensor_list; list; list = list->next)
+			{
+			s = (Sensor *) list->data;
+			if (   s->group == SENSOR_GROUP_MAINBOARD
+				&& s != sr
+				&& s->type == sr->type
+				&& s->location == sr->location
+			   )
+				s->location = SENSOR_PANEL_LOCATION;	/* is being replaced */
+			}
+	rebuild_sensors_monitor(DO_TEMP, DO_FAN, FALSE);
+
+	if (sr->location != location)	/* location failed */
+		{
+		gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),
+					SENSOR_PANEL_LOCATION);
+		msg = g_strdup_printf(
+					_("Can't find a %s panel to relocate sensor to."),
+					pname ? pname : "?");
+		gkrellm_config_message_dialog(NULL, msg);
+		g_free(msg);
+		}
+	}
+
+
+static void
+create_location_menu(gint group)
+	{
+	GtkWidget	*menu;
+	GtkWidget	*menuitem;
+	gint		n, n_cpus;
+	static gint	sig_id, last_group = -1;
+
+	if (group == last_group)
+		return;
+	last_group = group;
+
+	if (optionmenu_menu)
+		{
+		g_signal_handler_disconnect(G_OBJECT(optionmenu), sig_id);
+		gtk_widget_destroy(optionmenu_menu);
+		}
+
+	menu = gtk_menu_new();
+
+	menuitem = gtk_menu_item_new_with_label("default");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	gtk_widget_show(menuitem);
+
+	if (group == SENSOR_GROUP_MAINBOARD)
+		{
+		menuitem = gtk_menu_item_new_with_label(
+					gkrellm_proc_get_sensor_panel_label());
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+		gtk_widget_show(menuitem);
+
+		n_cpus = gkrellm_smp_cpus();
+		if (n_cpus == 0)
+			n_cpus = 1;
+		for (n = 0; n < n_cpus; ++n)
+			{
+			menuitem = gtk_menu_item_new_with_label(
+						gkrellm_cpu_get_sensor_panel_label(n));
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+			gtk_widget_show(menuitem);
+			}
+		}
+	else if (group == SENSOR_GROUP_DISK)
+		{
+		menuitem = gtk_menu_item_new_with_label(_("Disk"));
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+		gtk_widget_show(menuitem);
+		}
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu), menu);
+	optionmenu_menu = menu;
+	sig_id = g_signal_connect(G_OBJECT(optionmenu), "changed",
+				G_CALLBACK(cb_location_menu), NULL);
+	}
+
 static void
 set_sensor_widget_states(Sensor *s)
 	{
@@ -1929,6 +2125,7 @@ set_sensor_widget_states(Sensor *s)
 			p_sensitive = TRUE;
 			}
 		}
+	create_location_menu(s ? s->group : 0);
 	gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu), location);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(factor_spin_button), factor);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(offset_spin_button), offset);
@@ -2049,7 +2246,6 @@ cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
 	model = GTK_TREE_MODEL(data);
 	path = gtk_tree_path_new_from_string(path_string);
 	gtk_tree_model_get_iter(model, &iter, path);
-	gtk_tree_path_free(path);
 
 	gtk_tree_model_get(model, &iter,
 				ENABLE_COLUMN, &enabled,
@@ -2060,6 +2256,7 @@ cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
 				ENABLE_COLUMN, s->enabled, 
 				-1);
     change_row_reference(model, path);
+	gtk_tree_path_free(path);
 
 	rebuild_sensors_monitor(s->type == SENSOR_TEMPERATURE,
 				s->type == SENSOR_FAN, s->type == SENSOR_VOLTAGE);
@@ -2134,69 +2331,6 @@ cb_set_alert(GtkWidget *widget, gpointer data)
 	gkrellm_alert_config_window(&s->alert);
 	}
 
-static void
-cb_location_menu(GtkOptionMenu *om, gpointer data)
-	{
-	GList		*list;
-	Sensor		*sr, *s;
-	gint		location;
-
-	location = gtk_option_menu_get_history(om);
-	sr = get_referenced_sensor();
-	if (!sr || !sr->enabled || sr->location == location)
-		return;
-	if (sr->location == PROC_PANEL_LOCATION)
-		gkrellm_proc_set_sensor(NULL, sr->type);
-	else if (sr->location >= CPU_PANEL_LOCATION)
-		gkrellm_cpu_set_sensor(NULL, sr->type,
-					sr->location - CPU_PANEL_LOCATION);
-
-	sr->location = location;
-
-	if (location != SENSOR_PANEL_LOCATION)
-		for (list = sensor_list; list; list = list->next)
-			{
-			s = (Sensor *) list->data;
-			if (s->type == sr->type && s != sr && s->location == sr->location)
-				s->location = SENSOR_PANEL_LOCATION;	/* is being replaced */
-			}
-	rebuild_sensors_monitor(DO_TEMP, DO_FAN, FALSE);
-
-	if (sr->location != location)	/* location failed */
-		gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),
-					SENSOR_PANEL_LOCATION);
-	}
-
-static GtkWidget *
-create_location_menu(void)
-	{
-	GtkWidget	*menu;
-	GtkWidget	*menuitem;
-	gint		n, n_cpus;
-
-	menu = gtk_menu_new();
-
-	menuitem = gtk_menu_item_new_with_label("default");
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-	gtk_widget_show(menuitem);
-
-	menuitem = gtk_menu_item_new_with_label(
-				gkrellm_proc_get_sensor_panel_label());
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-	gtk_widget_show(menuitem);
-
-	n_cpus = gkrellm_smp_cpus();
-	if (n_cpus == 0)
-		n_cpus = 1;
-	for (n = 0; n < n_cpus; ++n)
-		{
-		menuitem = gtk_menu_item_new_with_label(
-					gkrellm_cpu_get_sensor_panel_label(n));
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-		gtk_widget_show(menuitem);
-		}
-	return menu;
-	}
 
 static void
 cb_config_deleted(gpointer data)
@@ -2235,7 +2369,6 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	GtkWidget				*text;
 	GtkWidget				*vbox, *vbox1, *hbox, *box;
 	GtkWidget				*scrolled;
-	GtkWidget				*menu;
 	GtkWidget				*image;
 	GtkWidget				*label;
 	GtkTreeModel			*model;
@@ -2323,12 +2456,11 @@ create_sensors_tab(GtkWidget *tab_vbox)
 				cb_correction_modified, NULL, FALSE, NULL);
 
 	box = gkrellm_gtk_framed_vbox(vbox1, _("Location"), 2, FALSE, 0, 2);
-	menu = create_location_menu();
+
 	optionmenu = gtk_option_menu_new();
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu), menu);
+	optionmenu_menu = NULL;
+
 	gtk_box_pack_start(GTK_BOX(box), optionmenu, FALSE, FALSE, 4);
-	g_signal_connect(G_OBJECT(optionmenu), "changed",
-				G_CALLBACK(cb_location_menu), NULL);
 
 	box = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(box), 2);
@@ -2422,7 +2554,7 @@ static GkrellmMonitor	monitor_config_sensors =
 	};
 
 GkrellmMonitor *
-init_sensors_config_monitor(void)
+gkrellm_init_sensors_config_monitor(void)
 	{
 	if (!setup_sensor_interface() && !_GK.demo)
 		return NULL;
