@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -285,8 +285,8 @@ gkrellm_sys_fs_get_fsusage(gpointer fs, gchar *dir)
 
 	if (!statfs(dir, &st))
 		gkrellm_fs_assign_fsusage_data(fs,
-					(gulong) st.f_blocks, (gulong) st.f_bavail,
-					(gulong) st.f_bfree, (gulong) st.f_bsize);
+					(glong) st.f_blocks, (glong) st.f_bavail,
+					(glong) st.f_bfree, (glong) st.f_bsize);
 	else
 		gkrellm_fs_assign_fsusage_data(fs, 0, 0, 0, 0);
 	}
