@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -357,8 +357,8 @@ gkrellm_sys_fs_get_fsusage(gpointer fs, gchar *dir)
 	glibtop_fsusage	fsusage;
 
 	glibtop_get_fsusage(&fsusage, dir);
-	gkrellm_fs_assign_fsusage_data(fs, fsusage.blocks,
-				fsusage.bavail, fsusage.bfree, 512);
+	gkrellm_fs_assign_fsusage_data(fs, (glong) fsusage.blocks,
+				(glong) fsusage.bavail, (glong) fsusage.bfree, 512);
 	}
 
 gboolean
