@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -486,6 +486,7 @@ check_media_type(kstat_t *ksp) {
 	   di_devfs_path_free(phys_path);
 	   return 1;
         }
+	return 0;
     }
     return -1; /* shouldn't be reached */
 } 
@@ -956,8 +957,8 @@ gkrellm_sys_fs_get_fsusage(gpointer fs, gchar *dir){
 
 	if (dir && statvfs(dir, &st) == 0) {
 		gkrellm_fs_assign_fsusage_data(fs,
-					(gulong) st.f_blocks, (gulong) st.f_bavail,
-					(gulong) st.f_bfree, (gulong) st.f_bsize);
+					(glong) st.f_blocks, (glong) st.f_bavail,
+					(glong) st.f_bfree, (glong) st.f_bsize);
     } else {
 		gkrellm_fs_assign_fsusage_data(fs, 0, 0, 0, 0);
     } 
