@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -304,7 +304,7 @@ static GkrellmMonitor	monitor_uptime =
 	};
 
 GkrellmMonitor *
-init_uptime_monitor(void)
+gkrellm_init_uptime_monitor(void)
 	{
 	monitor_uptime.name = _(monitor_uptime.name);
 	style_id = gkrellm_add_meter_style(&monitor_uptime, UPTIME_STYLE_NAME);
