@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -45,14 +45,6 @@ static GdkGC	*trans_gc;
 
 static gchar 	*client_id;
 
-void
-gkrellm_winop_text_extents(GdkFont *font, gchar *text, gint text_length,
-		gint *lbearing, gint *rbearing,
-		gint *width, gint *ascent, gint *descent)
-	{
-	gdk_text_extents(font, text, text_length, lbearing, rbearing,
-			width, ascent, descent);
-	}
 
 void
 gkrellm_winop_reset(void)
@@ -96,8 +88,10 @@ cb_ice_connection_messages(IceConn ice_connection, gint source,
 static void
 smc_connect(gint argc, gchar **argv)
 	{
-	SmProp			userid, program, restart, clone, pid, *props[5];
-	SmPropValue		userid_val, pid_val;
+	SmProp			userid, program, restart, restart_style, clone, pid,
+					*props[6];
+	SmPropValue		userid_val, pid_val, restart_style_val;
+	CARD8			restartstyle;
 	SmcCallbacks	*callbacks;
 	SmcConn			smc_connection;
 	IceConn			ice_connection;
@@ -169,6 +163,14 @@ smc_connect(gint argc, gchar **argv)
 	restart.vals[j++].length = strlen(client_id);
 	restart.num_vals = j;
 
+	restartstyle = SmRestartImmediately;
+	restart_style.name = SmRestartStyleHint;
+	restart_style.type = SmCARD8;
+	restart_style.num_vals = 1;
+	restart_style.vals = &restart_style_val;
+	restart_style_val.value = (SmPointer) &restartstyle;
+	restart_style_val.length = 1;
+
 	clone.name = SmCloneCommand;
 	clone.type = SmLISTofARRAY8;
 	clone.vals = restart.vals;
@@ -184,8 +186,14 @@ smc_connect(gint argc, gchar **argv)
 	props[2] = &restart;
 	props[3] = &clone;
 	props[4] = &pid;
-
+#if 0
+	/* Make this an option? */
+	props[5] = &restart_style;
+	SmcSetProperties(smc_connection, 6, props);
+#else
 	SmcSetProperties(smc_connection, 5, props);
+#endif
+
 	g_free(restart.vals);
 
 	ice_connection = SmcGetIceConnection(smc_connection);
@@ -270,6 +278,76 @@ _gkrellm_get_lock(void)
 	return TRUE;
 	}
 
+enum
+	{
+	STRUT_LEFT = 0,
+	STRUT_RIGHT = 1,
+	STRUT_TOP = 2,
+	STRUT_BOTTOM = 3,
+	STRUT_LEFT_START = 4,
+	STRUT_LEFT_END = 5,
+	STRUT_RIGHT_START = 6,
+	STRUT_RIGHT_END = 7,
+	STRUT_TOP_START = 8,
+	STRUT_TOP_END = 9,
+	STRUT_BOTTOM_START = 10,
+	STRUT_BOTTOM_END = 11
+	};
+
+static Atom net_wm_strut_partial = None;
+static Atom net_wm_strut = None;
+
+void
+gkrellm_winop_update_struts(void)
+	{
+	gulong	struts[12] = { 0, };
+	Display	*display;
+	Window	window;
+	gint	width; 
+	gint	height;
+
+	if (!_GK.is_dock_type)
+		return;
+
+	display = GDK_WINDOW_XDISPLAY(gkrellm_get_top_window()->window);
+	window  = GDK_WINDOW_XWINDOW(gkrellm_get_top_window()->window);
+
+	if (net_wm_strut_partial == None)
+		{
+		net_wm_strut_partial
+				= XInternAtom(display, "_NET_WM_STRUT_PARTIAL", False);
+		}
+	if (net_wm_strut == None)
+		{
+		net_wm_strut = XInternAtom(display, "_NET_WM_STRUT", False);
+		}
+
+	width = _GK.chart_width + _GK.frame_left_width + _GK.frame_right_width;
+	height = _GK.monitor_height +  _GK.total_frame_height;
+
+	if (_GK.x_position == 0)
+		{
+		struts[STRUT_LEFT] = width;
+		struts[STRUT_LEFT_START] = _GK.y_position;
+		struts[STRUT_LEFT_END] = _GK.y_position + height;
+		}
+	else if (_GK.x_position == _GK.w_display - width)
+		{
+		struts[STRUT_RIGHT] = width;
+		struts[STRUT_RIGHT_START] = _GK.y_position;
+		struts[STRUT_RIGHT_END] = _GK.y_position + height;
+		}
+
+	gdk_error_trap_push();
+	XChangeProperty (display, window, net_wm_strut,
+					XA_CARDINAL, 32, PropModeReplace,
+					(guchar *) &struts, 4);
+	XChangeProperty (display, window, net_wm_strut_partial,
+					XA_CARDINAL, 32, PropModeReplace,
+					(guchar *) &struts, 12);
+	gdk_error_trap_pop();
+	}
+
 void
 gkrellm_winop_options(gint argc, gchar **argv)
 	{
@@ -332,23 +410,32 @@ gkrellm_winop_options(gint argc, gchar **argv)
 		XChangeProperty(display, window,
 					XInternAtom(display, "_NET_WM_STATE", False),
 					XA_ATOM, 32, PropModeReplace, (guchar *) atoms, n);
-
-	if (_GK.withdrawn)
-		{
-		if (!_GK.is_dock_type)
-			{
-			XWMHints mywmhints; 
-			mywmhints.initial_state = WithdrawnState; 
-			mywmhints.flags=StateHint;
-
-			XSetWMHints(display, window, &mywmhints); 
-			}
-		else
-			gkrellm_message_dialog(NULL,
-			_("Warning: -w flag is ignored when the window dock type is set"));
-		}
 	} 
 
+void
+gkrellm_winop_withdrawn(void)
+	{
+	Display		*display;
+	Window		window;
+
+	if (!_GK.withdrawn)
+		return;
+	display = GDK_WINDOW_XDISPLAY(gkrellm_get_top_window()->window);
+	window  = GDK_WINDOW_XWINDOW(gkrellm_get_top_window()->window);
+
+	if (!_GK.is_dock_type)
+		{
+		XWMHints mywmhints; 
+		mywmhints.initial_state = WithdrawnState; 
+		mywmhints.flags=StateHint;
+
+		XSetWMHints(display, window, &mywmhints); 
+		}
+	else
+		gkrellm_message_dialog(NULL,
+			_("Warning: -w flag is ignored when the window dock type is set"));
+	}
+
   /* Use XParseGeometry, but width and height are ignored.
   |  If GKrellM is moved, update _GK.y_position.
   */
@@ -369,6 +456,10 @@ gkrellm_winop_place_gkrellm(gchar *geom)
 		x = _GK.w_display - w_gkrell + x;
 	gdk_window_move(gkrellm_get_top_window()->window, x, y);
 	_GK.y_position = y;
+	_GK.x_position = x;
+	_GK.position_valid = TRUE;
+	if (_GK.debug_level & DEBUG_POSITION)
+		printf("geometry moveto %d %d\n", x, y);
 	}
 
 void
@@ -481,6 +572,7 @@ gkrellm_winop_draw_rootpixmap_onto_transparent_chart(GkrellmChart *cp)
 					cp->bottom_spacer.pixmap, NULL);
 		}
 	gdk_gc_set_clip_mask(_GK.text_GC, NULL);
+	cp->bg_sequence_id += 1;
 	return TRUE;
 	}	
 
